@@ -1,13 +1,13 @@
 /** 
    EOAccessFault.m <title>EOAccessFault Class</title>
 
-   Copyright (C) 2000 Free Software Foundation, Inc.
+   Copyright (C) 2000,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: June 2000
 
-   $Revision: 1.12 $
-   $Date: 2004/03/19 16:41:18 $
+   $Revision: 21645 $
+   $Date: 2005-08-17 10:07:57 +0200 (Mit, 17 Aug 2005) $
 
    <abstract></abstract>
 
@@ -27,17 +27,19 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
 #include "config.h"
 
-RCS_ID("$Id: EOAccessFault.m,v 1.12 2004/03/19 16:41:18 ayers Exp $")
+RCS_ID("$Id: EOAccessFault.m 21645 2005-08-17 08:07:57Z ayers $")
 
 #ifdef GNUSTEP
 #include <Foundation/NSObjCRuntime.h>
 #include <Foundation/NSException.h>
+#include <Foundation/NSThread.h>
+#include <Foundation/NSInvocation.h>
 #include <Foundation/NSDebug.h>
 #else
 #include <Foundation/Foundation.h>
@@ -53,9 +55,10 @@ RCS_ID("$Id: EOAccessFault.m,v 1.12 2004/03/19 16:41:18 ayers Exp $")
 #include <EOControl/EODebug.h>
 
 #include <EOAccess/EOAccessFault.h>
-#include <EOAccess/EOAccessFaultPriv.h>
 #include <EOAccess/EODatabaseContext.h>
-#include <EOAccess/EODatabaseContextPriv.h>
+
+#include "EOAccessFaultPriv.h"
+#include "EODatabaseContextPriv.h"
 
 
 NSString *EOAccessFaultObjectNotAvailableException = @"EOAccessFaultObjectNotAvailableException";
@@ -143,6 +146,8 @@ NSString *EOAccessFaultObjectNotAvailableException = @"EOAccessFaultObjectNotAva
 {
   if ((self = [super init]))
     {
+      NSDebugFLog(@"INIT EOAccessFaultHandler %p. ThreadID=%@",
+		  (void*)self,GSCurrentThread());
     }
 
   return self;
@@ -179,8 +184,8 @@ NSString *EOAccessFaultObjectNotAvailableException = @"EOAccessFaultObjectNotAva
 - (void)dealloc
 {
 #ifdef DEBUG
-//  NSDebugFLog(@"Dealloc EOAccessFaultHandler %p. ThreadID=%p",
-//              (void*)self,(void*)objc_thread_id());
+  NSDebugFLog(@"Dealloc EOAccessFaultHandler %p. ThreadID=%@",
+              (void*)self, GSCurrentThread());
 #endif
 
   DESTROY(gid);
@@ -189,10 +194,6 @@ NSString *EOAccessFaultObjectNotAvailableException = @"EOAccessFaultObjectNotAva
 
   [super dealloc];
 
-#ifdef DEBUG
-//  NSDebugFLog(@"Dealloc EOAccessFaultHandler %p. ThreadID=%p",
-//              (void*)self,(void*)objc_thread_id());
-#endif
 }
 
 - (EOKeyGlobalID *)globalID
@@ -219,6 +220,10 @@ NSString *EOAccessFaultObjectNotAvailableException = @"EOAccessFaultObjectNotAva
 {
   EOFLOGObjectFnStart();
 
+  // We want to be sure that we will not be autoreleased 
+  // in an autorelease pool of another thread!
+  AUTORELEASE(RETAIN(self)); 
+
   [databaseContext _fireFault: anObject];
 
 //MIRKO: replaced
@@ -235,12 +240,14 @@ NSString *EOAccessFaultObjectNotAvailableException = @"EOAccessFaultObjectNotAva
       [self unableToFaultObject: anObject
             databaseContext: databaseContext];
     }
-
   EOFLOGObjectFnStop();
 }
 
 - (BOOL)shouldPerformInvocation: (NSInvocation *)invocation
 {
+  NSDebugFLLog(@"gsdb",@"invocation selector=%@ target: %p",
+               NSStringFromSelector([invocation selector]),
+               [invocation target]);
   return YES;
 }
 
@@ -316,8 +323,8 @@ NSString *EOAccessFaultObjectNotAvailableException = @"EOAccessFaultObjectNotAva
 - (void)dealloc
 {
 #ifdef DEBUG
-//  NSDebugFLog(@"Dealloc EOAccessArrayFaultHandler %p. ThreadID=%p",
-//              (void*)self,(void*)objc_thread_id());
+  NSDebugFLog(@"Dealloc EOAccessArrayFaultHandler %p. ThreadID=%@",
+              (void*)self,GSCurrentThread());
 #endif
 
   DESTROY(sgid);
@@ -326,10 +333,6 @@ NSString *EOAccessFaultObjectNotAvailableException = @"EOAccessFaultObjectNotAva
   DESTROY(editingContext);
 
   [super dealloc];
-#ifdef DEBUG
-//  NSDebugFLog(@"Stop Dealloc EOAccessArrayFaultHandler %p. ThreadID=%p",
-//              (void*)self,(void*)objc_thread_id());
-#endif
 }
 
 - (EOKeyGlobalID *)sourceGlobalID
@@ -356,6 +359,10 @@ NSString *EOAccessFaultObjectNotAvailableException = @"EOAccessFaultObjectNotAva
 {
   EOFLOGObjectFnStart();
 
+  // We want to be sure that we will not be autoreleased 
+  // in an autorelease pool of another thread!
+  AUTORELEASE(RETAIN(self)); 
+
   [databaseContext _fireArrayFault: anObject];
   [(EOCheapCopyMutableArray *)anObject _setCopy: NO];
 
@@ -379,6 +386,9 @@ NSString *EOAccessFaultObjectNotAvailableException = @"EOAccessFaultObjectNotAva
 
 - (BOOL)shouldPerformInvocation: (NSInvocation *)invocation
 {
+  NSDebugFLLog(@"gsdb",@"invocation selector=%@ target: %p",
+               NSStringFromSelector([invocation selector]),
+               [invocation target]);
   return YES;
 }
 
