@@ -1,7 +1,7 @@
 /* -*-objc-*-
    EOAdaptor.h
 
-   Copyright (C) 2000 Free Software Foundation, Inc.
+   Copyright (C) 2000,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: February 2000
@@ -21,7 +21,7 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 #ifndef __EOAdaptor_h__
@@ -75,8 +75,8 @@ GDL2ACCESS_EXPORT NSString *EOGeneralAdaptorException;
 }
 
 /* Creating an EOAdaptor */
-+ (EOAdaptor *)adaptorWithModel: (EOModel *)model;
-+ (EOAdaptor *)adaptorWithName: (NSString *)name;
++ (id)adaptorWithModel: (EOModel *)model;
++ (id)adaptorWithName: (NSString *)name;
 
 + (void)setExpressionClassName: (NSString *)sqlExpressionClassName
               adaptorClassName: (NSString *)adaptorClassName;
