@@ -1,7 +1,7 @@
 /** 
    EOAdaptor.m <title>EOAdaptor Class</title>
 
-   Copyright (C) 1996-2001 Free Software Foundation, Inc.
+   Copyright (C) 1996-2001,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Ovidiu Predescu <ovidiu@bx.logicnet.ro>
    Date: October 1996
@@ -9,8 +9,8 @@
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: February 2000
 
-   $Revision: 1.21 $
-   $Date: 2004/03/19 16:41:18 $
+   $Revision: 21645 $
+   $Date: 2005-08-17 10:07:57 +0200 (Mit, 17 Aug 2005) $
 
    <abstract></abstract>
 
@@ -30,13 +30,13 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
 #include "config.h"
 
-RCS_ID("$Id: EOAdaptor.m,v 1.21 2004/03/19 16:41:18 ayers Exp $")
+RCS_ID("$Id: EOAdaptor.m 21645 2005-08-17 08:07:57Z ayers $")
 
 #if HAVE_LIBC_H
 # include <libc.h>
@@ -58,6 +58,7 @@ RCS_ID("$Id: EOAdaptor.m,v 1.21 2004/03/19 16:41:18 ayers Exp $")
 #include <Foundation/NSException.h>
 #include <Foundation/NSFileManager.h>
 #include <Foundation/NSData.h>
+#include <Foundation/NSSet.h>
 #include <Foundation/NSDebug.h>
 #else
 #include <Foundation/Foundation.h>
@@ -75,7 +76,6 @@ RCS_ID("$Id: EOAdaptor.m,v 1.21 2004/03/19 16:41:18 ayers Exp $")
 #include <EOControl/EODebug.h>
 
 #include <EOAccess/EOAdaptor.h>
-#include <EOAccess/EOAdaptorPriv.h>
 #include <EOAccess/EOAdaptorContext.h>
 #include <EOAccess/EOAdaptorChannel.h>
 #include <EOAccess/EOAttribute.h>
@@ -83,6 +83,8 @@ RCS_ID("$Id: EOAdaptor.m,v 1.21 2004/03/19 16:41:18 ayers Exp $")
 #include <EOAccess/EOModel.h>
 #include <EOAccess/EOSQLExpression.h>
 
+#include "EOAdaptorPriv.h"
+
 
 NSString *EOGeneralAdaptorException = @"EOGeneralAdaptorException";
 
@@ -149,7 +151,7 @@ NSString *EOAdministrativeConnectionDictionaryKey
   NSArray *paths;
   Class adaptorClass;
   NSString *adaptorClassName;
-  int i, count;
+  unsigned i, count;
 
   /* Check error */
   if ([adaptorName length] == 0)
@@ -301,6 +303,13 @@ NSString *EOAdministrativeConnectionDictionaryKey
   return panel;
 }
 
+/**
+ * Returns an array of EOAdaptor frameworks found in the standard
+ * framework locations.  If an adaptor is found in multiple locations
+ * the name is listed only once.  An adaptor framework is recognized
+ * the the "EOAdaptor.framework" suffix.  The framework name without
+ * this suffix is the name returned in the array.
+ */
 + (NSArray *)availableAdaptorNames
 {
   NSArray	 *pathArray = NSStandardLibraryPaths();
@@ -310,34 +319,33 @@ NSString *EOAdministrativeConnectionDictionaryKey
   NSArray	 *fileNames;
   NSEnumerator	 *filesEnum;
   NSString	 *fileName;
-  NSMutableArray *adaptorNames = AUTORELEASE([NSMutableArray new]);
+  NSMutableSet   *adaptorNames = [NSMutableSet set];
+  NSString       *adaptorSuffix = @"EOAdaptor.framework";
   
   EOFLOGObjectFnStartOrCond2(@"AdaptorLevel", @"EOAdaptor");
 
   while ((searchPath = [pathEnum nextObject]))
     {
-      fileNames = [defaultManager
-		    directoryContentsAtPath:
-		      [searchPath stringByAppendingPathComponent:@"Frameworks"]];
+      searchPath = [searchPath stringByAppendingPathComponent: @"Frameworks"];
+      fileNames = [defaultManager directoryContentsAtPath: searchPath];
       filesEnum = [fileNames objectEnumerator];
     
-      //NSLog(@"EOAdaptor : availableAdaptorNames, path = %@", searchPath);
+      NSDebugMLLog(@"EOAdaptor", @"path = %@", searchPath);
     
       while ((fileName = [filesEnum nextObject]))
 	{
-	  //NSLog(@"EOAdaptor : availableAdaptorNames, fileName = %@", fileName);
-	  if ([fileName hasSuffix:@"EOAdaptor.framework"]) {
-	    [adaptorNames addObject:
-			    [fileName substringToIndex: 
-					([fileName length]
-					 - [@"EOAdaptor.framework" length])]];
-	  }
+	  NSDebugMLLog(@"EOAdaptor", @"fileName = %@", fileName);
+	  if ([fileName hasSuffix: adaptorSuffix])
+	    {
+	      fileName = [fileName stringByDeletingSuffix: adaptorSuffix];
+	      [adaptorNames addObject: fileName];
+	    }
 	}
     }
 
   EOFLOGObjectFnStopOrCond2(@"AdaptorLevel", @"EOAdaptor");
   
-  return adaptorNames;
+  return [adaptorNames allObjects];
 }
 
 - (void)_performAdministativeStatementsForSelector: (SEL)sel
@@ -363,7 +371,7 @@ NSString *EOAdministrativeConnectionDictionaryKey
       EOAdaptorContext *admContext;
       EOAdaptorChannel *admChannel;
       NSArray          *stmts;
-      int i;
+      unsigned i;
 
       stmts = [(id)[self expressionClass] performSelector: sel
 					  withObject: connDict
@@ -377,8 +385,10 @@ NSString *EOAdministrativeConnectionDictionaryKey
       admChannel = [admContext createAdaptorChannel];
       NS_DURING
 	{
+          unsigned stmtsCount=0;
 	  [admChannel openChannel];
-	  for (i = 0; i < [stmts count]; i++)
+          stmtsCount=[stmts count];
+	  for (i = 0; i < stmtsCount; i++)
 	    {
 	      [admChannel evaluateExpression: [stmts objectAtIndex: i]];
 	    }
@@ -417,7 +427,7 @@ NSString *EOAdministrativeConnectionDictionaryKey
   if (model)
     {
       NSArray *entities;
-      int i, count;
+      unsigned i, count;
 
       attributes = [NSMutableArray arrayWithCapacity: 20];
 
@@ -493,12 +503,13 @@ NSString *EOAdministrativeConnectionDictionaryKey
 
 - (BOOL)hasOpenChannels
 {
-  int i;
+  unsigned i;
 
-  for (i = [_contexts count] - 1; i >= 0; i--)
+  i = [_contexts count];
+  while (i--)
     {
-      EOAdaptorContext *ctx = [[_contexts objectAtIndex: i]
-				nonretainedObjectValue];
+      EOAdaptorContext *ctx 
+	= [[_contexts objectAtIndex: i] nonretainedObjectValue];
 
       if([ctx hasOpenChannels] == YES)
 	return YES;
@@ -574,7 +585,7 @@ NSString *EOAdministrativeConnectionDictionaryKey
   NSString	   *encodingString=nil;
   NSDictionary	   *encodingsDict = [self connectionDictionary];
   const NSStringEncoding *availableEncodingsArray;
-  int		    count = 0;
+  unsigned	    count = 0;
   NSStringEncoding  availableEncodingValue;
   NSString	   *availableEncodingString;
   
@@ -668,7 +679,7 @@ NSString *EOAdministrativeConnectionDictionaryKey
       //TODO-NOW: correct this code which loop!
       /*
       const char *cstr=NULL;
-      int i=0, spc=0;
+      unsigned i=0, spc=0;
       cstr = [value cString];
       while(*cstr)
         {
@@ -718,7 +729,7 @@ NSString *EOAdministrativeConnectionDictionaryKey
 - (void)handleDroppedConnection
 {
   NSDictionary *newConnectionDictionary = nil;
-  int i;
+  unsigned i;
   
   for (i = [_contexts count] - 1; i >= 0; i--)
     {
@@ -920,8 +931,9 @@ NSString *EOAdministrativeConnectionDictionaryKey
  */
 + (void)assignExternalInfoForEntity: (EOEntity *)entity
 {
-  NSArray  *attributes;
-  unsigned i;
+  NSArray  *attributes=nil;
+  unsigned i=0;
+  unsigned attributesCount=0;
 
   if ([[entity externalName] length] == 0)
     {
@@ -933,8 +945,9 @@ NSString *EOAdministrativeConnectionDictionaryKey
     }
 
   attributes = [entity attributes];
+  attributesCount=[attributes count];
 
-  for (i = 0; i < [attributes count]; i++)
+  for (i = 0; i < attributesCount; i++)
     {
       [self assignExternalInfoForAttribute: [attributes objectAtIndex: i]];
     }
@@ -948,12 +961,14 @@ NSString *EOAdministrativeConnectionDictionaryKey
  */
 + (void)assignExternalInfoForEntireModel: (EOModel *)model
 {
-  NSArray  *entities;
-  unsigned i;
+  NSArray  *entities=nil;
+  unsigned i=0;
+  unsigned entitiesCount=0;
 
   entities = [model entities];
+  entitiesCount=[entities count];
 
-  for (i = 0; i < [entities count]; i++)
+  for (i = 0; i < entitiesCount; i++)
     {
       [self assignExternalInfoForEntity: [entities objectAtIndex: i]];
     }
@@ -971,7 +986,7 @@ NSString *EOAdministrativeConnectionDictionaryKey
 
 - (void) _unregisterAdaptorContext: (EOAdaptorContext*)adaptorContext
 {
-  int i = 0;    
+  unsigned i = 0;    
 
   for (i = [_contexts count] - 1; i >= 0; i--)
     {
