@@ -1,7 +1,7 @@
 /* -*-objc-*-
    EOAdaptorContext.h
 
-   Copyright (C) 2000 Free Software Foundation, Inc.
+   Copyright (C) 2000,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: February 2000
@@ -21,7 +21,7 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 #ifndef __EOAdaptorContext_h__
@@ -51,7 +51,7 @@ typedef enum {
 
 /* The EOAdaptorContext class could be overriden for a concrete database
    adaptor. You have to override only those methods marked in this header
-   with `override'.
+   with 'override'.
 */
 
 @interface EOAdaptorContext : NSObject
@@ -83,6 +83,7 @@ typedef enum {
 
 - (EOAdaptorChannel *)createAdaptorChannel;	// override
 
+- (NSArray *)channels;
 - (BOOL)hasOpenChannels;
 - (BOOL)hasBusyChannels;
 
