@@ -1,13 +1,13 @@
 /** 
    EOAdaptorContext.m <title>EOAdaptorContext Class</title>
 
-   Copyright (C) 2000 Free Software Foundation, Inc.
+   Copyright (C) 2000,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: February 2000
 
-   $Revision: 1.9 $
-   $Date: 2004/03/19 16:41:18 $
+   $Revision: 21645 $
+   $Date: 2005-08-17 10:07:57 +0200 (Mit, 17 Aug 2005) $
 
    <abstract></abstract>
 
@@ -27,13 +27,13 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
 #include "config.h"
 
-RCS_ID("$Id: EOAdaptorContext.m,v 1.9 2004/03/19 16:41:18 ayers Exp $")
+RCS_ID("$Id: EOAdaptorContext.m 21645 2005-08-17 08:07:57Z ayers $")
 
 #ifdef GNUSTEP
 #include <Foundation/NSArray.h>
@@ -52,12 +52,14 @@ RCS_ID("$Id: EOAdaptorContext.m,v 1.9 2004/03/19 16:41:18 ayers Exp $")
 #endif
 
 #include <EOControl/EODebug.h>
+#include <EOControl/EONSAddOns.h>
 
 #include <EOAccess/EOAdaptor.h>
-#include <EOAccess/EOAdaptorPriv.h>
 #include <EOAccess/EOAdaptorContext.h>
 #include <EOAccess/EOAdaptorChannel.h>
 
+#include "EOAdaptorPriv.h"
+
 
 NSString *EOAdaptorContextBeginTransactionNotification = @"EOAdaptorContextBeginTransactionNotofication";
 NSString *EOAdaptorContextCommitTransactionNotification = @"EOAdaptorContextCommitTransactionNotofication";
@@ -103,6 +105,12 @@ NSString *EOAdaptorContextRollbackTransactionNotification = @"EOAdaptorContextRo
   return nil;
 }
 
+- (NSArray *)channels
+{
+  SEL sel = @selector(nonretainedObjectValue);
+  return [_channels resultsOfPerformingSelector: sel];
+}
+
 - (BOOL)hasOpenChannels
 {
   int i, count = [_channels count];
