@@ -1,7 +1,7 @@
 /* -*-objc-*-
    EOAttribute.h
 
-   Copyright (C) 2000 Free Software Foundation, Inc.
+   Copyright (C) 2000,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: Feb 2000
@@ -21,7 +21,7 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 #ifndef __EOAttribute_h__
@@ -57,7 +57,7 @@ typedef enum {
 } EOFactoryMethodArgumentType;
 
 typedef enum {
-  EOAdaptorNumberType = 0,
+  EOAdaptorNumberType,
   EOAdaptorCharactersType,
   EOAdaptorBytesType,
   EOAdaptorDateType
@@ -84,7 +84,9 @@ typedef enum {
   unsigned int _width;
   unsigned short _precision;
   short _scale;
+  char _valueTypeChar; /** First char of _valueType or \0 **/
   Class _valueClass;
+  EOAdaptorValueType _adaptorValueType;
   EOFactoryMethodArgumentType _argumentType;
   NSString *_valueFactoryMethodName;
   NSString *_adaptorValueConversionMethodName;
@@ -94,11 +96,12 @@ typedef enum {
     unsigned int allowsNull:1;
     unsigned int isReadOnly:1;
     unsigned int isParentAnEOEntity:1;
-    unsigned int protoOverride:17;
-    unsigned int unused : 12;
-    unsigned int extraRefCount;
+    unsigned int protoOverride:18;
+    unsigned int isAttributeValueInitialized:1;
+    unsigned int unused : 10;
   } _flags;
     
+  unsigned int extraRefCount;
   NSDictionary *_sourceToDestinationKeyMap;
   EOParameterDirection _parameterDirection;
   NSDictionary *_userInfo;
@@ -108,7 +111,7 @@ typedef enum {
     /* Garbage collectable objects */
 //    EOEntity *_entity;
 //    GCMutableArray *_definitionArray;	// These variables are meaningful only
-  id _parent;
+  id _parent; /* unretained */
   EOAttribute *_prototype;
   EOExpressionArray *_definitionArray;
   EOAttribute *_realAttribute;		// if the attribute is flattened //Not in EOF !
