@@ -1,7 +1,7 @@
 /* -*-objc-*-
    EODatabase.h
 
-   Copyright (C) 2000 Free Software Foundation, Inc.
+   Copyright (C) 2000,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: Jun 2000
@@ -21,7 +21,7 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 #ifndef __EODatabase_h__
@@ -53,6 +53,8 @@
 
 GDL2ACCESS_EXPORT NSString *EOGeneralDatabaseException;
 
+GDL2ACCESS_EXPORT NSTimeInterval EODistantPastTimeInterval; 
+
 
 @interface EODatabase : NSObject
 {
@@ -99,6 +101,8 @@ GDL2ACCESS_EXPORT NSString *EOGeneralDatabaseException;
 
 - (void)recordSnapshot: (NSDictionary *)snapshot forGlobalID: (EOGlobalID *)gid;
 
+- (NSDictionary *)snapshotForGlobalID: (EOGlobalID *)gid
+                                after: (NSTimeInterval)ti;
 - (NSDictionary *)snapshotForGlobalID: (EOGlobalID *)gid;
 
 - (void)recordSnapshot: (NSArray *)gids
