@@ -1,13 +1,13 @@
 /** 
    EODatabase.m <title>EODatabase Class</title>
 
-   Copyright (C) 2000 Free Software Foundation, Inc.
+   Copyright (C) 2000,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: June 2000
 
-   $Revision: 1.12 $
-   $Date: 2004/04/17 12:41:32 $
+   $Revision: 21645 $
+   $Date: 2005-08-17 10:07:57 +0200 (Mit, 17 Aug 2005) $
 
    <abstract></abstract>
 
@@ -27,13 +27,13 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
 #include "config.h"
 
-RCS_ID("$Id: EODatabase.m,v 1.12 2004/04/17 12:41:32 ayers Exp $")
+RCS_ID("$Id: EODatabase.m 21645 2005-08-17 08:07:57Z ayers $")
 
 #ifdef GNUSTEP
 #include <Foundation/NSArray.h>
@@ -65,6 +65,7 @@ RCS_ID("$Id: EODatabase.m,v 1.12 2004/04/17 12:41:32 ayers Exp $")
 #include <EOAccess/EODatabase.h>
 #include <EOAccess/EODatabaseContext.h>
 
+#include "EOPrivate.h"
 /* TODO
 
    Controllare il resultCache, ad ogni forget/invalidate deve essere
@@ -72,7 +73,7 @@ RCS_ID("$Id: EODatabase.m,v 1.12 2004/04/17 12:41:32 ayers Exp $")
  */
 
 NSString *EOGeneralDatabaseException = @"EOGeneralDatabaseException";
-
+NSTimeInterval EODistantPastTimeInterval = -603979776.0;
 
 @implementation EODatabase
 
@@ -84,8 +85,16 @@ static NSMutableArray *databaseInstances;
 
 + (void)initialize
 {
-  // THREAD
-  databaseInstances = [NSMutableArray new];
+  static BOOL initialized=NO;
+  if (!initialized)
+    {
+      initialized=YES;
+
+      GDL2_EOAccessPrivateInit();
+
+      // THREAD
+      databaseInstances = [NSMutableArray new];
+    }
 }
 
 + (void)makeAllDatabasesPerform: (SEL)aSelector withObject: anObject
@@ -289,7 +298,7 @@ static NSMutableArray *databaseInstances;
 
   EOFLOGObjectLevelArgs(@"EODatabaseContext", @"object=%p (of class %@)",
 			object, [object class]);
-  NSAssert(!isNilOrEONull(object), @"No object");
+  NSAssert(!_isNilOrEONull(object), @"No object");
 
   if ([EOFault isFault: object])
     {
@@ -449,6 +458,13 @@ static NSMutableArray *databaseInstances;
 }
 
 - (NSDictionary *)snapshotForGlobalID: (EOGlobalID *)gid
+{
+  return [self snapshotForGlobalID: gid
+	       after: EODistantPastTimeInterval];
+}
+
+- (NSDictionary *)snapshotForGlobalID: (EOGlobalID *)gid
+				after: (NSTimeInterval)ti
 {
   //seems OK
   NSDictionary *snapshot = nil;
