@@ -1,7 +1,7 @@
 /** 
    EODatabaseChannel.m <title>EODatabaseChannel</title>
 
-   Copyright (C) 2000-2003 Free Software Foundation, Inc.
+   Copyright (C) 2000-2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: June 2000
@@ -9,8 +9,8 @@
    Author: Manuel Guesdon <mguesdon@orange-concept.com>
    Date: October 2000
 
-   $Revision: 1.11 $
-   $Date: 2004/03/19 16:41:18 $
+   $Revision: 21645 $
+   $Date: 2005-08-17 10:07:57 +0200 (Mit, 17 Aug 2005) $
 
    <abstract></abstract>
 
@@ -30,13 +30,13 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
 #include "config.h"
 
-RCS_ID("$Id: EODatabaseChannel.m,v 1.11 2004/03/19 16:41:18 ayers Exp $")
+RCS_ID("$Id: EODatabaseChannel.m 21645 2005-08-17 08:07:57Z ayers $")
 
 #ifdef GNUSTEP
 #include <Foundation/NSString.h>
@@ -65,9 +65,7 @@ RCS_ID("$Id: EODatabaseChannel.m,v 1.11 2004/03/19 16:41:18 ayers Exp $")
 #include <EOControl/EODebug.h>
 
 #include <EOAccess/EODatabaseChannel.h>
-#include <EOAccess/EODatabaseChannelPriv.h>
 #include <EOAccess/EODatabaseContext.h>
-#include <EOAccess/EODatabaseContextPriv.h>
 #include <EOAccess/EODatabase.h>
 
 #include <EOAccess/EOAdaptor.h>
@@ -81,13 +79,20 @@ RCS_ID("$Id: EODatabaseChannel.m,v 1.11 2004/03/19 16:41:18 ayers Exp $")
 #include <EOAccess/EOSQLExpression.h>
 #include <EOAccess/EOSQLQualifier.h>
 
+#include "EOPrivate.h"
+#include "EOEntityPriv.h"
+#include "EODatabaseContextPriv.h"
+#include "EODatabaseChannelPriv.h"
 
 @implementation EODatabaseChannel
 
 + (void)initialize
 {
-  if (self == [EODatabaseChannel class])
+  static BOOL initialized=NO;
+  if (!initialized)
     {
+      initialized=YES;
+      GDL2_EOAccessPrivateInit();
       [[NSNotificationCenter defaultCenter]
         addObserver: self
         selector: @selector(_registerDatabaseChannel:)
@@ -316,7 +321,6 @@ RCS_ID("$Id: EODatabaseChannel.m,v 1.11 2004/03/19 16:41:18 ayers Exp $")
       row = [_adaptorChannel fetchRowWithZone: NULL];
 
       EOFLOGObjectLevelArgs(@"gsdb", @"row=%@", row);
-      //NSDebugMLog(@"TEST attributesToFetch=%@", [_currentEntity attributesToFetch]);
 
       if (!row)
         {
@@ -344,7 +348,6 @@ RCS_ID("$Id: EODatabaseChannel.m,v 1.11 2004/03/19 16:41:18 ayers Exp $")
 				isFinal: YES];//OK
 
           EOFLOGObjectLevelArgs(@"gsdb", @"gid=%@", gid);
-          //NSDebugMLog(@"TEST attributesToFetch=%@",[_currentEntity attributesToFetch]);
 
           object = [_currentEditingContext objectForGlobalID: gid]; //OK //nil
 
@@ -358,7 +361,6 @@ RCS_ID("$Id: EODatabaseChannel.m,v 1.11 2004/03/19 16:41:18 ayers Exp $")
           snapshot = [_databaseContext snapshotForGlobalID: gid]; //OK
 
           EOFLOGObjectLevelArgs(@"gsdb", @"snapshot=%@", snapshot);
-          //NSDebugMLog(@"TEST attributesToFetch=%@", [_currentEntity attributesToFetch]);
 
           if (snapshot)
             {
@@ -390,7 +392,6 @@ RCS_ID("$Id: EODatabaseChannel.m,v 1.11 2004/03/19 16:41:18 ayers Exp $")
             }
           else
             {
-              //NSDebugMLog(@"TEST attributesToFetch=%@", [_currentEntity attributesToFetch]);
               EOFLOGObjectLevelArgs(@"gsdb", @"database class=%@", [database class]);
 
               NSAssert(database, @"No database-context database");
@@ -421,12 +422,11 @@ RCS_ID("$Id: EODatabaseChannel.m,v 1.11 2004/03/19 16:41:18 ayers Exp $")
 					      globalID: gid
 					      zone: NULL];
 
-              //NSDebugMLog(@"TEST attributesToFetch=%@", [_currentEntity attributesToFetch]);
               EOFLOGObjectLevelArgs(@"gsdb", @"object=%@", object);
               NSAssert1(object, @"No Object. entityClassDescripton=%@", entityClassDescripton);
 
-              [_currentEditingContext recordObject: object
-                                      globalID: gid];
+              EOEditingContext_recordObjectGlobalIDWithImpPtr(_currentEditingContext,
+							      NULL,object,gid);
             }
           else if (object && [EOFault isFault: object])
             {
@@ -479,8 +479,6 @@ RCS_ID("$Id: EODatabaseChannel.m,v 1.11 2004/03/19 16:41:18 ayers Exp $")
 
 - (BOOL)isFetchInProgress
 {
-  //NSDebugMLog(@"TEST attributesToFetch=%@", [_currentEntity attributesToFetch]);
-
   return [_adaptorChannel isFetchInProgress];
 }
 
@@ -644,7 +642,7 @@ RCS_ID("$Id: EODatabaseChannel.m,v 1.11 2004/03/19 16:41:18 ayers Exp $")
 
   _hints = [fetch _hints];
 
-  customQueryExpressionHint = [_hints objectForKey: @"EOCustomQueryExpressionHintKey"];//TODO use it
+  customQueryExpressionHint = [_hints objectForKey: EOCustomQueryExpressionHintKey];//TODO use it
 
   if (customQueryExpressionHint)
     {
@@ -721,7 +719,7 @@ RCS_ID("$Id: EODatabaseChannel.m,v 1.11 2004/03/19 16:41:18 ayers Exp $")
   entity = [database entityNamed:entityName];
   primaryKeyAttributes = [entity primaryKeyAttributes];
   hints = [fetch hints]; // ret {} 
-  storedProcedureName = [hints objectForKey: @"EOStoredProcedureNameHintKey"];//TODO use it
+  storedProcedureName = [hints objectForKey: EOStoredProcedureNameHintKey];//TODO use it
   model = [entity model];
   modelGroup = [model modelGroup]; //ret nil
   //TODO if model gr
