@@ -1,7 +1,7 @@
 /* -*-objc-*-
    EODatabaseContext.h
 
-   Copyright (C) 2000 Free Software Foundation, Inc.
+   Copyright (C) 2000,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: July 2000
@@ -21,7 +21,7 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 #ifndef __EODatabaseContext_h__
@@ -69,37 +69,28 @@ struct _EOTransactionScope;
   EODatabase *_database;
   EOAdaptorContext *_adaptorContext;
   EOUpdateStrategy _updateStrategy;
-/*TOADD
-    NSMutableArray *_uniqueStack;
-    NSMutableArray *_deleteStack;
-    NSMutableArray *_modifiedObjects;
-*/
+  NSMutableArray *_uniqueStack; /* simple snapshots */
+  NSMutableArray *_deleteStack; 
+
   NSMutableArray *_registeredChannels;
   NSMapTable *_dbOperationsByGlobalID;
   EOObjectStoreCoordinator *_coordinator;	/* unretained */
   EOEditingContext *_editingContext;		/* unretained */
-  id *_lockedObjects;//void*
+  NSHashTable *_lockedObjects;//void*
 /*TO ADD    unsigned int _currentGeneration;
     unsigned int _concurentFetches;
 */
 
-  unsigned int _numLocked;//TO REMOVE
   NSMutableDictionary *_batchFaultBuffer;
   NSMutableDictionary *_batchToManyFaultBuffer;
 
-//  NSMutableDictionary *_snapshots;
-//  NSMutableDictionary *_toManySnapshots;
-
   EOEntity* _lastEntity;
 /*TOADD  
     EOGlobalID *_currentGlobalID;
     NSDictionary *_currentSnapshot;
     objc_object *_currentBatch;
 */
-  NSMutableArray *_uniqueStack;// snaps
-  NSMutableArray *_uniqueArrayStack;//to many snaps
-  NSMutableArray *_deleteStack;
-
+  NSMutableArray *_uniqueArrayStack; /* to-many snapshots */
   NSHashTable *_nonPrimaryKeyGenerators;
 
   struct {
@@ -258,9 +249,9 @@ It's invoked after prepareForSaveWithCoordinator:editingContext: and before owns
 				  sourceObject: (id)sourceObject
 			     destinationObject: (id)destinationObject;
 
-- (id)databaseOperationForObject: (id)param0;
-- (id)databaseOperationForGlobalID: (id)param0;
-- (void)recordDatabaseOperation: (id)param0;
+- (EODatabaseOperation*)databaseOperationForObject: (id)object;
+- (EODatabaseOperation*)databaseOperationForGlobalID: (EOGlobalID *)gid;
+- (void)recordDatabaseOperation: (EODatabaseOperation*)databaseOpe;
 - (void)recordDeleteForObject: (id)param0;
 - (void)recordInsertForObject: (id)param0;
 
@@ -352,7 +343,7 @@ Only searches locally (in the transaction scope), not in the EODatabase. **/
 - (void)_commitTransaction;
 - (void)_beginTransaction;
 - (EODatabaseChannel *)_obtainOpenChannel;
-- (BOOL)_openChannelWithLoginPanel: (id)param0;
+- (BOOL)_openChannelWithLoginPanel: (EODatabaseChannel *)databaseChannel;
 - (void)_forceDisconnect;
 - (void)initializeObject: (id)object
                      row: (NSDictionary *)row
@@ -438,6 +429,7 @@ shouldRaiseExceptionForLockFailure: (NSException *)exception;
 @end
 
 GDL2ACCESS_EXPORT NSString *EOCustomQueryExpressionHintKey;
+GDL2ACCESS_EXPORT NSString *EOStoredProcedureNameHintKey;
 
 GDL2ACCESS_EXPORT NSString *EODatabaseContextKey;
 GDL2ACCESS_EXPORT NSString *EODatabaseOperationsKey;
