@@ -1,7 +1,7 @@
 /* -*-objc-*-
    EODatabaseContextPriv.h
 
-   Copyright (C) 2000 Free Software Foundation, Inc.
+   Copyright (C) 2000,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: July 2000
@@ -21,7 +21,7 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 #ifndef __EODatabaseContextPriv_h__
@@ -60,10 +60,10 @@
 
 - (void)_verifyNoChangesToReadonlyEntity: (EODatabaseOperation *)dbOpe;
 - (EOGlobalID *)_globalIDForObject: (id)object;
-- (id)_primaryKeyForObject: (id)object;
+- (NSDictionary *)_primaryKeyForObject: (id)object;
 - (NSDictionary *)_primaryKeyForObject: (id)object
 			raiseException: (BOOL)raiseException;
-- (id)_currentCommittedSnapshotForObject: (id)object;
+- (NSDictionary *)_currentCommittedSnapshotForObject: (id)object;
 - (id)_addDatabaseContextStateToException: (id)param0;
 - (id)_databaseContextState;
 - (void)_cleanUpAfterSave;
@@ -73,5 +73,4 @@
 
 @end
 
-
 #endif /* __EODatabaseContextPriv_h__ */
