@@ -1,7 +1,7 @@
 /** 
    EODatabaseDataSource.m <title>EODatabaseDataSource Class</title>
 
-   Copyright (C) 2000 Free Software Foundation, Inc.
+   Copyright (C) 2000,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: July 2000
@@ -9,8 +9,8 @@
    Author: Manuel Guesdon <mguesdon@orange-concept.com>
    Date: December 2001
 
-   $Revision: 1.12 $
-   $Date: 2004/03/19 16:41:18 $
+   $Revision: 21645 $
+   $Date: 2005-08-17 10:07:57 +0200 (Mit, 17 Aug 2005) $
 
    <abstract></abstract>
 
@@ -32,13 +32,13 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
 #include "config.h"
 
-RCS_ID("$Id: EODatabaseDataSource.m,v 1.12 2004/03/19 16:41:18 ayers Exp $")
+RCS_ID("$Id: EODatabaseDataSource.m 21645 2005-08-17 08:07:57Z ayers $")
 
 #ifdef GNUSTEP
 #include <Foundation/NSString.h>
@@ -72,7 +72,7 @@ RCS_ID("$Id: EODatabaseDataSource.m,v 1.12 2004/03/19 16:41:18 ayers Exp $")
 #include <EOAccess/EOModelGroup.h>
 #include <EOAccess/EODatabase.h>
 #include <EOAccess/EODatabaseContext.h>
-
+#include <EOAccess/EOPrivate.h>
 
 @interface EODatabaseDataSource(Private)
 - (id)_partialInitWithEditingContext: (EOEditingContext*)editingContext
@@ -191,42 +191,56 @@ RCS_ID("$Id: EODatabaseDataSource.m,v 1.12 2004/03/19 16:41:18 ayers Exp $")
 
 - (EOEntity *)entity
 {
-  return [[[self databaseContext] database]
-	   entityNamed: [_fetchSpecification entityName]];
+  EOObjectStore *store;
+  NSString *entityName = [_fetchSpecification entityName];
+  static SEL modelGroupSel = @selector(modelGroup);
+  EOModelGroup *modelGroup = nil;
+  
+  store = [_editingContext rootObjectStore];
+  
+  if ([store isKindOfClass: [EOObjectStoreCoordinator class]])
+    {
+      return [[(EOObjectStoreCoordinator *)store modelGroup] entityNamed: entityName];
+    }
+  else if ([store isKindOfClass:GDL2_EODatabaseContextClass])
+    {
+      EODatabase *database = [(EODatabaseContext *)store database];
+      NSArray *models = [database models];
+      int i, c;
+      
+      for (i = 0, c = [models count]; i < c; i++)
+	{
+	  EOEntity *entity;
+	  
+	  modelGroup = [[models objectAtIndex: i] modelGroup];
+	  entity = [modelGroup entityNamed: entityName];
+	  if (entity)
+	    return entity;
+	}
+      return nil;
+    }
+  else if ([store respondsToSelector:modelGroupSel])
+    {
+      modelGroup = [store performSelector:modelGroupSel];
+    }
+  
+  if (modelGroup != nil)
+    {
+      return [modelGroup entityNamed: entityName];
+    }
+  else
+    {
+      return [[EOModelGroup defaultModelGroup] entityNamed: entityName];
+    }
 }
 
 - (EODatabaseContext *)databaseContext
 {
-  NSArray *stores = nil;
-  EODatabaseContext *store = nil;
-  NSEnumerator *storeEnum = nil;
-  NSString *entityName = nil;
-  id rootStore = nil;
+  EOModel *model;
 
-  entityName = [_fetchSpecification entityName];
-
-  rootStore = [_editingContext rootObjectStore];
-  if ([rootStore isKindOfClass: [EOObjectStoreCoordinator class]] == YES)
-    {
-      stores = [rootStore cooperatingObjectStores];
-
-      storeEnum = [stores objectEnumerator];
-      while ((store = [storeEnum nextObject]))
-	{
-	  if ([store isKindOfClass: [EODatabaseContext class]] == YES)
-	    {
-	      if ([[store database] entityNamed: entityName])
-		break;
-	    }
-	}
-    }
-  else if ([rootStore isKindOfClass: [EODatabaseContext class]] == YES)
-    {
-      if ([[store database] entityNamed: entityName])
-	store = rootStore;
-    }
-
-  return store;
+  model = [[self entity] model];
+  return [EODatabaseContext registeredDatabaseContextForModel:model
+				editingContext:_editingContext];
 }
 
 - (void)setFetchSpecification: (EOFetchSpecification *)fetchSpecification
@@ -301,12 +315,17 @@ RCS_ID("$Id: EODatabaseDataSource.m,v 1.12 2004/03/19 16:41:18 ayers Exp $")
                              detailKey: detailKey];
 }
 
-- (void)insertObject: object
+/**
+ * Overrides superclasses implementation but doesn't do anything
+ * useful.  You must insert the object into the editing context
+ * manually.
+ */
+- (void)insertObject: (id)object
 {
-  [_editingContext insertObject: object];
+
 }
 
-- (void)deleteObject: object
+- (void)deleteObject: (id)object
 {
   [_editingContext deleteObject: object];
 }
