@@ -1,13 +1,13 @@
 /** 
    EOAdaptorOperation.m <title>EOAdaptorOperation Class</title>
 
-   Copyright (C) 2000-2002 Free Software Foundation, Inc.
+   Copyright (C) 2000-2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: February 2000
 
-   $Revision: 1.12 $
-   $Date: 2004/03/19 16:41:19 $
+   $Revision: 21645 $
+   $Date: 2005-08-17 10:07:57 +0200 (Mit, 17 Aug 2005) $
 
    <abstract></abstract>
 
@@ -27,13 +27,13 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
 #include "config.h"
 
-RCS_ID("$Id: EODatabaseOperation.m,v 1.12 2004/03/19 16:41:19 ayers Exp $")
+RCS_ID("$Id: EODatabaseOperation.m 21645 2005-08-17 08:07:57Z ayers $")
 
 #ifdef GNUSTEP
 #include <Foundation/NSString.h>
@@ -58,6 +58,8 @@ RCS_ID("$Id: EODatabaseOperation.m,v 1.12 2004/03/19 16:41:19 ayers Exp $")
 #include <EOAccess/EOAttribute.h>
 #include <EOAccess/EOEntity.h>
 
+#include "EOEntityPriv.h"
+
 
 @implementation EODatabaseOperation
 
