@@ -1,7 +1,7 @@
 /** 
    EOExpressionArray.m <title>EOExpressionArray</title>
 
-   Copyright (C) 1996-2002 Free Software Foundation, Inc.
+   Copyright (C) 1996-2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Ovidiu Predescu <ovidiu@bx.logicnet.ro>
    Date: September 1996
@@ -9,8 +9,8 @@
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: February 2000
 
-   $Revision: 1.13 $
-   $Date: 2004/04/17 12:41:32 $
+   $Revision: 21645 $
+   $Date: 2005-08-17 10:07:57 +0200 (Mit, 17 Aug 2005) $
 
    <abstract></abstract>
 
@@ -30,13 +30,13 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
 #include "config.h"
 
-RCS_ID("$Id: EOExpressionArray.m,v 1.13 2004/04/17 12:41:32 ayers Exp $")
+RCS_ID("$Id: EOExpressionArray.m 21645 2005-08-17 08:07:57Z ayers $")
 
 #include <ctype.h>
 
