@@ -1,7 +1,7 @@
 /* -*-objc-*-
    EOModel.h
 
-   Copyright (C) 2000-2002 Free Software Foundation, Inc.
+   Copyright (C) 2000-2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: February 2000
@@ -21,7 +21,7 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 #ifndef __EOModel_h__
@@ -41,6 +41,7 @@
 
 @class NSArray;
 @class NSDictionary;
+@class NSNotification;
 
 @class EOEntity;
 @class EOAttribute;
@@ -137,16 +138,16 @@
 
 - (void)_resetPrototypeCache;
 - (BOOL)isPrototypesEntity: (id)param0;
-- (void)_classDescriptionNeeded: (id)param0;
+- (void)_classDescriptionNeeded: (NSNotification *)notification;
 - (id)_instantiatedEntities;
 - (void)_setPath: (NSString *)path;
-- (id)_entityForClass: (Class)param0;
+- (EOEntity *)_entityForClass: (Class)param0;
 - (id)_childrenForEntityNamed: (id)param0;
 - (void)_registerChild: (id)param0
              forParent: (id)param1;
 - (void)_setInheritanceLinks: (id)param0;
 - (void)_removeEntity: (id)param0;
-- (id)_addEntityWithPropertyList: (NSDictionary *)propertyList;
+- (EOEntity *)_addEntityWithPropertyList: (NSDictionary *)propertyList;
 - (void)_addFakeEntityWithPropertyList: (NSDictionary *)propertyList;
 - (id)_addEntity: (EOEntity *)entity;
 - (void)_setEntity: (id)entity
