@@ -1,14 +1,14 @@
 /** 
    EOModel.m <title>EOModel Class</title>
 
-   Copyright (C) 2000-2002 Free Software Foundation, Inc.
+   Copyright (C) 2000-2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
            Manuel Guesdon <mguesdon@orange-concept.com>
    Date: February 2000
 
-   $Revision: 1.19 $
-   $Date: 2004/04/16 13:10:43 $
+   $Revision: 22681 $
+   $Date: 2006-03-19 19:03:50 +0100 (Son, 19 Mär 2006) $
 
    <abstract></abstract>
 
@@ -28,13 +28,13 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
 #include "config.h"
 
-RCS_ID("$Id: EOModel.m,v 1.19 2004/04/16 13:10:43 ayers Exp $")
+RCS_ID("$Id: EOModel.m 22681 2006-03-19 18:03:50Z ratmice $")
 
 #ifdef GNUSTEP
 #include <Foundation/NSBundle.h>
@@ -63,17 +63,22 @@ RCS_ID("$Id: EOModel.m,v 1.19 2004/04/16 13:10:43 ayers Exp $")
 #include <EOControl/EOKeyGlobalID.h>
 #include <EOControl/EOClassDescription.h>
 #include <EOControl/EOObserver.h>
+#include <EOControl/EOKeyValueCoding.h>
 #include <EOControl/EONSAddOns.h>
 #include <EOControl/EODebug.h>
 
 #include <EOAccess/EOModel.h>
 #include <EOAccess/EOEntity.h>
-#include <EOAccess/EOEntityPriv.h>
 #include <EOAccess/EOStoredProcedure.h>
 #include <EOAccess/EOModelGroup.h>
 #include <EOAccess/EOAccessFault.h>
 #include <EOAccess/EOAdaptor.h>
 #include <EOAccess/EOAttribute.h>
+#include <EOAccess/EORelationship.h>
+
+#include "EOEntityPriv.h"
+#include "EOPrivate.h"
+#include "EOAttributePriv.h"
 
 #define DEFAULT_MODEL_VERSION 2
 
@@ -196,7 +201,7 @@ NSString *EOEntityLoadedNotification = @"EOEntityLoadedNotification";
    return modelPath;
 }
 
-- (id) init
+- (id)init
 {
   EOFLOGObjectFnStart();
 
@@ -211,6 +216,7 @@ NSString *EOEntityLoadedNotification = @"EOEntityLoadedNotification";
 						  NSObjectMapValueCallBacks,
 						  8,
 						  [self zone]);
+      _storedProcedures = [GCMutableArray new];
   
       [[NSNotificationCenter defaultCenter]
         addObserver: self
@@ -325,7 +331,7 @@ NSString *EOEntityLoadedNotification = @"EOEntityLoadedNotification";
 {
   EOEntity *entity = nil;
 
-  NSAssert(name,@"No entityt name");
+  NSAssert(name,@"No entity name");
 
   entity = [_entitiesByName objectForKey: name];
   if (entity != nil)
@@ -358,59 +364,37 @@ NSString *EOEntityLoadedNotification = @"EOEntityLoadedNotification";
 	   sortedArrayUsingSelector: @selector(compare:)];
 }
 
-- (NSArray*) storedProcedureNames
+- (NSArray *)storedProcedureNames
 {
-
-  NSEnumerator *stEnum;
-  EOStoredProcedure *st;
-  NSMutableArray *stNames = [NSMutableArray arrayWithCapacity:
-					      [_storedProcedures count]];
-
-  stEnum = [_storedProcedures objectEnumerator];
-  while ((st = [stEnum nextObject]))
-    [stNames addObject: st];
-
-  return stNames;
+  /* We do not load procedures lazy 
+     so we can rely on the instance variable.  */
+  return [_storedProcedures valueForKey: @"name"];
 }
 
-- (EOStoredProcedure*) storedProcedureNamed: (NSString *)name
+- (EOStoredProcedure*)storedProcedureNamed: (NSString *)name
 {
-  NSEnumerator *stEnum;
-  EOStoredProcedure *st;
+  EOStoredProcedure *proc;
+  NSString *procName;
+  unsigned i,n;
 
-  stEnum = [_storedProcedures objectEnumerator];
-  while ((st = [stEnum nextObject]))
+  n = [_storedProcedures count];
+  for (i=0; i<n; i++)
     {
-      if ([[st name] isEqual:name])
-	return st;
+      proc = [_storedProcedures objectAtIndex: i];
+      procName = [proc name];
+      if ([procName isEqual: name])
+	{
+	  return proc;
+	}
     }
 
   return nil;
 }
 
-- (NSArray*) storedProcedures
+- (NSArray*)storedProcedures
 {
-  //TODO revoir ?
-  if (!_storedProcedures)
-    {
-      NSArray *storedProcedures = nil;
-      NSArray *storedProcedureNames = [self storedProcedureNames];
-
-      EOFLOGObjectLevelArgs(@"gsdb", @"storedProcedureNames=%@",
-			    storedProcedureNames);
-
-      storedProcedures = [self resultsOfPerformingSelector:
-				 @selector(storedProcedureNamed:)
-			       withEachObjectInArray: storedProcedureNames];
-
-      EOFLOGObjectLevelArgs(@"gsdb", @"storedProcedures=%@", storedProcedures);
-
-      ASSIGN(_storedProcedures, [GCArray arrayWithArray:storedProcedures]);
-/*      [self performSelector:@selector(storedProcedureNamed:)
-            withEachObjectInArray:storedProcedureNames];
-*/
-    }
-
+  /* We do not load procedures lazy 
+     so we can rely on the instance variable.  */
   return _storedProcedures;
 }
 
@@ -577,6 +561,7 @@ NSString *EOEntityLoadedNotification = @"EOEntityLoadedNotification";
         }
 
       fileContents = [NSString stringWithContentsOfFile: indexPath];
+      NSAssert1(fileContents!=nil, @"File %@ could not be read.", indexPath);
       propList = [fileContents propertyList];
       EOFLOGObjectLevelArgs(@"gsdb", @"propList=%@", propList);
       NSAssert1(propList!=nil, @"Model at path %@ is invalid", indexPath);
@@ -599,30 +584,37 @@ NSString *EOEntityLoadedNotification = @"EOEntityLoadedNotification";
 }
 
 /**
- * Writes the receivers plist representation into an
- * .eomodeld file wrapper located at path.  
- * Depending on the the path extension .eomodeld or .eomodel
+ * <p>Writes the receivers plist representation into an
+ * .eomodeld file wrapper located at path.  </p>
+ * <p>Depending on the the path extension .eomodeld or .eomodel
  * the corresponding format will be used.
  * If the path has neither .eomodeld nor .eomodel path
- * extension, .eomodeld will be used.
- * If the file located at path already exists, a back is created
+ * extension, .eomodeld will be used.</p>
+ * <p>If the file located at path already exists, a back is created
  * by appending a '~' character to file name.
  * If a backup file already exists, when trying to create a backup,
- * the old backup will be deleted.
- * If any of the file operations fail, an NSInvalidArgumentException
- * will be raised.
+ * the old backup will be deleted.</p>
+ * <p>If any of the file operations fail, an NSInvalidArgumentException
+ * will be raised.</p>
+ * <p>This method as the side effeect of setting the receivers path and
+ * name.  The this change can happen even if the write operation fails
+ * with an exception.</p>
  */
-- (void) writeToFile: (NSString *)path
+- (void)writeToFile: (NSString *)path
 {
   NSFileManager		*mgr = [NSFileManager defaultManager];
   NSMutableDictionary	*pList;
   NSDictionary		*attributes;
   NSDictionary		*entityPList;
+  NSDictionary		*stProcPList;
   NSEnumerator		*entityEnum;
+  NSEnumerator		*stProcEnum;
   NSString              *fileName;
   NSString              *extension;
   BOOL writeSingleFile;
 
+  [self loadAllModelObjects];
+
   path = [path stringByStandardizingPath];
   extension = [path pathExtension];
 
@@ -635,8 +627,6 @@ NSString *EOEntityLoadedNotification = @"EOEntityLoadedNotification";
   
   writeSingleFile = [extension isEqualToString: @"eomodel"] ? YES : NO;
 
-  [self _setPath: path];
-
   if ([mgr fileExistsAtPath: path])
     {
       NSString *backupPath;
@@ -664,6 +654,8 @@ NSString *EOEntityLoadedNotification = @"EOEntityLoadedNotification";
 	}
     }
 
+  [self _setPath: path];
+
   pList = [NSMutableDictionary dictionaryWithCapacity: 10];
 
   [self encodeIntoPropertyList: pList];
@@ -701,6 +693,25 @@ NSString *EOEntityLoadedNotification = @"EOEntityLoadedNotification";
 	}
     }
 
+  stProcEnum = [[pList objectForKey: @"storedProcedures"] objectEnumerator];
+  while (writeSingleFile == NO
+	 && (stProcPList = [stProcEnum nextObject]))
+    {
+      NSString *fileName;
+
+      fileName = [stProcPList objectForKey: @"name"];
+      fileName = [fileName stringByAppendingPathExtension: @"storedProcedure"];
+      fileName = [path stringByAppendingPathComponent: fileName];
+      if ([stProcPList writeToFile: fileName atomically: YES] == NO)
+	{
+	  NSString *fmt;
+	  fmt = [NSString stringWithFormat: @"Could not create file: %@",
+			  fileName];
+	  [NSException raise: NSInvalidArgumentException
+		       format: fmt];
+	}
+    }
+
   if (writeSingleFile == NO)
     {
       fileName = [path stringByAppendingPathComponent: @"index.eomodeld"];
@@ -726,14 +737,15 @@ NSString *EOEntityLoadedNotification = @"EOEntityLoadedNotification";
 
 @implementation EOModel (EOModelPropertyList)
 
-- (id) initWithTableOfContentsPropertyList: (NSDictionary *)tableOfContents
-                                      path: (NSString *)path
+- (id)initWithTableOfContentsPropertyList: (NSDictionary *)tableOfContents
+                                     path: (NSString *)path
 {
   //OK
   NS_DURING
     {
       if ((self = [self init]))
         {
+	  NSString *modelPath;
           NSString *versionString = nil;
           NSArray  *entities = nil;
           int i, count = 0;
@@ -742,7 +754,8 @@ NSString *EOEntityLoadedNotification = @"EOEntityLoadedNotification";
 				tableOfContents);
 
 	  /* The call to _setPath: also sets the name implicitly. */
-          [self _setPath: [isa _formatModelPath: path checkFileSystem: YES]];
+	  modelPath = [isa _formatModelPath: path checkFileSystem: YES];
+	  [self _setPath: modelPath];
           EOFLOGObjectLevelArgs(@"gsdb", @"name=%@ path=%@", _name, _path);
 
           versionString = [tableOfContents objectForKey: @"EOModelVersion"];
@@ -770,47 +783,64 @@ NSString *EOEntityLoadedNotification = @"EOEntityLoadedNotification";
           if (_version >= 2)
             {
               NSMutableDictionary *markSP = [NSMutableDictionary dictionary];
-              NSArray *storedProcedures = [tableOfContents
-                                           objectForKey: @"storedProcedures"];
-              EOStoredProcedure *sp = nil;
-              NSEnumerator *enumerator = nil;
+              NSArray *storedProcedures 
+		= [tableOfContents objectForKey: @"storedProcedures"];
 
               count = [storedProcedures count];
-
               for (i = 0; i < count; i++)
                 {
-                  EOStoredProcedure *st;
+                  EOStoredProcedure *proc;
                   NSDictionary *plist;
                   NSString *fileName;
+		  NSString *procName;
+		  NSString *fullPath;
 
-                  fileName = [NSString stringWithFormat: @"%@.storedProcedure",
-                                       [[storedProcedures objectAtIndex: i]
-                                         objectForKey: @"name"]];	  
-                  plist = [[NSString stringWithContentsOfFile:
-                                       [_name stringByAppendingPathComponent:
-                                                fileName]]
-                            propertyList];	  
+		  procName = [storedProcedures objectAtIndex: i];
 
-                  [markSP setObject: plist
-                          forKey: [plist objectForKey: @"name"]];
+                  fileName = [procName stringByAppendingPathExtension:
+					 @"storedProcedure"];
 
-                  st = [EOStoredProcedure storedProcedureWithPropertyList: plist
-                                           owner: self];
-                  [self addStoredProcedure: st];
+		  fullPath = [_path stringByAppendingPathComponent: fileName];
+                  plist 
+		    = [NSDictionary dictionaryWithContentsOfFile: fullPath];
+
+		  NSAssert2([procName isEqual: [plist objectForKey: @"name"]],
+			    @"Name mismatch: index.plist: %@"
+			    @" *.storedProcedure: %@", procName,
+			    [plist objectForKey: @"name"]);
+                  [markSP setObject: plist forKey: procName];
+
+                  proc
+		    = [EOStoredProcedure storedProcedureWithPropertyList: plist
+					 owner: self];
+                  [self addStoredProcedure: proc];
                 }
 
-              enumerator = [_storedProcedures objectEnumerator];
-              while ((sp = [enumerator nextObject]))
-                [sp awakeWithPropertyList: [markSP objectForKey: [sp name]]];
+              count = [_storedProcedures count];
+              for (i = 0; i < count; i++)
+                {
+		  EOStoredProcedure *proc;
+		  NSString *name;
+		  NSDictionary *plist;
+
+		  proc = [_storedProcedures objectAtIndex: i];
+		  name = [proc name];
+		  plist = [markSP objectForKey: name];
+
+		  if (plist)
+		    {
+		      [proc awakeWithPropertyList: plist];
+		    }
+		}
             }
         
           entities = [tableOfContents objectForKey: @"entities"];
           count = [entities count];
 
-          for (i = 0; i < count; i++)
+          for (i=0; i<count; i++)
             {
-              [self _addFakeEntityWithPropertyList:
-                      [entities objectAtIndex: i]];
+	      NSDictionary *entityPList = [entities objectAtIndex: i];
+              [self _addFakeEntityWithPropertyList: entityPList];
             }
         }
     }
@@ -825,11 +855,12 @@ NSString *EOEntityLoadedNotification = @"EOEntityLoadedNotification";
   return self;
 }
 
-- (void) encodeTableOfContentsIntoPropertyList:
+- (void)encodeTableOfContentsIntoPropertyList:
   (NSMutableDictionary *)propertyList
 {
   int i, count;
   NSMutableArray *entitiesArray;
+  NSMutableArray *stProcArray;
 
   [propertyList setObject:
     [[NSNumber numberWithFloat: DEFAULT_MODEL_VERSION] stringValue]
@@ -868,6 +899,9 @@ NSString *EOEntityLoadedNotification = @"EOEntityLoadedNotification";
 
       [entitiesArray addObject: entityPList];
     }
+
+  stProcArray = [_storedProcedures valueForKey: @"name"];
+  [propertyList setObject: stProcArray forKey: @"storedProcedures"];
 }
 
 - (id) initWithPropertyList: (NSDictionary *)propertyList
@@ -939,8 +973,9 @@ NSString *EOEntityLoadedNotification = @"EOEntityLoadedNotification";
               [markEntities setObject: plist
                             forKey: [plist objectForKey: @"name"]];
           
-              entity = [EOEntity entityWithPropertyList: plist
-				 owner: self];
+              entity 
+		= AUTORELEASE([[EOEntity alloc] initWithPropertyList: plist
+						owner: self]);
               [self addEntity: entity];
             }
 
@@ -1053,8 +1088,9 @@ NSString *EOEntityLoadedNotification = @"EOEntityLoadedNotification";
       for (i = 0; i < count; i++)
         {
 	  NSMutableDictionary *entityPList = [NSMutableDictionary dictionary];
+	  EOEntity *entity = [_entities objectAtIndex: i];
 	  
-	  [[_entities objectAtIndex: i] encodeIntoPropertyList: entityPList];
+	  [entity encodeIntoPropertyList: entityPList];
 	  [entitiesArray addObject: entityPList];
         }
     }
@@ -1064,13 +1100,13 @@ NSString *EOEntityLoadedNotification = @"EOEntityLoadedNotification";
     {
       NSMutableArray *stArray = [NSMutableArray arrayWithCapacity: count];
 
-      [propertyList setObject: stArray forKey: @"entities"];
+      [propertyList setObject: stArray forKey: @"storedProcedures"];
       for (i = 0; i < count; i++)
         {
 	  NSMutableDictionary *stPList = [NSMutableDictionary dictionary];
+	  EOStoredProcedure *proc = [_storedProcedures objectAtIndex: i];
 
-	  [[_storedProcedures objectAtIndex: i]
-	    encodeIntoPropertyList: stPList];
+	  [proc encodeIntoPropertyList: stPList];
 	  [stArray addObject: stPList];
         }
     }
@@ -1125,14 +1161,6 @@ NSString *EOEntityLoadedNotification = @"EOEntityLoadedNotification";
 
           [EOClassDescription registerClassDescription: classDescription
                               forClass: entityClass];
-
-          /*      classDescription = [[EOClassDescription new] autorelease];
-                  EOFLOGObjectLevelArgs(@"gsdb",
-		                        @"classDescription=%@ aClass=%@",
-					classDescription, aClass);
-                  [EOClassDescription registerClassDescription: classDescription
-                  forClass: aClass];
-          */
         }
     }
   else if ([notificationName
@@ -1298,8 +1326,8 @@ NSString *EOEntityLoadedNotification = @"EOEntityLoadedNotification";
   NSAssert(propertyList, @"no propertyList");
   EOFLOGObjectLevelArgs(@"gsdb", @"propertyList=%@", propertyList);
 
-  entity = [EOEntity entityWithPropertyList: propertyList
-		     owner: self];
+  entity = AUTORELEASE([[EOEntity alloc] initWithPropertyList: propertyList
+					 owner: self]);
 
   NSAssert2([entity className], @"Entity %p named %@ has no class name",
 	    entity, [entity name]);
@@ -1391,28 +1419,35 @@ NSString *EOEntityLoadedNotification = @"EOEntityLoadedNotification";
 {
   if (![name isEqualToString: _name])
     {
-      //TODO
-/*
-      //???
-      self retain;
-      self modelGroup;
-      //todo if modelgroup;
-*/
+      EOModelGroup *group;
+
+      AUTORELEASE(RETAIN(self));
+
+      group = [self modelGroup];
+      if (group)
+	{
+	  [group removeModel: self];
+	}
+
+      [self willChange];
       ASSIGN(_name, name);
-/*
-      self modelGroup;
-      self release;
-*/
+
+      if (group)
+	{
+	  [group addModel: self];
+	}
     }
 }
 
 - (void) setAdaptorName: (NSString *)adaptorName
 {
+  [self willChange];
   ASSIGN(_adaptorName, adaptorName);
 }
 
 - (void) setConnectionDictionary: (NSDictionary *)connectionDictionary
 {
+  [self willChange];
   ASSIGN(_connectionDictionary, connectionDictionary);
 }
 
@@ -1435,14 +1470,15 @@ NSString *EOEntityLoadedNotification = @"EOEntityLoadedNotification";
 //  int count;
   NSString *className = nil;
 
-  if ([self entityNamed: [entity name]])
-    [NSException raise: NSInvalidArgumentException
-                 format: @"%@ -- %@ 0x%x: \"%@\" already registered as entity name ",
-                 NSStringFromSelector(_cmd),
-                 NSStringFromClass([self class]),
-                 self,
-                 entityName];
+  NSAssert1([self entityNamed: [entity name]] == nil,
+	    @"Entity '%@' already registered as with this model",
+	    entityName);
 
+  NSAssert2([entity model]==nil,
+	    @"Entity '%@' is already owned by model '%@'.",
+	    [entity name], [[entity model] name]);
+
+  [self willChange];
   /* Do not access _entities until cache is triggered */
   if ([self createsMutableObjects])
     [(GCMutableArray *)[self entities] addObject: entity];
@@ -1466,20 +1502,22 @@ NSString *EOEntityLoadedNotification = @"EOEntityLoadedNotification";
 
   [_entitiesByName setObject: entity 
                    forKey: entityName];
-  [entity setModel: self];
+  [entity _setModel: self];
 }
 
 - (void) removeEntity: (EOEntity *)entity
 {
   NSString *className = nil;
 
-  [entity setModel: nil];
+  /* as a special case this method can call _setModel: with nil */
+  [entity _setModel: nil];
   [_entitiesByName removeObjectForKey: [entity name]];
 
   NSAssert(_entitiesByClass, @"No _entitiesByClass");
 
   className = [entity className];
   NSAssert1(className, @"No className in %@", entity);
+  [self willChange];
   NSMapRemove(_entitiesByClass, className);
 
   /* Do not access _entities until cache is triggered */
@@ -1500,7 +1538,7 @@ NSString *EOEntityLoadedNotification = @"EOEntityLoadedNotification";
   // TODO;
 }
 
-- (void) addStoredProcedure: (EOStoredProcedure *)storedProcedure
+- (void)addStoredProcedure: (EOStoredProcedure *)storedProcedure
 {
   if ([self storedProcedureNamed: [storedProcedure name]])
     [NSException raise: NSInvalidArgumentException
@@ -1509,27 +1547,34 @@ NSString *EOEntityLoadedNotification = @"EOEntityLoadedNotification";
                  NSStringFromClass([self class]),
                  self,
                  [storedProcedure name]];
-  
+  NSAssert(_storedProcedures, @"Uninitialised _storedProcedures!");
+  [self willChange];
   if ([self createsMutableObjects])
     [(GCMutableArray *)_storedProcedures addObject: storedProcedure];
   else
     {
-      _storedProcedures = [[[GCMutableArray alloc] initWithArray:[_storedProcedures autorelease] copyItems:NO]
-			    autorelease];
-      [(GCMutableArray *)_storedProcedures addObject: storedProcedure];
-      _storedProcedures = [[GCArray alloc] initWithArray:_storedProcedures copyItems:NO];
+      NSMutableArray *mCopy = AUTORELEASE([_storedProcedures mutableCopy]);
+      [mCopy removeObject: storedProcedure];
+      mCopy = AUTORELEASE([[GCArray alloc] initWithArray: mCopy
+					   copyItems: NO]);
+      ASSIGN(_storedProcedures, mCopy);
     }
 }
 
-- (void) removeStoredProcedure: (EOStoredProcedure *)storedProcedure
+- (void)removeStoredProcedure: (EOStoredProcedure *)storedProcedure
 {
+  NSAssert(_storedProcedures, @"Uninitialised _storedProcedures!");
+
+  [self willChange];
   if ([self createsMutableObjects])
     [(GCMutableArray *)_storedProcedures removeObject: storedProcedure];
   else
     {
-      _storedProcedures = [[_storedProcedures autorelease] mutableCopy];
-      [(GCMutableArray *)_storedProcedures removeObject: storedProcedure];
-      _storedProcedures = [[GCArray alloc] initWithArray:[_storedProcedures autorelease] copyItems:NO];
+      NSMutableArray *mCopy = AUTORELEASE([_storedProcedures mutableCopy]);
+      [mCopy removeObject: storedProcedure];
+      mCopy = AUTORELEASE([[GCArray alloc] initWithArray: mCopy
+					   copyItems: NO]);
+      ASSIGN(_storedProcedures, mCopy);
     }
 }
 
@@ -1543,23 +1588,59 @@ NSString *EOEntityLoadedNotification = @"EOEntityLoadedNotification";
   EOFLOGObjectFnStop();
 }
 
-- (void) loadAllModelObjects
+- (void)loadAllModelObjects
 {
-  //Ayers: Review
-  //NSArray *storedProcedures = [self storedProcedures];
-  //TODO something if storedProcedures ?
-  //NSArray *entities = [self entities];
+  NSArray *entityNames = [_entitiesByName allKeys];
+  unsigned i,n = [entityNames count];
 
-  //TODO something if entities ?
-  [self willChange];
+  for (i=0; i<n; i++)
+    {
+      NSString *name = [entityNames objectAtIndex: i];
+      id entity = [_entitiesByName objectForKey: name];
+      [self _verifyBuiltEntityObject: entity
+	    named: name];
+    }
 }
 
-- (NSArray *) referencesToProperty: property
+/**
+ * Returns an array of flattened attributes and relationships in the receiver's
+ * entities that reference property, or nil if nothing references it.
+ */
+- (NSArray *) referencesToProperty: (id)property
 {
-  // TODO
-  [self notImplemented: _cmd];
-
-  return nil;
+  // TODO test
+  NSEnumerator *entityEnumerator = [[self entities] objectEnumerator];
+  IMP enumNO=NULL;
+  EOEntity *ent;
+  NSMutableArray *refProps = [NSMutableArray array];
+  
+  while ((ent = GDL2_NextObjectWithImpPtr(entityEnumerator,&enumNO)))
+    {
+      NSEnumerator *propEnumerator = [[ent attributes] objectEnumerator];
+      EOAttribute *attr;
+      EORelationship *rel;
+      IMP propEnumNO=NULL;
+      
+      while ((attr = GDL2_NextObjectWithImpPtr(propEnumerator,&propEnumNO)))
+	{
+          if ([attr isFlattened] && [[attr realAttribute] isEqual: property])
+	    {
+	      [refProps addObject:attr];
+	    }
+	}
+      
+      propEnumerator = [[ent relationships] objectEnumerator];
+      propEnumNO = NULL;
+      while ((rel = GDL2_NextObjectWithImpPtr(propEnumerator, &propEnumNO)))
+	{
+	  if ([rel referencesProperty:property])
+	    {
+	      [refProps addObject:rel];
+	    }
+	}
+    }	
+  
+  return [refProps count] ? [NSArray arrayWithArray:refProps] : nil;
 }
 
 - (NSArray *) externalModelsReferenced
@@ -1659,7 +1740,7 @@ NSString *EOEntityLoadedNotification = @"EOEntityLoadedNotification";
       searchPath =
 	  [searchPath stringByAppendingPathComponent: @"index.eomodeld"];
     }
-        
+  
   searchPath = [searchPath stringByStandardizingPath];
 
   if (chkFS==YES)
@@ -1704,13 +1785,21 @@ NSString *EOEntityLoadedNotification = @"EOEntityLoadedNotification";
 {
   if (_flags.createsMutableObjects != flag)
     {
+      NSArray *entityArray = [self entities];
       _flags.createsMutableObjects = flag;
       
       /* Do not access _entities until cache is triggered */
       if (_flags.createsMutableObjects)
-	_entities = [[GCMutableArray alloc] initWithArray:[[self entities] autorelease] copyItems:NO];
+	{
+	  entityArray = [[GCMutableArray alloc] initWithArray: entityArray
+						copyItems:NO];
+	}
       else
-	_entities = [[GCArray alloc] initWithArray:[[self entities] autorelease] copyItems:NO];
+	{
+	  entityArray = [[GCArray alloc] initWithArray: entityArray
+					 copyItems:NO];
+	}
+      ASSIGN(_entities, entityArray);
     }
 }
 
@@ -1742,7 +1831,7 @@ NSString *EOEntityLoadedNotification = @"EOEntityLoadedNotification";
           EOFLOGObjectLevelArgs(@"gsdb", @"[self path]=%@", [self path]);
 
           basePath = [self path]; 
-          [RETAIN(entity) autorelease]; //so it won't be lost in _removeEntity
+          AUTORELEASE(RETAIN(entity)); //so it won't be lost in _removeEntity
 
           EOFLOGObjectLevelArgs(@"gsdb", @"basePath =%@", basePath);
 
@@ -1797,4 +1886,32 @@ NSString *EOEntityLoadedNotification = @"EOEntityLoadedNotification";
   return entity;
 }
 
+/*
+  This method rebuilds the caches:
+  _entitiesByName, _subEntitiesCache, _entitiesByClass
+  from _entities.  Insure this works for real entities and dictionaries
+  which could be contained in _entities.  Also note that className need
+  not be unique.
+*/
+- (void)_updateCache
+{
+  NSArray  *names;
+  EOEntity *entity;
+  NSString *className;
+  unsigned int i,c;
+
+  DESTROY(_entitiesByName);
+  DESTROY(_subEntitiesCache);
+  NSResetMapTable(_entitiesByClass);
+
+  names = [_entities valueForKey: @"name"];
+  _entitiesByName = [[GCMutableDictionary alloc] initWithObjects: _entities
+						 forKeys: names];
+  for (i = 0, c = [_entities count]; i < c; i++)
+    {
+      entity = [_entities objectAtIndex: i]; /* entity or dictionary */
+      className = [entity valueForKey: @"className"];
+      NSMapInsertIfAbsent(_entitiesByClass, className, entity);
+    }
+}
 @end /* EOModel (EOModelPrivate) */
