@@ -1,13 +1,13 @@
 /** 
    EOModelGroup.m <title>EOModelGroup Class</title>
 
-   Copyright (C) 2000 Free Software Foundation, Inc.
+   Copyright (C) 2000,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: June 2000
 
-   $Revision: 1.10 $
-   $Date: 2004/03/29 19:00:45 $
+   $Revision: 21645 $
+   $Date: 2005-08-17 10:07:57 +0200 (Mit, 17 Aug 2005) $
 
    <abstract></abstract>
 
@@ -27,13 +27,13 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
 #include "config.h"
 
-RCS_ID("$Id: EOModelGroup.m,v 1.10 2004/03/29 19:00:45 ayers Exp $")
+RCS_ID("$Id: EOModelGroup.m 21645 2005-08-17 08:07:57Z ayers $")
 
 #ifdef GNUSTEP
 #include <Foundation/NSBundle.h>
@@ -123,10 +123,11 @@ static EOModelGroup *globalModelGroup = nil;
     {
       NSMutableArray *bundles = [NSMutableArray arrayWithCapacity: 2];
       NSBundle *bundle = nil;
-      NSArray *paths = nil;
+      NSMutableArray *paths = nil;
       NSEnumerator *pathsEnum = nil;
       NSEnumerator *bundleEnum = nil;
       NSString *path = nil;
+      id tmp;
 
       globalModelGroup = [EOModelGroup new];
 
@@ -138,8 +139,13 @@ static EOModelGroup *globalModelGroup = nil;
       bundleEnum = [bundles objectEnumerator];
       while ((bundle = [bundleEnum nextObject]))
 	{
-	  paths = [bundle pathsForResourcesOfType: @"eomodeld"
-			  inDirectory: nil];
+	  paths = (id)[NSMutableArray array];
+	  tmp = [bundle pathsForResourcesOfType: @"eomodeld"
+			inDirectory: nil];
+	  [paths addObjectsFromArray: tmp];
+	  tmp = [bundle pathsForResourcesOfType: @"eomodel"
+			inDirectory: nil];
+	  [paths addObjectsFromArray: tmp];
 
 	  if (!paths)
 	    {
@@ -150,8 +156,6 @@ static EOModelGroup *globalModelGroup = nil;
 	  pathsEnum = [paths objectEnumerator];
 	  while ((path = [pathsEnum nextObject]))
 	    {
-	      path = [path stringByDeletingPathExtension];
-	      NSLog(@"%@", path);
 	      [globalModelGroup addModelWithFile: path];
 	    }
 	}
