@@ -1,7 +1,7 @@
 /* -*-objc-*-
    EORelationship.h
 
-   Copyright (C) 2000 Free Software Foundation, Inc.
+   Copyright (C) 2000,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: February 2000
@@ -21,7 +21,7 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 #ifndef __EORelationship_h__
@@ -45,6 +45,7 @@
 @class EOEntity;
 @class EOAttribute;
 @class EOExpressionArray;
+@class EOMutableKnownKeyDictionary;
 @class EOMKKDSubsetMapping;
 @class EOQualifier;
 
@@ -205,7 +206,7 @@ typedef enum {
 - (NSArray *)_intermediateAttributes;
 - (EORelationship *)lastRelationship;
 - (EORelationship *)firstRelationship;
-- (id)intermediateEntity;
+- (EOEntity*) intermediateEntity;
 - (BOOL)isMultiHop;
 - (void)_setSourceToDestinationKeyMap: (id)param0;
 - (id)qualifierForDBSnapshot: (id)param0;
@@ -221,7 +222,7 @@ typedef enum {
 - (id)qualifierOmittingAuxiliaryQualifierWithSourceRow: (id)param0;
 - (id)auxiliaryQualifier;
 - (void)setAuxiliaryQualifier: (id)param0;
-- (id)_foreignKeyForSourceRow: (NSDictionary *)row;
+- (EOMutableKnownKeyDictionary *)_foreignKeyForSourceRow: (NSDictionary *)row;
 - (EOMKKDSubsetMapping *)_sourceRowToForeignKeyMapping;
 - (NSArray *)_sourceAttributeNames;
 - (EOJoin *)joinForAttribute: (EOAttribute *)attribute;
