@@ -1,7 +1,7 @@
 /** 
    EORelationship.m <title>EORelationship</title>
 
-   Copyright (C) 2000-2002 Free Software Foundation, Inc.
+   Copyright (C) 2000-2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: February 2000
@@ -9,8 +9,8 @@
    Author: Manuel Guesdon <mguesdon@orange-concept.com>
    Date: October 2000
 
-   $Revision: 1.18 $
-   $Date: 2004/03/19 16:41:19 $
+   $Revision: 23472 $
+   $Date: 2006-09-12 21:36:24 +0200 (Die, 12 Sep 2006) $
 
    <abstract></abstract>
 
@@ -30,13 +30,13 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
 #include "config.h"
 
-RCS_ID("$Id: EORelationship.m,v 1.18 2004/03/19 16:41:19 ayers Exp $")
+RCS_ID("$Id: EORelationship.m 23472 2006-09-12 19:36:24Z ayers $")
 
 #ifdef GNUSTEP
 #include <Foundation/NSArray.h>
@@ -60,18 +60,38 @@ RCS_ID("$Id: EORelationship.m,v 1.18 2004/03/19 16:41:19 ayers Exp $")
 
 #include <EOAccess/EOModel.h>
 #include <EOAccess/EOAttribute.h>
-#include <EOAccess/EOAttributePriv.h>
 #include <EOAccess/EOEntity.h>
-#include <EOAccess/EOEntityPriv.h>
 #include <EOAccess/EOStoredProcedure.h>
 #include <EOAccess/EORelationship.h>
 #include <EOAccess/EOJoin.h>
 #include <EOAccess/EOExpressionArray.h>
 
+#include "EOPrivate.h"
+#include "EOAttributePriv.h"
+#include "EOEntityPriv.h"
+
 
 @implementation EORelationship
 
-- init
++ (void)initialize
+{
+  static BOOL initialized = NO;
+  if (!initialized)
+    {
+      initialized = YES;
+
+      GDL2_EOAccessPrivateInit();
+    }
+}
+
++ (id) relationshipWithPropertyList: (NSDictionary *)propertyList
+                              owner: (id)owner
+{
+  return AUTORELEASE([[self alloc] initWithPropertyList: propertyList
+				   owner: owner]);
+}
+
+- (id)init
 {
 //OK
   if ((self = [super init]))
@@ -183,13 +203,6 @@ RCS_ID("$Id: EORelationship.m,v 1.18 2004/03/19 16:41:19 ayers Exp $")
   return [_name hash];
 }
 
-+ (id) relationshipWithPropertyList: (NSDictionary *)propertyList
-                              owner: (id)owner
-{
-  return [[[self alloc] initWithPropertyList: propertyList
-                        owner: owner] autorelease];
-}
-
 - (id) initWithPropertyList: (NSDictionary *)propertyList
                       owner: (id)owner
 {
@@ -797,17 +810,21 @@ to know what to-many mean :-)  **/
 - (BOOL)referencesProperty: (id)property
 {
   BOOL referencesProperty = NO;
+  NSArray *srcAttribs = [self sourceAttributes];
+  NSArray *destAttribs = [self destinationAttributes];
+  NSArray *compRels = [self componentRelationships];
 
   NSEmitTODO();  //TODO
   EOFLOGObjectLevelArgs(@"EORelationship", @"in referencesProperty:%@",
 			property);
 
-  referencesProperty = ([[self sourceAttributes]
-			  indexOfObject: property] != NSNotFound
-			|| [[self destinationAttributes]
-			     indexOfObject: property] != NSNotFound
-			|| [[self componentRelationships]
-			     indexOfObject: property] != NSNotFound);
+  referencesProperty =
+	  	((srcAttribs
+		  && [srcAttribs indexOfObject: property] != NSNotFound)
+		|| (destAttribs
+		    && [destAttribs indexOfObject: property] != NSNotFound)
+		|| (compRels
+		    && [compRels indexOfObject: property] != NSNotFound));
 
   return referencesProperty;
 }
@@ -1205,6 +1222,7 @@ relationships. Nil if none" **/
   int exc = 0;
   NSArray *storedProcedures = nil;
 
+  if ([_name isEqual:name]) return nil;
   if (!name || ![name length])
     exc++;
   if (!exc)
@@ -1222,6 +1240,16 @@ relationships. Nil if none" **/
         }
       if (!exc && *s == '$')
         exc++;
+  
+    if (exc)
+      return [NSException exceptionWithName: NSInvalidArgumentException
+                         reason: [NSString stringWithFormat: @"%@ -- %@ 0x%x: argument \"%@\" contains invalid char '%c'", 
+					  NSStringFromSelector(_cmd),
+					  NSStringFromClass([self class]),
+					  self,
+                                         name,
+					 *p]
+                        userInfo: nil];
       
       if ([[self entity] anyAttributeNamed: name])
         exc++;
@@ -1253,15 +1281,17 @@ relationships. Nil if none" **/
     }
 
   if (exc)
-    return [NSException exceptionWithName: NSInvalidArgumentException
-                        reason: [NSString stringWithFormat: @"%@ -- %@ 0x%x: argument \"%@\" contains invalid chars", 
-					  NSStringFromSelector(_cmd),
-					  NSStringFromClass([self class]),
-					  self,
-                                         name]
+    {
+      return [NSException exceptionWithName: NSInvalidArgumentException
+                         reason: [NSString stringWithFormat: @"%@ -- %@ 0x%x: \"%@\" already used in the model",
+                                 NSStringFromSelector(_cmd),
+                                 NSStringFromClass([self class]),
+                                 self,
+                                 name]
                         userInfo: nil];
-  else
-    return nil;
+    }
+
+  return nil;
 }
 
 - (void)setToMany: (BOOL)flag
@@ -1274,6 +1304,7 @@ relationships. Nil if none" **/
                  NSStringFromClass([self class]),
                  self];
 
+  [self willChange];
   _flags.isToMany = flag;
 }
 
@@ -1294,13 +1325,16 @@ relationships. Nil if none" **/
 
   EOFLOGObjectLevelArgs(@"EORelationship", @"definition=%@", definition);
 
+  [self _flushCache];
+  [self willChange];
+
   if (definition)
     {
-      [self _flushCache];
-      [self willChange];
-
       _flags.isToMany = NO;
 
+      NSAssert1(_entity,@"No entity for relationship %@",
+                self);
+
       ASSIGN(_definitionArray, [_entity _parseRelationshipPath: definition]);
 
       EOFLOGObjectLevelArgs(@"EORelationship", @"_definitionArray=%@", _definitionArray);
@@ -1330,8 +1364,13 @@ relationships. Nil if none" **/
         }
       }
 
-      [_entity _setIsEdited];
     }
+  else /* definition == nil */
+    {
+      DESTROY(_definitionArray);
+    }
+  /* Ayers: Not sure what justifies this. */
+  [_entity _setIsEdited];
 
   EOFLOGObjectFnStop();
 }
@@ -1343,6 +1382,8 @@ relationships. Nil if none" **/
     {
       [self _flushCache];
       [self willChange];
+      [_entity _setIsEdited];
+      [entity _setIsEdited];
       ASSIGN(_entity, entity);
     }
 }
@@ -1352,6 +1393,7 @@ relationships. Nil if none" **/
   //OK
   [self willChange];
   ASSIGN(_userInfo, dictionary);
+  /* Ayers: Not sure what justifies this. */
   [_entity _setIsEdited];
 }
 
@@ -1360,6 +1402,7 @@ relationships. Nil if none" **/
   //OK
   [self willChange];
   ASSIGN(_internalInfo, dictionary);
+  /* Ayers: Not sure what justifies this. */
   [_entity _setIsEdited];
 }
 
@@ -1368,6 +1411,7 @@ relationships. Nil if none" **/
   //OK
   [self willChange];
   ASSIGN(_docComment, docComment);
+  /* Ayers: Not sure what justifies this. */
   [_entity _setIsEdited];
 }
 
@@ -1549,7 +1593,8 @@ relationships. Nil if none" **/
                 EOFLOGObjectLevel(@"EORelationship", @"added");
 
                 [self _joinsChanged];
-                [_entity _setIsEdited];
+		/* Ayers: Not sure what justifies this. */
+		[_entity _setIsEdited];
             }
         }
     }
@@ -1562,7 +1607,6 @@ relationships. Nil if none" **/
   EOFLOGObjectFnStart();
 
   [self _flushCache];
-  [self willChange];
 
   if ([self isFlattened] == YES)
     [NSException raise: NSInvalidArgumentException
@@ -1572,6 +1616,7 @@ relationships. Nil if none" **/
                  self];
   else
     {
+      [self willChange];
       if ([self createsMutableObjects])
         {
           [(GCMutableArray *)_joins removeObject: join];
@@ -1612,6 +1657,7 @@ relationships. Nil if none" **/
         }
 
       [self _joinsChanged];
+      /* Ayers: Not sure what justifies this. */
       [_entity _setIsEdited];
     }
 
@@ -1672,6 +1718,7 @@ becomes "name", and "FIRST_NAME" becomes "firstName".*/
 
 - (void)setNumberOfToManyFaultsToBatchFetch: (unsigned int)size
 {
+  [self willChange];
   _batchCount = size;
 }
 
@@ -1681,6 +1728,7 @@ becomes "name", and "FIRST_NAME" becomes "firstName".*/
 	    @"Bad deleteRule numeric value: %d",
             deleteRule);
 
+  [self willChange];
   _flags.deleteRule = deleteRule;
 }
 
@@ -1693,8 +1741,17 @@ becomes "name", and "FIRST_NAME" becomes "firstName".*/
 
 @end
 
-@implementation EORelationship(EORelationshipValueMapping)
+@implementation EORelationship (EORelationshipValueMapping)
 
+/**
+ * If the reciever is a manditory relationship, this method
+ * returns an exception if the value pointed to by VALUEP is
+ * either nil or the EONull instance for to-one relationships
+ * or an empty NSArray for to-many relationships.  Otherwise
+ * it returns nil.  EOClassDescription adds further information
+ * to this exception before it gets passed to the application or
+ * user.
+ */
 - (NSException *)validateValue: (id*)valueP
 {
   //OK
@@ -1708,7 +1765,7 @@ becomes "name", and "FIRST_NAME" becomes "firstName".*/
     {
       BOOL isToMany = [self isToMany];
 
-      if ((isToMany == NO && *valueP == nil)
+      if ((isToMany == NO && _isNilOrEONull(*valueP))
 	  || (isToMany == YES && [*valueP count] == 0))
         {
           EOEntity *destinationEntity = [self destinationEntity];
@@ -1719,31 +1776,9 @@ becomes "name", and "FIRST_NAME" becomes "firstName".*/
 				   [self name],
 				   [entity name],
 				   [destinationEntity name]];
-          /* //TODO userinfo:
-            userInfo {
-            EOValidatedObjectUserInfoKey = {
-            ...
-            }; 
-            }; 
-            EOValidatedPropertyUserInfoKey = quotationPlace; 
-            }EOValidatedObjectUserInfoKey={
-            ...
-            }; 
-            }
-            EOValidatedPropertyUserInfoKey=quotationPlace            
-          */
         }
     }
 
-  if (!exception)
-    {
-      NSEmitTODO(); //TODO
-      NSDebugMLog(@"relationship=%@ valueP=%p",self,valueP);
-      if (valueP)
-        NSDebugMLog(@"*valueP=%@",*valueP);
-      //[self notImplemented:_cmd]; //TODO
-    }
-
   EOFLOGObjectFnStop();
 
   return exception;
@@ -1795,8 +1830,10 @@ becomes "name", and "FIRST_NAME" becomes "firstName".*/
   return _flags.createsMutableObjects;
 }
 
+/* TODO this method should probably be private. */
 - (void)setInverseRelationship: (EORelationship*)relationship
 {
+  [self willChange]; // TODO: verify
   ASSIGN(_inverseRelationship,relationship);
 }
 
@@ -2131,9 +2168,9 @@ dst entity primaryKeyAttributeNames
 }
 
 /** Return dictionary of key/value for destination object of source row/object **/
-- (NSDictionary*) _foreignKeyForSourceRow: (NSDictionary*)row
+- (EOMutableKnownKeyDictionary *) _foreignKeyForSourceRow: (NSDictionary*)row
 {
-  NSDictionary *foreignKey = nil;
+  EOMutableKnownKeyDictionary *foreignKey = nil;
   EOMKKDSubsetMapping *sourceRowToForeignKeyMapping = nil;
 
   EOFLOGObjectFnStart();
@@ -2343,7 +2380,7 @@ dst entity primaryKeyAttributeNames
   return keyMap;
 }
 
-- (id) _leftSideKeyMap
+- (NSDictionary *) _leftSideKeyMap
 {
   NSDictionary *keyMap = nil;
 
