@@ -1,7 +1,7 @@
 /* -*-objc-*-
    EOSQLExpression.h
 
-   Copyright (C) 2000 Free Software Foundation, Inc.
+   Copyright (C) 2000,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: February 2000
@@ -21,7 +21,7 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 #ifndef __EOSQLExpression_h__
@@ -94,7 +94,7 @@ GDL2ACCESS_EXPORT NSString *EOBindVariableColumnKey;
 /* The endity parameter is typed to 'id' for signature compatibility
    with EOF.  Note that only EOEntity * objects should be passed.  */
 + (EOSQLExpression *)deleteStatementWithQualifier: (EOQualifier *)qualifier
-					   entity: (id)entity;
+					   entity: (EOEntity *)entity;
 
 + (EOSQLExpression *)selectStatementForAttributes: (NSArray *)attributes
 					     lock: (BOOL)yn
