@@ -1,7 +1,7 @@
 /** 
    EOSQLExpression.m <title>EOSQLExpression Class</title>
 
-   Copyright (C) 2000-2002 Free Software Foundation, Inc.
+   Copyright (C) 2000-2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: February 2000
@@ -9,8 +9,8 @@
    Author: Manuel Guesdon <mguesdon@orange-concept.com>
    Date: November 2001
 
-   $Revision: 1.20 $
-   $Date: 2004/03/19 16:41:19 $
+   $Revision: 23472 $
+   $Date: 2006-09-12 21:36:24 +0200 (Die, 12 Sep 2006) $
 
    <abstract></abstract>
 
@@ -30,13 +30,13 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
 #include "config.h"
 
-RCS_ID("$Id: EOSQLExpression.m,v 1.20 2004/03/19 16:41:19 ayers Exp $")
+RCS_ID("$Id: EOSQLExpression.m 23472 2006-09-12 19:36:24Z ayers $")
 
 #include <string.h>
 
@@ -67,18 +67,20 @@ RCS_ID("$Id: EOSQLExpression.m,v 1.20 2004/03/19 16:41:19 ayers Exp $")
 #include <EOAccess/EOModel.h>
 #include <EOAccess/EOEntity.h>
 #include <EOAccess/EOAttribute.h>
-#include <EOAccess/EOAttributePriv.h>
 #include <EOAccess/EORelationship.h>
 #include <EOAccess/EOAdaptor.h>
 #include <EOAccess/EOAdaptorContext.h>
 #include <EOAccess/EOAdaptorChannel.h>
 #include <EOAccess/EOJoin.h>
 #include <EOAccess/EOSQLExpression.h>
-#include <EOAccess/EOSQLExpressionPriv.h>
 #include <EOAccess/EOSQLQualifier.h>
 #include <EOAccess/EOExpressionArray.h>
 #include <EOAccess/EOSchemaGeneration.h>
 
+#include "EOPrivate.h"
+#include "EOEntityPriv.h"
+#include "EOAttributePriv.h"
+#include "EOSQLExpressionPriv.h"
 
 
 NSString *EOBindVariableNameKey = @"EOBindVariableNameKey";
@@ -87,13 +89,21 @@ NSString *EOBindVariableValueKey = @"EOBindVariableValueKey";
 NSString *EOBindVariablePlaceHolderKey = @"EOBindVariablePlaceHolderKey";
 NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
 
-
 @interface EOSQLExpression(Private)
 + (id)sqlExpressionWithEntity: (EOEntity *)entity;
 @end
 
 @implementation EOSQLExpression
 
++ (void) initialize
+{
+  static BOOL initialized=NO;
+  if (!initialized)
+    {
+      GDL2_EOAccessPrivateInit();
+    };
+};
+
 + (id)sqlExpressionWithEntity: (EOEntity *)entity
 {
   return [[[self alloc] initWithEntity: entity] autorelease];
@@ -152,8 +162,8 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
   return exp;
 }
 
-+ insertStatementForRow: (NSDictionary *)row
-                 entity: (EOEntity *)entity
++ (EOSQLExpression *)insertStatementForRow: (NSDictionary *)row
+				    entity: (EOEntity *)entity
 {
   EOSQLExpression *sqlExpression;
 
@@ -172,9 +182,9 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
   return sqlExpression;
 }
 
-+ updateStatementForRow: (NSDictionary *)row
-              qualifier: (EOQualifier *)qualifier
-                 entity: (EOEntity *)entity
++ (EOSQLExpression *)updateStatementForRow: (NSDictionary *)row
+				 qualifier: (EOQualifier *)qualifier
+				    entity: (EOEntity *)entity
 {
   EOSQLExpression *sqlExpression;
 
@@ -205,8 +215,8 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
   return sqlExpression;
 }
 
-+ deleteStatementWithQualifier: (EOQualifier *)qualifier
-                        entity: (EOEntity *)entity
++ (EOSQLExpression *)deleteStatementWithQualifier: (EOQualifier *)qualifier
+					   entity: (EOEntity *)entity
 {
   EOSQLExpression *sqlExpression;
 
@@ -227,10 +237,10 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
   return sqlExpression;
 }
 
-+ selectStatementForAttributes: (NSArray *)attributes
-                          lock: (BOOL)flag
-            fetchSpecification: (EOFetchSpecification *)fetchSpecification
-                        entity: (EOEntity *)entity
++ (EOSQLExpression *)selectStatementForAttributes: (NSArray *)attributes
+					     lock: (BOOL)flag
+			       fetchSpecification: (EOFetchSpecification *)fetchSpecification
+					   entity: (EOEntity *)entity
 {
   EOSQLExpression *sqlExpression;
 
@@ -339,12 +349,13 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
 // insert: ret quotation_place ?? / select: ret quotation_place t0
 
   NSMutableString *entitiesString = [NSMutableString string];
-  NSEnumerator *relationshipEnum;
-  NSString *relationshipPath;
-  EOEntity *currentEntity;
+  IMP entitiesStringAppendStringIMP = NULL;
+  NSEnumerator *relationshipEnum = nil;
+  NSString *relationshipPath = nil;
+  EOEntity *currentEntity = nil;
   int i = 0;
 
-  EOFLOGObjectFnStart();
+  EOFLOGObjectFnStartCond(@"EOSQLExpression");
 
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"entity=%@", entity);
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"_aliasesByRelationshipPath=%@",
@@ -356,7 +367,10 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
       currentEntity = entity;
 
       if (i)
-	[entitiesString appendString: @", "];
+	GDL2_AppendStringWithImp(entitiesString,
+                                entitiesStringAppendStringIMP,@", ");
+      else
+        entitiesStringAppendStringIMP = [entitiesString methodForSelector:@selector(appendString:)];
 
       if ([relationshipPath isEqualToString: @""])
         {
@@ -373,7 +387,8 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
 		    @"No external name for entity %@",
                     [currentEntity name]);
 
-	  [entitiesString appendString: tableName];
+	  GDL2_AppendStringWithImp(entitiesString,
+                                  entitiesStringAppendStringIMP,tableName);
 
 	  if (_flags.useAliases)
 	    [entitiesString appendFormat: @" %@",
@@ -384,7 +399,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
         {
 	  NSEnumerator *defEnum = nil;
 	  NSArray *defArray = nil;
-	  NSString *relationshipString;
+	  NSString *relationshipString = nil;
           NSString *tableName = nil;
 
 	  defArray = [relationshipPath componentsSeparatedByString: @"."];
@@ -420,14 +435,18 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
 		    @"No external name for entity %@",
                     [currentEntity name]);
 
-	  [entitiesString appendString: tableName];
+	  GDL2_AppendStringWithImp(entitiesString,
+                                  entitiesStringAppendStringIMP,tableName);
 
 	  if (_flags.useAliases)
             {
               NSString *alias = [_aliasesByRelationshipPath
 				  objectForKey: relationshipPath];
 
-              [entitiesString appendFormat: @" %@",alias];
+              GDL2_AppendStringWithImp(entitiesString,
+                                  entitiesStringAppendStringIMP,@" ");
+              GDL2_AppendStringWithImp(entitiesString,
+                                  entitiesStringAppendStringIMP,alias);
 
               EOFLOGObjectLevelArgs(@"EOSQLExpression",
 				    @"appending alias %@ in entitiesString",
@@ -442,7 +461,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
                         @"entitiesString=%@",
                         entitiesString);
 
-  EOFLOGObjectFnStop();
+  EOFLOGObjectFnStopCond(@"EOSQLExpression");
 
   return entitiesString;
 }
@@ -455,7 +474,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
   NSEnumerator *rowEnum;
   NSString *attributeName;
 
-  EOFLOGObjectFnStart();
+  EOFLOGObjectFnStartCond(@"EOSQLExpression");
 
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"row=%@", row);
 
@@ -504,7 +523,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
 
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"_statement=%@", _statement);
 
-  EOFLOGObjectFnStop();
+  EOFLOGObjectFnStopCond(@"EOSQLExpression");
 }
 
 - (void)prepareUpdateExpressionWithRow: (NSDictionary *)row
@@ -518,7 +537,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
   NSEnumerator *rowEnum;
   NSString *attributeName;
 
-  EOFLOGObjectFnStart();
+  EOFLOGObjectFnStartCond(@"EOSQLExpression");
 
   rowEnum = [row keyEnumerator];
   while ((attributeName = [rowEnum nextObject]))
@@ -549,7 +568,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
 
   ASSIGN(_statement, statement);
 
-  EOFLOGObjectFnStop();
+  EOFLOGObjectFnStopCond(@"EOSQLExpression");
 }
 
 - (void)prepareDeleteExpressionForQualifier: (EOQualifier *)qualifier
@@ -575,7 +594,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
   NSString *lockClause = nil;
   NSArray *sortOrderings;
 
-  EOFLOGObjectFnStartOrCond(@"EOSQLExpression");
+  EOFLOGObjectFnStartCond(@"EOSQLExpression");
 
   // Turbocat (RawRow Additions)
   if ([fetchSpecification rawRowKeyPaths]) {
@@ -637,7 +656,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
 					  _orderByString : nil)
 			   lockClause:lockClause]);
 
-  EOFLOGObjectFnStopOrCond(@"EOSQLExpression");
+  EOFLOGObjectFnStopCond(@"EOSQLExpression");
 }
 */
 
@@ -658,7 +677,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
   //Add Attributes to listString
   int i, count = [attributes count];
 
-  EOFLOGObjectFnStart();
+  EOFLOGObjectFnStartCond(@"EOSQLExpression");
 
   //OK
   for (i = 0; i < count; i++)
@@ -763,7 +782,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
 		    lockClause: lockClauseString];
   ASSIGN(_statement, statement);
 
-  EOFLOGObjectFnStop();
+  EOFLOGObjectFnStopCond(@"EOSQLExpression");
 }
 
 - (NSString *)assembleJoinClauseWithLeftName: (NSString *)leftName
@@ -773,7 +792,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
   NSString *op = nil;
   NSString *joinClause = nil;
 
-  EOFLOGObjectFnStart();
+  EOFLOGObjectFnStartCond(@"EOSQLExpression");
 
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"join parts=%@ %d %@",
 			leftName,
@@ -808,7 +827,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
 
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"joinClause=%@", joinClause);
 
-  EOFLOGObjectFnStop();
+  EOFLOGObjectFnStopCond(@"EOSQLExpression");
 
   return joinClause;
 }
@@ -819,7 +838,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
 {
   NSString *joinClause = nil;
 
-  EOFLOGObjectFnStart();
+  EOFLOGObjectFnStartCond(@"EOSQLExpression");
 
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"join parts=%@ %d %@",
 			leftName,
@@ -845,7 +864,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"_joinClauseString=%@",
 			_joinClauseString);
 
-  EOFLOGObjectFnStop();
+  EOFLOGObjectFnStopCond(@"EOSQLExpression");
 }
 
 /** Build join expression for all used relationships (call this) after all other query parts construction) **/
@@ -855,7 +874,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
   NSEnumerator *relationshipEnum;
   NSString *relationshipPath;
 
-  EOFLOGObjectFnStart();
+  EOFLOGObjectFnStartCond(@"EOSQLExpression");
 
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"_aliasesByRelationshipPath=%@",
 			_aliasesByRelationshipPath);
@@ -949,7 +968,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
         }
     }
 
-  EOFLOGObjectFnStop();
+  EOFLOGObjectFnStopCond(@"EOSQLExpression");
 }
 
 - (NSString *)assembleInsertStatementWithRow: (NSDictionary *)row
@@ -998,7 +1017,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
 { //TODO selectString ??
   NSMutableString *sqlString;
 
-  EOFLOGObjectFnStart();
+  EOFLOGObjectFnStartCond(@"EOSQLExpression");
 
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"attributes=%@", attributes);
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"qualifier=%@", qualifier);
@@ -1037,7 +1056,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
     [sqlString appendFormat: @" ORDER BY %@", orderByClause];
 
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"sqlString=%@", sqlString);
-  EOFLOGObjectFnStop();
+  EOFLOGObjectFnStopCond(@"EOSQLExpression");
 
   return sqlString;
 }
@@ -1069,7 +1088,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
   NSMutableString *listString;
   NSString *attributeSQLString;
 
-  EOFLOGObjectFnStart();
+  EOFLOGObjectFnStartCond(@"EOSQLExpression");
 
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"attribute name=%@",
 			[attribute name]);
@@ -1113,7 +1132,8 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
       writeFormat = [attribute writeFormat];
       if ([writeFormat length] > 0)
         {
-          //TODO
+          NSEmitTODO();  //TODO
+          NSDebugMLog(@"writeFormat '%@' not yet handled",writeFormat);
         }
 
       valueList = [self valueList];
@@ -1127,7 +1147,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
     }
   NS_ENDHANDLER;
 
-  EOFLOGObjectFnStop();
+  EOFLOGObjectFnStopCond(@"EOSQLExpression");
 }
 
 - (void)addUpdateListAttribute: (EOAttribute *)attribute
@@ -1140,7 +1160,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
   NSString *valueSQLString;
   NSString *writeFormat;
 
-  EOFLOGObjectFnStart();
+  EOFLOGObjectFnStartCond(@"EOSQLExpression");
 
   attributeSQLString = [self sqlStringForAttribute: attribute];
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"attributeSQLString=%@",
@@ -1155,7 +1175,8 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
 
   if ([writeFormat length] > 0)
     {
-      //TODO
+      NSEmitTODO();  //TODO
+      NSDebugMLog(@"writeFormat '%@' not yet handled",writeFormat);
     }
 
   listString = [self listString];
@@ -1166,7 +1187,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
   [self appendItem: sqlStringToAdd
         toListString: listString];
 
-  EOFLOGObjectFnStop();
+  EOFLOGObjectFnStopCond(@"EOSQLExpression");
 }
 
 + (NSString *)formatStringValue: (NSString *)string
@@ -1192,7 +1213,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
 //mirko new:return [value sqlString];
   NSString *formattedValue = nil;
 
-  EOFLOGObjectFnStart();
+  EOFLOGObjectFnStartCond(@"EOSQLExpression");
 
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @" value=%@ class=%@",
 			value, [value class]);
@@ -1208,9 +1229,9 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
 	  string = [value sqlString];
 
 	  EOFLOGObjectLevelArgs(@"EOSQLExpression", @" value %p=%@ null %p=%@",
-				value, value, [EONull null], [EONull null]);
+				value, value, GDL2_EONull, GDL2_EONull);
 
-	  if ([value isEqual: [EONull null]])
+	  if (value == GDL2_EONull)
 	    formattedValue = string;
 	  else
 	    formattedValue = [self formatSQLString: [self formatStringValue:
@@ -1225,7 +1246,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
     }
   NS_ENDHANDLER;
 
-  EOFLOGObjectFnStop();
+  EOFLOGObjectFnStopCond(@"EOSQLExpression");
 
   return formattedValue;
 }
@@ -1235,7 +1256,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
 {
   NSString *formatted = nil;  
 
-  EOFLOGObjectFnStart();
+  EOFLOGObjectFnStartCond(@"EOSQLExpression");
 
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @" sqlString=%@ format=%@",
 			sqlString, format);
@@ -1251,25 +1272,26 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
 	  char *s;
 	  NSMutableString *str = [NSMutableString stringWithCapacity:
 						    [format length]];
+          IMP appendStringIMP = [str methodForSelector:@selector(appendString:)];
 
 	  while ((s = strchr(p, '%')))
 	    {
 	      switch (*(s + 1))
 		{
 		case '%':
-		  [str appendString: [NSString stringWithCString: p
-					       length: s-p+1]];
+		  GDL2_AppendStringWithImp(str,appendStringIMP,
+                                          GDL2_StringWithCStringAndLength(p,s-p+1));
 		  break;
 		case 'P':
 		  if (s != p)
-		    [str appendString: [NSString stringWithCString: p
-						 length: s-p]];
+		    GDL2_AppendStringWithImp(str,appendStringIMP,
+                                            GDL2_StringWithCStringAndLength(p,s-p));
 		  [str appendString: sqlString];
 		  break;
 		default:
 		  if (s != p)
-		    [str appendString: [NSString stringWithCString: p
-						 length: s-p]];
+		    GDL2_AppendStringWithImp(str,appendStringIMP,
+                                            GDL2_StringWithCStringAndLength(p,s-p));
 		  break;
 		}
 
@@ -1277,7 +1299,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
 	    }
 
 	  if (*p)
-	    [str appendString: [NSString stringWithCString: p]];
+	    GDL2_AppendStringWithImp(str,appendStringIMP,[NSString stringWithCString: p]);
 
 	  formatted = str;
 	}
@@ -1291,7 +1313,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
 
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @" formatted=%@", formatted);
 
-  EOFLOGObjectFnStop();
+  EOFLOGObjectFnStopCond(@"EOSQLExpression");
 
   return formatted;
 }
@@ -1305,7 +1327,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
   int i, count;
   int nb=0;
 
-  EOFLOGObjectFnStart();
+  EOFLOGObjectFnStartCond(@"EOSQLExpression");
 
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"operation=%@ qualifiers=%@",
                         operation, qualifiers);
@@ -1355,7 +1377,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"operation=%@ qualifiers=%@ count=%d nb=%d sqlString=%@",
                         operation, qualifiers, count, nb, sqlString);
 
-  EOFLOGObjectFnStop();
+  EOFLOGObjectFnStopCond(@"EOSQLExpression");
 
   return sqlString;
 }
@@ -1365,14 +1387,14 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
   //OK
   NSString *sqlString;
 
-  EOFLOGObjectFnStart();
+  EOFLOGObjectFnStartCond(@"EOSQLExpression");
 
   sqlString = [self sqlStringForArrayOfQualifiers: qualifiers
                     operation: @" AND "];
 
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"sqlString=%@", sqlString);
 
-  EOFLOGObjectFnStop();
+  EOFLOGObjectFnStopCond(@"EOSQLExpression");
 
   return sqlString;
 }
@@ -1382,14 +1404,14 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
   //OK
   NSString *sqlString;
 
-  EOFLOGObjectFnStart();
+  EOFLOGObjectFnStartCond(@"EOSQLExpression");
 
   sqlString = [self sqlStringForArrayOfQualifiers: qualifiers
 		    operation: @" OR "];
 
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"sqlString=%@", sqlString);
 
-  EOFLOGObjectFnStop();
+  EOFLOGObjectFnStopCond(@"EOSQLExpression");
 
   return sqlString;
 }
@@ -1398,7 +1420,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
 {
   NSString *sqlQual = nil;
 
-  EOFLOGObjectFnStart();
+  EOFLOGObjectFnStartCond(@"EOSQLExpression");
 
   sqlQual = [(id)qualifier sqlStringForSQLExpression: self];
   if (sqlQual)
@@ -1406,7 +1428,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
 
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"sqlQual=%@", sqlQual);
 
-  EOFLOGObjectFnStop();    
+  EOFLOGObjectFnStopCond(@"EOSQLExpression");    
 
   return sqlQual;
 }
@@ -1423,7 +1445,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
   EOAttribute* attribute=nil;
   NSString* readFormat=nil;
 
-  EOFLOGObjectFnStart();
+  EOFLOGObjectFnStartCond(@"EOSQLExpression");
 
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"qualifier=%@", qualifier);
 
@@ -1432,13 +1454,15 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
             [qualifier class],
             qualifier);
 
-  key = [qualifier key];//OK
+  key = [qualifier key];
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"key=%@", key);
+  NSAssert1([key length]>0,
+           @"No key in EOKeyValueQualifier: %@",qualifier);
 
-  value = [qualifier value];//OK
+  value = [qualifier value];
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"value=%@", value);
 
-  attributeSQLString = [self sqlStringForAttributeNamed: key];//OK
+  attributeSQLString = [self sqlStringForAttributeNamed: key];
 
   NSAssert1(attributeSQLString, @"No sqlStringForAttributeNamed:%@", key);
   EOFLOGObjectLevelArgs(@"EOSQLExpression",@"attributeSQLString=%@",
@@ -1446,17 +1470,18 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
 
   attribute = [_entity attributeForPath: key];
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"attribute=%@", attribute);
+  NSAssert3(attribute,
+            @"No attribute forKeyPath: '%@' in entity named '%@' in EOKeyValueQualifier: %@",
+            key,_entity,qualifier);
 
   readFormat = [attribute readFormat];
 
   if (readFormat)
     {
       NSEmitTODO();  //TODO
+      NSDebugMLog(@"readFormat '%@' not yet handled",readFormat);
     }
 
-  valueSQLString = [self sqlStringForValue: value
-			 attributeNamed: key];//OK
-
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"valueSQLString=%@ qualifier=%@ [qualifier selector]=%p %@",
 			valueSQLString,
                         qualifier,
@@ -1464,23 +1489,32 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
                         NSStringFromSelector([qualifier selector]));
 
   selectorSQLString = [self sqlStringForSelector: [qualifier selector]
-			    value: value];//OK //value ?? 
+			    value: value];
 
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"selectorSQLString=%@",
 			selectorSQLString);
 
-  //??
   if (sel_eq([qualifier selector], EOQualifierOperatorLike))
-    valueSQLString = [[self class] sqlPatternFromShellPattern: valueSQLString];
+    {
+      value = [[self class] sqlPatternFromShellPattern: value];
+      valueSQLString = [self sqlStringForValue: value
+                             attributeNamed: key];
+    }
   else if (sel_eq([qualifier selector], EOQualifierOperatorCaseInsensitiveLike))
     {      
-      valueSQLString = [[self class] sqlPatternFromShellPattern: valueSQLString];
-      //VERIFY
+      value = [[self class] sqlPatternFromShellPattern: value];
+
+      valueSQLString = [self sqlStringForValue: value
+                             attributeNamed: key];
+
       attributeSQLString = [NSString stringWithFormat: @"UPPER(%@)",
 				     attributeSQLString];
       valueSQLString = [NSString stringWithFormat: @"UPPER(%@)",
 				 valueSQLString];
     }
+  else
+    valueSQLString = [self sqlStringForValue: value
+                           attributeNamed: key];
 
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"attributeSQLString=%@",
 			attributeSQLString);
@@ -1506,7 +1540,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
 
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"sqlString=%@", sqlString);
 
-  EOFLOGObjectFnStop();
+  EOFLOGObjectFnStopCond(@"EOSQLExpression");
 
   return sqlString; //return someting like t1.label = 'XXX'
 }
@@ -1626,7 +1660,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
   EORelationship *rel = nil;
 
 
-  EOFLOGObjectFnStart();
+  EOFLOGObjectFnStartCond(@"EOSQLExpression");
 
   NSAssert(entity,@"no entity");
   NSAssert(name,@"no attribute name");
@@ -1713,7 +1747,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
                 attribute);
     }
 
-  EOFLOGObjectFnStop();
+  EOFLOGObjectFnStopCond(@"EOSQLExpression");
 
   return sqlString;
 }
@@ -1724,14 +1758,14 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
   //seems OK
   if (sel_eq(selector, EOQualifierOperatorEqual))
     {
-      if ([value isKindOfClass: [[EONull null] class]])
+      if (value==GDL2_EONull)
         return @"is";
       else
         return @"=";
     }
   else if (sel_eq(selector, EOQualifierOperatorNotEqual))
     {
-      if ([value isKindOfClass: [[EONull null] class]])
+      if (value==GDL2_EONull)
         return @"is not";
       else
         return @"<>";
@@ -1765,7 +1799,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
   EOAttribute *attribute;
   NSString *sqlString = nil;
 
-  EOFLOGObjectFnStart();
+  EOFLOGObjectFnStartCond(@"EOSQLExpression");
 
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"value=%@", value);
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"attributeName=%@",
@@ -1815,7 +1849,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
 				format: [attribute readFormat]];
     }
 
-  EOFLOGObjectFnStop();
+  EOFLOGObjectFnStopCond(@"EOSQLExpression");
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"sqlString=%@", sqlString);
 
   return sqlString;
@@ -1825,7 +1859,7 @@ NSString *EOBindVariableColumnKey = @"EOBindVariableColumnKey";
 {
   NSString *sqlString = nil;
 
-  EOFLOGObjectFnStart();
+  EOFLOGObjectFnStartCond(@"EOSQLExpression");
 
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"anAttribute=%@",
 			anAttribute);
@@ -1955,7 +1989,7 @@ else if([anAttribute isDerived] == YES)
     }
 
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"sqlString=%@", sqlString);
-  EOFLOGObjectFnStop();
+  EOFLOGObjectFnStopCond(@"EOSQLExpression");
 
   return sqlString;
 }
@@ -1964,7 +1998,7 @@ else if([anAttribute isDerived] == YES)
 {
   NSString *sqlString = nil;
 
-  EOFLOGObjectFnStart();
+  EOFLOGObjectFnStartCond(@"EOSQLExpression");
 
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"path=%@", path);
 
@@ -2026,7 +2060,7 @@ else if([anAttribute isDerived] == YES)
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"path=%@ sqlString=%@", 
                         path, sqlString);
 
-  EOFLOGObjectFnStop();
+  EOFLOGObjectFnStopCond(@"EOSQLExpression");
 
   return sqlString;
 }
@@ -2057,67 +2091,79 @@ else if([anAttribute isDerived] == YES)
 
 + (NSString *)sqlPatternFromShellPattern: (NSString *)pattern
 {
-  const char *s, *p, *init = [pattern cString];
-  NSMutableString *str = [NSMutableString stringWithCapacity:
-					    [pattern length]];
-
-  for (s = p = init; *s; s++)
+  NSString* sqlPattern=nil;
+  int patternLength=[pattern length];
+  if (patternLength==0)
+    sqlPattern=pattern;
+  else
     {
-      switch (*s)
+      const char *s, *p, *init = [pattern cString];
+      NSMutableString *str = [NSMutableString stringWithCapacity:
+                                                patternLength];
+      IMP appendStringIMP = [str methodForSelector:@selector(appendString:)];
+
+      for (s = p = init; *s; s++)
         {
-	case '*':
-	  if (s != p)
-	    [str appendString: [NSString stringWithCString: p
-					 length: s-p]];
-	  [str appendString: @"%"];
-	  p = s+1;
-	  break;
-	case '?':
-	  if (s != p)
-	    [str appendString:[NSString stringWithCString: p
-					length: s-p]];
-	  [str appendString: @"_"];
-	  p = s+1;
-	  break;
-	case '%':
-	  if (s != p)
-	    [str appendString:[NSString stringWithCString: p
-					length: s-p]];
-	  
-	  if (s != init && *(s-1) == '[' && *(s+1) == ']')
-	    {
-	      [str appendString: @"%]"];
-	      p = s+2; s++;
-	    }
-	  else
-	    {
-	      [str appendString: @"[%]"];
-	      p = s+1;
-	    }
-	  break;
-	case '_':
-	  if (s != p)
-	    [str appendString:[NSString stringWithCString: p
-					length: s-p]];
-	  
-	  if (s != init && *(s-1) == '[' && *(s+1) == ']')
-	    {
-	      [str appendString: @"_]"];
-	      p = s+2; p++;
-	    }
-	  else
-	    {
-	      [str appendString: @"[_]"];
-	      p = s+1;
-	    }
-	  break;
+          switch (*s)
+            {
+            case '*':
+              if (s != p)
+                GDL2_AppendStringWithImp(str,appendStringIMP,
+                                        GDL2_StringWithCStringAndLength(p,s-p));
+              [str appendString: @"%"];
+              p = s+1;
+              break;
+            case '?':
+              if (s != p)
+                GDL2_AppendStringWithImp(str,appendStringIMP,
+                                        GDL2_StringWithCStringAndLength(p,s-p));
+              GDL2_AppendStringWithImp(str,appendStringIMP,@"_");
+              p = s+1;
+              break;
+            case '%':
+              if (s != p)
+                GDL2_AppendStringWithImp(str,appendStringIMP,
+                                        GDL2_StringWithCStringAndLength(p,s-p));
+              
+              if (s != init && *(s-1) == '[' && *(s+1) == ']')
+                {
+                  GDL2_AppendStringWithImp(str,appendStringIMP,@"%]");
+                  p = s+2; s++;
+                }
+              else
+                {
+                  GDL2_AppendStringWithImp(str,appendStringIMP,@"[%]");
+                  p = s+1;
+                }
+              break;
+            case '_':
+              if (s != p)
+                GDL2_AppendStringWithImp(str,appendStringIMP,
+                                        GDL2_StringWithCStringAndLength(p,s-p));
+              
+              if (s != init && *(s-1) == '[' && *(s+1) == ']')
+                {
+                  GDL2_AppendStringWithImp(str,appendStringIMP,@"_]");
+                  p = s+2; p++;
+                }
+              else
+                {
+                  GDL2_AppendStringWithImp(str,appendStringIMP,@"[_]");
+                  p = s+1;
+                }
+              break;
+            }
         }
-    }
+      
+      if (*p)
+        GDL2_AppendStringWithImp(str,appendStringIMP,[NSString stringWithCString: p]);
+      sqlPattern=str;
+    };
 
-  if (*p)
-    [str appendString: [NSString stringWithCString: p]];
+  EOFLOGObjectLevelArgs(@"EOSQLExpression", @"pattern=%@ => %@",
+			pattern,sqlPattern);
 
-  return str;
+  return sqlPattern;
 }
 
 + (NSString *)sqlPatternFromShellPattern: (NSString *)pattern
@@ -2126,6 +2172,7 @@ else if([anAttribute isDerived] == YES)
   const char *s, *p, *init = [pattern cString];
   NSMutableString *str = [NSMutableString stringWithCapacity:
 					    [pattern length]];
+  IMP appendStringIMP = [str methodForSelector:@selector(appendString:)];
 
   for (s = p = init; *s; s++)
     {
@@ -2133,47 +2180,47 @@ else if([anAttribute isDerived] == YES)
         {
 	case '*':
 	  if (s != p)
-	    [str appendString: [NSString stringWithCString: p
-					 length: s-p]];
-	  [str appendString: @"%"];
+	    GDL2_AppendStringWithImp(str,appendStringIMP,
+                                    GDL2_StringWithCStringAndLength(p,s-p));
+	  GDL2_AppendStringWithImp(str,appendStringIMP,@"%");
 	  p = s+1;
 	  break;
 	case '?':
 	  if (s != p)
-	    [str appendString: [NSString stringWithCString: p
-					 length: s-p]];
-	  [str appendString: @"_"];
+	    GDL2_AppendStringWithImp(str,appendStringIMP,
+                                    GDL2_StringWithCStringAndLength(p,s-p));
+	  GDL2_AppendStringWithImp(str,appendStringIMP,@"_");
 	  p = s+1;
 	  break;
 	case '%':
 	  if (s != p)
-	    [str appendString:[NSString stringWithCString: p
-					length: s-p]];
+	    GDL2_AppendStringWithImp(str,appendStringIMP,
+                                    GDL2_StringWithCStringAndLength(p,s-p));
 	  
 	  if (s != init && *(s-1) == '[' && *(s+1) == ']')
 	    {
-	      [str appendString: @"%]"];
+	      GDL2_AppendStringWithImp(str,appendStringIMP,@"%]");
 	      p = s+2; s++;
 	    }
 	  else
 	    {
-	      [str appendString: @"[%]"];
+	      GDL2_AppendStringWithImp(str,appendStringIMP,@"[%]");
 	      p = s+1;
 	    }
 	  break;
 	case '_':
 	  if (s != p)
-	    [str appendString:[NSString stringWithCString: p
-					length: s-p]];
+	    GDL2_AppendStringWithImp(str,appendStringIMP,
+                                    GDL2_StringWithCStringAndLength(p,s-p));
 	  
 	  if (s != init && *(s-1) == '[' && *(s+1) == ']')
 	    {
-	      [str appendString: @"_]"];
+	      GDL2_AppendStringWithImp(str,appendStringIMP,@"_]");
 	      p = s+2; p++;
 	    }
 	  else
 	    {
-	      [str appendString: @"[_]"];
+	      GDL2_AppendStringWithImp(str,appendStringIMP,@"[_]");
 	      p = s+1;
 	    }
 	  break;
@@ -2181,7 +2228,7 @@ else if([anAttribute isDerived] == YES)
     }
 
   if (*p)
-    [str appendString:[NSString stringWithCString:p]];
+    GDL2_AppendStringWithImp(str,appendStringIMP,[NSString stringWithCString:p]);
 
   return str;
 }
@@ -2241,7 +2288,7 @@ else if([anAttribute isDerived] == YES)
 This add a new alias if there not already one
 This also add alias for all relationships used by relationshipPath (so joinExpression can build join expressions for really all used relationships)
 All relationshipPaths in _aliasesByRelationshipPath are direct paths **/
-- (NSString*) _aliasForRelationshipPath:(NSString*)relationshipPath//toLanguage
+- (NSString*) _aliasForRelationshipPath:(NSString*)relationshipPath
 {
   //OK ?
   NSString *flattenRelPath;
@@ -2251,7 +2298,7 @@ All relationshipPaths in _aliasesByRelationshipPath are direct paths **/
   int count = 0;
   int contextStackCurrentIndex = 0;
   
-  EOFLOGObjectFnStart();
+  EOFLOGObjectFnStartCond(@"EOSQLExpression");
 
   contextStackCurrentIndex = [_contextStack count];
 
@@ -2327,7 +2374,7 @@ All relationshipPaths in _aliasesByRelationshipPath are direct paths **/
     }
 
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"alias=%@", alias);
-  EOFLOGObjectFnStop();
+  EOFLOGObjectFnStopCond(@"EOSQLExpression");
 
   return alias;
 }
@@ -2342,7 +2389,7 @@ All relationshipPaths in _aliasesByRelationshipPath are direct paths **/
   NSArray *pathElements = nil;
   int i, count;
 
-  EOFLOGObjectFnStart();
+  EOFLOGObjectFnStartCond(@"EOSQLExpression");
 
   NSAssert(relationshipPath, @"No relationshipPath");
   NSAssert([relationshipPath length] > 0, @"Empty relationshipPath");
@@ -2397,7 +2444,7 @@ All relationshipPaths in _aliasesByRelationshipPath are direct paths **/
 
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"flattenRelPath=%@",
 			flattenRelPath);
-  EOFLOGObjectFnStop();
+  EOFLOGObjectFnStopCond(@"EOSQLExpression");
 
   return flattenRelPath;
 }
@@ -2416,7 +2463,7 @@ All relationshipPaths in _aliasesByRelationshipPath are direct paths **/
   NSString *relPathAlias = nil;
   NSString *attributeColumnName = nil;
 
-  EOFLOGObjectFnStart();
+  EOFLOGObjectFnStartCond(@"EOSQLExpression");
 
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"attribute=%@", attribute);
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"relationshipPath=%@",
@@ -2437,7 +2484,7 @@ All relationshipPaths in _aliasesByRelationshipPath are direct paths **/
 		    attributeColumnName];
 
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"alias=%@", alias);
-  EOFLOGObjectFnStop();
+  EOFLOGObjectFnStopCond(@"EOSQLExpression");
 
   return alias;//Like t1.label
 }
@@ -2492,7 +2539,7 @@ NSString *EODropDatabaseKey = @"EODropDatabaseKey";
 @implementation EOSQLExpression (EOSchemaGeneration)
 
 + (NSArray *)_administrativeDatabaseStatementsForSelector:(SEL) sel
-					    forEntityGroup:(NSArray *)group
+					   forEntityGroup:(NSArray *)group
 {
   EOEntity     *entity;
   EOModel      *model;
@@ -2512,7 +2559,7 @@ NSString *EODropDatabaseKey = @"EODropDatabaseKey";
   [[NSNotificationCenter defaultCenter] postNotificationName: notifName
 					object: notifDict];
   admDict = [notifDict objectForKey: EOAdministrativeConnectionDictionaryKey];
-
+/* TODO: ayers 
   if (admDict == nil && [admDict count] == 0)
     {
       EOAdaptor    *adaptor;
@@ -2522,7 +2569,7 @@ NSString *EODropDatabaseKey = @"EODropDatabaseKey";
       panel = [[adaptor class] sharedLoginPanelInstance];
       admDict = [panel administrativeConnectionDictionaryForAdaptor: adaptor];
     }
-
+*/
   stmts = [self performSelector: sel 
 		withObject: connDict 
 		withObject: admDict];
