@@ -1,7 +1,7 @@
 /* -*-objc-*-
    EOSQLExpression.h
 
-   Copyright (C) 2002 Free Software Foundation, Inc.
+   Copyright (C) 2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: Mars 2002
@@ -21,7 +21,7 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 #ifndef __EOSQLExpressionPriv_h__
@@ -37,13 +37,13 @@
 @interface EOSQLExpression (EOSQLExpressionPrivate)
 
 - (EOEntity *)_rootEntityForExpression;
-- (id)_aliasForRelationshipPath: (id)param0;
-- (id)_flattenRelPath: (id)param0
-	       entity: (id)param1;
+- (NSString*) _aliasForRelationshipPath:(NSString*)relationshipPath;
+- (NSString*) _flattenRelPath: (NSString*)relationshipPath
+                       entity: (EOEntity*)entity;
 - (NSString *)_sqlStringForJoinSemantic: (EOJoinSemantic)joinSemantic
 			  matchSemantic: (int)param1;
-- (id)_aliasForRelatedAttribute: (id)param0
-	       relationshipPath: (id)param1;
+- (NSString*) _aliasForRelatedAttribute: (EOAttribute*)attribute
+                       relationshipPath: (NSString*)relationshipPath;
 - (id)_entityForRelationshipPath: (id)param0
 			  origin: (id)param1;
 
