@@ -1,13 +1,13 @@
 /** 
    EOSQLQualifier.m <title>EOSQLQualifier Class</title>
 
-   Copyright (C) 2002 Free Software Foundation, Inc.
+   Copyright (C) 2002,2003,2004,2005 Free Software Foundation, Inc.
 
-   Author: Manuel Guesdon <mguesdon@orange-concept.co>
+   Author: Manuel Guesdon <mguesdon@orange-concept.com>
    Date: February 2002
 
-   $Revision: 1.12 $
-   $Date: 2004/03/19 16:41:19 $
+   $Revision: 21682 $
+   $Date: 2005-08-25 20:32:44 +0200 (Don, 25 Aug 2005) $
 
    <abstract></abstract>
 
@@ -27,13 +27,13 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
 #include "config.h"
 
-RCS_ID("$Id: EOSQLQualifier.m,v 1.12 2004/03/19 16:41:19 ayers Exp $")
+RCS_ID("$Id: EOSQLQualifier.m 21682 2005-08-25 18:32:44Z mguesdon $")
 
 #include <stdio.h>
 #include <string.h>
@@ -66,6 +66,8 @@ RCS_ID("$Id: EOSQLQualifier.m,v 1.12 2004/03/19 16:41:19 ayers Exp $")
 #include <EOControl/EONull.h>
 #include <EOControl/EODebug.h>
 
+#include "EOPrivate.h"
+#include "EOEntityPriv.h"
 
 @implementation EOSQLQualifier
 
@@ -159,12 +161,22 @@ RCS_ID("$Id: EOSQLQualifier.m,v 1.12 2004/03/19 16:41:19 ayers Exp $")
           if (schemaBasedQualifierTmp != qualifier)
             atLeastOneDifferentQualifier = YES;
 
-	  [qualifiers addObject: schemaBasedQualifierTmp];
+          // Allows nil schemaBasedQualifier
+          if (schemaBasedQualifierTmp)
+            [qualifiers addObject: schemaBasedQualifierTmp];
 	}
 
       // If we've found at least a different qualifier, return a new EOAndQualifier
       if (atLeastOneDifferentQualifier)
-          returnedQualifier = [[self class] qualifierWithQualifierArray:qualifiers];
+        {
+          if ([qualifiers count]>0)
+            {
+              returnedQualifier = [[self class] 
+                                    qualifierWithQualifierArray:qualifiers];
+            }
+          else
+            returnedQualifier = nil;
+        };
     }
 
   EOFLOGObjectFnStop();
@@ -232,12 +244,22 @@ RCS_ID("$Id: EOSQLQualifier.m,v 1.12 2004/03/19 16:41:19 ayers Exp $")
           if (schemaBasedQualifierTmp != qualifier)
             atLeastOneDifferentQualifier = YES;
 
-	  [qualifiers addObject: schemaBasedQualifierTmp];
+          // Allows nil schemaBasedQualifier
+          if (schemaBasedQualifierTmp)
+            [qualifiers addObject: schemaBasedQualifierTmp];
 	}
 
       // If we've found at least a different qualifier, return a new EOOrQualifier
       if (atLeastOneDifferentQualifier)
-          returnedQualifier = [[self class] qualifierWithQualifierArray:qualifiers];
+        {
+          if ([qualifiers count]>0)
+            {
+              returnedQualifier = [[self class] 
+                                    qualifierWithQualifierArray:qualifiers];
+            }
+          else
+            returnedQualifier = nil;
+        };
     }
 
   EOFLOGObjectFnStop();
@@ -256,15 +278,24 @@ RCS_ID("$Id: EOSQLQualifier.m,v 1.12 2004/03/19 16:41:19 ayers Exp $")
 
 - (EOQualifier *)schemaBasedQualifierWithRootEntity: (EOEntity *)entity
 {
-  //TODO
-  [self notImplemented: _cmd];
-  return nil;
+  return self; // MG: Not sure
 }
 
 @end
 
 @implementation EOKeyValueQualifier (EOQualifierSQLGeneration)
 
++ (void)initialize
+{
+  static BOOL initialized=NO;
+  if (!initialized)
+    {
+      initialized=YES;
+
+      GDL2_EOAccessPrivateInit();
+    };
+};
+
 - (NSString *)sqlStringForSQLExpression: (EOSQLExpression *)sqlExpression
 {
   return [sqlExpression sqlStringForKeyValueQualifier: self];
@@ -279,12 +310,20 @@ RCS_ID("$Id: EOSQLQualifier.m,v 1.12 2004/03/19 16:41:19 ayers Exp $")
 
   EOFLOGObjectFnStart();
 
+  EOFLOGObjectLevelArgs(@"EOQualifier", @"self=%@", self);
+
   key = [self key];
   EOFLOGObjectLevelArgs(@"EOQualifier", @"key=%@", key);
+  
+  // 2 cases: key finish by an attrbue name  (attrName or rel1.rel2.rel3.attrName)
+  // or by an relationship  (rel1 or rel1.rel2.rel3)
+
+  // So find which one is it for key
 
   relationship = [entity relationshipForPath: key];
   EOFLOGObjectLevelArgs(@"EOQualifier", @"relationship=%@", relationship);
 
+  // It's a relationship (case 2), so we'll have to work
   if (relationship)
     {
       EORelationship *destinationRelationship;
@@ -297,11 +336,46 @@ RCS_ID("$Id: EOSQLQualifier.m,v 1.12 2004/03/19 16:41:19 ayers Exp $")
       int i, count;
       SEL sel = NULL;
 
+      // keyPrefix for new qualifier attribute names
+      NSString* keyPrefix=nil; 
+
+      NSString* relName=[relationship name];
+
+      // Verify if key is a single relationship or a relationship key path
+      if (![key isEqualToString:relName])
+        {
+          // It is a relationship key path: we'll have to prefix join(s) 
+          // attribute name
+          // keyPrefix is the keyPath without last relationship name
+          // ex: rel1.rel2. if key was rel1.rel2.rel3
+          keyPrefix=[key stringByDeletingSuffix:relName];
+        };
+
+      // if relationship is flattened, we'll have to add 
+      // last relationship path prefix to keyPrefix !
       if ([relationship isFlattened])
-        destinationRelationship = [relationship lastRelationship];
+        {
+          NSString* relDef=nil;
+          destinationRelationship = [relationship lastRelationship];
+          
+          relDef=[relationship definition];
+
+          // something like rel1.rel2.relA.relB. or relA.relB.
+          if (keyPrefix)
+            keyPrefix=[keyPrefix stringByAppendingString:relDef];
+          else
+            keyPrefix=relDef;
+
+          keyPrefix=[keyPrefix stringByAppendingString:@"."];
+        }
       else
-        destinationRelationship = relationship;
-      
+        {
+          destinationRelationship = relationship;
+        };
+
+      EOFLOGObjectLevelArgs(@"EOQualifier", @"key=%@ keyPrefix=%@", 
+                            key, keyPrefix);
+
       joins = [destinationRelationship joins];
       count = [joins count];
 
@@ -332,11 +406,6 @@ RCS_ID("$Id: EOSQLQualifier.m,v 1.12 2004/03/19 16:41:19 ayers Exp $")
       EOFLOGObjectLevelArgs(@"EOQualifier", @"keyValues=%@", keyValues);
 
       sel = [self selector];
-      /*
-when flattened: ???
-             entity relationshipForPath:key
-             and get joins on it ?
-      */
 
       for (i = 0; i < count; i++)
         {
@@ -345,7 +414,6 @@ when flattened: ???
           NSString *destinationAttributeName;
           EOJoin *join = [joins objectAtIndex: i];
           id attributeValue = nil;
-          EONull *eoNull=[EONull null];
 
           EOFLOGObjectLevelArgs(@"EOQualifier",@"join=%@",join);
 
@@ -355,9 +423,7 @@ when flattened: ???
           if (destinationRelationship != relationship)
             {
               // flattened: take destattr
-              attributeName = [NSString stringWithFormat: @"%@.%@",
-					key, destinationAttributeName];
-              //==> rel.attr
+              attributeName = destinationAttributeName;
             }
           else
             {
@@ -366,11 +432,27 @@ when flattened: ???
               attributeName = [sourceAttribute name];
             }
 
+          if (keyPrefix)
+            attributeName=[keyPrefix stringByAppendingString:attributeName];
+
+          EOFLOGObjectLevelArgs(@"EOQualifier", 
+                                @"key=%@ keyPrefix=%@ attributeName=%@", 
+                                key, keyPrefix,attributeName);
+
           attributeValue = [keyValues objectForKey:destinationAttributeName];
+
+          EOFLOGObjectLevelArgs(@"EOQualifier", 
+                                @"destinationAttributeName=%@ attributeValue=%@", 
+                                destinationAttributeName, attributeValue);
+
           tmpQualifier = [EOKeyValueQualifier
 			   qualifierWithKey: attributeName
 			   operatorSelector: sel
-			   value: (attributeValue ? attributeValue : eoNull)];
+			   value: (attributeValue ? attributeValue : GDL2_EONull)];
+
+          EOFLOGObjectLevelArgs(@"EOQualifier", 
+                                @"tmpQualifier=%@", 
+                                tmpQualifier);
 
           if (qualifier)//Already a qualifier
             {
@@ -392,9 +474,12 @@ when flattened: ???
           qualifier = [EOAndQualifier qualifierWithQualifierArray: qualifiers];
         }
     }
-  else
+  else // It's not a relationship. Nothing to do.
     qualifier = self;
 
+  EOFLOGObjectLevelArgs(@"EOQualifier", @"self=%@", self);
+  EOFLOGObjectLevelArgs(@"EOQualifier", @"result qualifier=%@", qualifier);
+
   EOFLOGObjectFnStop();
 
   return qualifier;
