@@ -1,13 +1,13 @@
 /** 
    EOStoredProcedure.m <title>EOStoredProcedure Class</title>
 
-   Copyright (C) 2000 Free Software Foundation, Inc.
+   Copyright (C) 2000,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: February 2000
 
-   $Revision: 1.10 $
-   $Date: 2004/03/19 16:41:19 $
+   $Revision: 21645 $
+   $Date: 2005-08-17 10:07:57 +0200 (Mit, 17 Aug 2005) $
 
    <abstract></abstract>
 
@@ -27,13 +27,13 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
 #include "config.h"
 
-RCS_ID("$Id: EOStoredProcedure.m,v 1.10 2004/03/19 16:41:19 ayers Exp $")
+RCS_ID("$Id: EOStoredProcedure.m 21645 2005-08-17 08:07:57Z ayers $")
 
 #ifdef GNUSTEP
 #include <Foundation/NSException.h>
@@ -50,6 +50,7 @@ RCS_ID("$Id: EOStoredProcedure.m,v 1.10 2004/03/19 16:41:19 ayers Exp $")
 #include <GNUstepBase/GCObject.h>
 
 #include <EOControl/EODebug.h>
+#include <EOControl/EOObserver.h>
 
 #include <EOAccess/EOStoredProcedure.h>
 #include <EOAccess/EOAttribute.h>
@@ -102,7 +103,7 @@ RCS_ID("$Id: EOStoredProcedure.m,v 1.10 2004/03/19 16:41:19 ayers Exp $")
 			owner: owner] autorelease];
 }
 
-- initWithPropertyList: (NSDictionary *)propertyList owner: (id)owner
+- (id)initWithPropertyList: (NSDictionary *)propertyList owner: (id)owner
 {
   NSArray *array;
   NSEnumerator *enumerator;
@@ -125,10 +126,10 @@ RCS_ID("$Id: EOStoredProcedure.m,v 1.10 2004/03/19 16:41:19 ayers Exp $")
       enumerator = [array objectEnumerator];
       while ((attributePList = [enumerator nextObject]))
         {
-	  EOAttribute *attribute = [EOAttribute
-				     attributeWithPropertyList: attributePList
-				     owner: self];
-
+	  EOAttribute *attribute 
+	    = [EOAttribute attributeWithPropertyList: attributePList
+			   owner: self];
+	  [attribute awakeWithPropertyList: attributePList];
 	  [(GCMutableArray *)_arguments addObject: attribute];
         }
     }
@@ -148,7 +149,51 @@ RCS_ID("$Id: EOStoredProcedure.m,v 1.10 2004/03/19 16:41:19 ayers Exp $")
 
 - (void)encodeIntoPropertyList: (NSMutableDictionary *)propertyList
 {
-  return;
+  unsigned i, count;
+
+  if (_name)
+    {
+      [propertyList setObject: _name forKey: @"name"];
+    }
+
+  if (_externalName)
+    {
+      [propertyList setObject: _externalName forKey: @"externalName"];
+    }
+
+  if (_userInfo)
+    {
+      [propertyList setObject: _userInfo forKey: @"userInfo"];
+    }
+
+  if ((count = [_arguments count]))
+    {
+      NSMutableArray *attributesPList 
+	= [NSMutableArray arrayWithCapacity: count];
+
+      for (i = 0; i < count; i++)
+	{
+	  NSMutableDictionary *attributePList 
+	    = [NSMutableDictionary dictionary];
+	  EOAttribute *attribute
+	    = [_arguments objectAtIndex: i];
+
+	  [attribute encodeIntoPropertyList: attributePList];
+	  [attributesPList addObject: attributePList];
+	}
+
+      [propertyList setObject: attributesPList forKey: @"attributes"];
+    }
+}
+
+- (NSString*) description
+{
+  NSMutableDictionary *plist;
+
+  plist = [NSMutableDictionary dictionaryWithCapacity: 6];
+  [self encodeIntoPropertyList: plist];
+
+  return [plist description];
 }
 
 - (NSString *)name
@@ -178,16 +223,19 @@ RCS_ID("$Id: EOStoredProcedure.m,v 1.10 2004/03/19 16:41:19 ayers Exp $")
 
 - (void)setName: (NSString *)name
 {
+  [self willChange];
   ASSIGN(_name, name);
 }
 
 - (void)setExternalName: (NSString *)name
 {
+  [self willChange];
   ASSIGN(_externalName, name);
 }
 
 - (void)setArguments: (NSArray *)arguments
 {
+  [self willChange];
   if ([arguments isKindOfClass: [GCArray class]]
       || [arguments isKindOfClass: [GCMutableArray class]])
     ASSIGN(_arguments, arguments);
@@ -197,6 +245,7 @@ RCS_ID("$Id: EOStoredProcedure.m,v 1.10 2004/03/19 16:41:19 ayers Exp $")
 
 - (void)setUserInfo: (NSDictionary *)dictionary
 {
+  [self willChange];
   ASSIGN(_userInfo, dictionary);
 }
 
@@ -207,23 +256,26 @@ RCS_ID("$Id: EOStoredProcedure.m,v 1.10 2004/03/19 16:41:19 ayers Exp $")
 
 - (void)beautifyName
 {
-  NSArray	*listItems;
-  NSString	*newString = [NSMutableString string];
-  int		 anz, i;
+  NSArray  *listItems;
+  NSString *newString = [NSMutableString string];
+  NSString *tmpString;
+  unsigned  anz, i;
   
   EOFLOGObjectFnStartOrCond2(@"ModelingClasses", @"EOStoredProcedure");
   
   if ((_name) && ([_name length] > 0))
     {
       listItems = [_name componentsSeparatedByString: @"_"];
-      newString = [newString stringByAppendingString: [[listItems objectAtIndex: 0]
-							lowercaseString]];
+      tmpString = [listItems objectAtIndex: 0];
+      tmpString = [tmpString lowercaseString];
+      newString = [newString stringByAppendingString: tmpString];
       anz = [listItems count];
 
       for (i = 1; i < anz; i++)
 	{
-	  newString = [newString stringByAppendingString:
-				   [[listItems objectAtIndex: i] capitalizedString]];
+	  tmpString = [listItems objectAtIndex: i];
+	  tmpString = [tmpString capitalizedString];
+	  newString = [newString stringByAppendingString: tmpString];
 	}
  
     NS_DURING
@@ -238,3 +290,9 @@ RCS_ID("$Id: EOStoredProcedure.m,v 1.10 2004/03/19 16:41:19 ayers Exp $")
 }
 
 @end
+
+@implementation EOStoredProcedure (privat)
+- (void)_setIsEdited
+{
+}
+@end
