@@ -1,13 +1,13 @@
 /** 
    EOUtilities.m
 
-   Copyright (C) 2000-2002 Free Software Foundation, Inc.
+   Copyright (C) 2000-2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Manuel Guesdon <mguesdon@orange-concept.com>
    Date: Sep 2000
 
-   $Revision: 1.9 $
-   $Date: 2004/03/19 16:41:19 $
+   $Revision: 23472 $
+   $Date: 2006-09-12 21:36:24 +0200 (Die, 12 Sep 2006) $
 
    <abstract></abstract>
 
@@ -27,13 +27,13 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
 #include "config.h"
 
-RCS_ID("$Id: EOUtilities.m,v 1.9 2004/03/19 16:41:19 ayers Exp $")
+RCS_ID("$Id: EOUtilities.m 23472 2006-09-12 19:36:24Z ayers $")
 
 #ifdef GNUSTEP
 #include <Foundation/NSString.h>
@@ -73,14 +73,15 @@ RCS_ID("$Id: EOUtilities.m,v 1.9 2004/03/19 16:41:19 ayers Exp $")
 
 #include <EOAccess/EOUtilities.h>
 
+#include "EOPrivate.h"
 
-static NSString *EOMoreThanOneException = @"EOMoreThanOneException";
-//static NSString *NSObjectNotAvailableException = @"NSObjectNotAvailableException";
+NSString *EOMoreThanOneException = @"EOMoreThanOneException";
+NSString *NSObjectNotAvailableException = @"NSObjectNotAvailableException";
 
 
 @implementation EOEditingContext (EOUtilities)
 
-- (NSArray*)objectsForEntityNamed: (NSString*)entityName
+- (NSArray *)objectsForEntityNamed: (NSString *)entityName
 {
   NSArray *objects;
   EOFetchSpecification *fetchSpecif;
@@ -96,7 +97,7 @@ static NSString *EOMoreThanOneException = @"EOMoreThanOneException";
   return objects;
 }
 
-- (NSArray*)objectsOfClass: (Class)classObject
+- (NSArray *)objectsOfClass: (Class)classObject
 {
   EOEntity *entity;
   NSArray *objects;
@@ -111,9 +112,9 @@ static NSString *EOMoreThanOneException = @"EOMoreThanOneException";
   return objects;
 }
 
-- (NSArray*)objectsWithFetchSpecificationNamed: (NSString*)fetchSpecName
-				   entityNamed: (NSString*)entityName
-				      bindings: (NSDictionary*)bindings
+- (NSArray *)objectsWithFetchSpecificationNamed: (NSString *)fetchSpecName
+				    entityNamed: (NSString *)entityName
+				       bindings: (NSDictionary *)bindings
 {
   EOModelGroup *modelGroup;
   EOFetchSpecification *unboundFetchSpec;
@@ -126,7 +127,7 @@ static NSString *EOMoreThanOneException = @"EOMoreThanOneException";
 
   if ( !unboundFetchSpec ) 
     {
-      [NSException raise: /*NSObjectNotAvailableException*/NSInvalidArgumentException 
+      [NSException raise: NSObjectNotAvailableException
                    format: @"%@: Fetch specification '%@' not found in entity named '%@'", 
                    NSStringFromSelector(_cmd), fetchSpecName, entityName];
     }
@@ -138,8 +139,8 @@ static NSString *EOMoreThanOneException = @"EOMoreThanOneException";
   return results;
 }
 
-- (NSArray*)objectsForEntityNamed: (NSString*)entityName
-		  qualifierFormat: (NSString*)format,...
+- (NSArray *)objectsForEntityNamed: (NSString *)entityName
+		   qualifierFormat: (NSString *)format,...
 {
   EOQualifier *qualifier;
   EOFetchSpecification *fetchSpec;
@@ -160,9 +161,9 @@ static NSString *EOMoreThanOneException = @"EOMoreThanOneException";
   return results;
 }
 
-- (NSArray*)objectsMatchingValue: (id)value
-			  forKey: (NSString*)key
-		     entityNamed: (NSString*)entityName
+- (NSArray *)objectsMatchingValue: (id)value
+			   forKey: (NSString *)key
+		      entityNamed: (NSString *)entityName
 {
   //OK
   NSArray *objects;
@@ -172,7 +173,7 @@ static NSString *EOMoreThanOneException = @"EOMoreThanOneException";
 	       value, key, entityName);
 
   if (!value)
-    value=[EONull null];
+    value=GDL2_EONull;
 
   NSAssert(value, @"No Value"); //Transform it to EONull ?
   NSAssert(key, @"No Key");
@@ -211,8 +212,8 @@ static NSString *EOMoreThanOneException = @"EOMoreThanOneException";
 */
 }
 
-- (NSArray*)objectsMatchingValues: (NSDictionary*)values
-		      entityNamed: (NSString*)entityName
+- (NSArray *)objectsMatchingValues: (NSDictionary *)values
+		       entityNamed: (NSString *)entityName
 {
   //OK
   NSArray *objects = nil;
@@ -285,9 +286,9 @@ static NSString *EOMoreThanOneException = @"EOMoreThanOneException";
 */
 }
 
-- (id)objectWithFetchSpecificationNamed: (NSString*)fetchSpecName
-			    entityNamed: (NSString*)entityName
-			       bindings: (NSDictionary*)bindings
+- (id)objectWithFetchSpecificationNamed: (NSString *)fetchSpecName
+			    entityNamed: (NSString *)entityName
+			       bindings: (NSDictionary *)bindings
 {  
   id object = nil;
   int count;
@@ -326,8 +327,8 @@ static NSString *EOMoreThanOneException = @"EOMoreThanOneException";
   return object;
 }
 
-- (id)objectForEntityNamed: (NSString*)entityName
-	   qualifierFormat: (NSString*)format,...
+- (id)objectForEntityNamed: (NSString *)entityName
+	   qualifierFormat: (NSString *)format,...
 {
   id object = nil;
   int count;
@@ -373,8 +374,8 @@ static NSString *EOMoreThanOneException = @"EOMoreThanOneException";
 }
 
 - (id)objectMatchingValue: (id)value
-		   forKey: (NSString*)key
-	      entityNamed: (NSString*)entityName
+		   forKey: (NSString *)key
+	      entityNamed: (NSString *)entityName
 {
   //OK
   id object = nil;
@@ -402,7 +403,7 @@ static NSString *EOMoreThanOneException = @"EOMoreThanOneException";
       switch (count)
         {
         case 0:
-          [NSException raise: NSInvalidArgumentException
+          [NSException raise: NSObjectNotAvailableException
 		       format: @"%@: No %@ found with key %@ matching %@",
                        NSStringFromSelector(_cmd),
                        entityName,
@@ -441,8 +442,8 @@ static NSString *EOMoreThanOneException = @"EOMoreThanOneException";
   return object;
 }
 
-- (id)objectMatchingValues: (NSDictionary*)values
-	       entityNamed: (NSString*)entityName
+- (id)objectMatchingValues: (NSDictionary *)values
+	       entityNamed: (NSString *)entityName
 {
   id object = nil;
   int count;
@@ -483,7 +484,7 @@ static NSString *EOMoreThanOneException = @"EOMoreThanOneException";
 }
 
 - (id)objectWithPrimaryKeyValue: (id)value
-		    entityNamed: (NSString*)entityName
+		    entityNamed: (NSString *)entityName
 {
   //OK
   id object = nil;
@@ -513,7 +514,7 @@ static NSString *EOMoreThanOneException = @"EOMoreThanOneException";
         {
           NSDictionary* pk;
           if (!value)
-            value=[EONull null];
+            value=GDL2_EONull;
 
           pk = [NSDictionary dictionaryWithObject: value
                              forKey: [(EOAttribute*)[primaryKeyAttributes
@@ -528,8 +529,8 @@ static NSString *EOMoreThanOneException = @"EOMoreThanOneException";
   return object;
 }
 
-- (id)objectWithPrimaryKey: (NSDictionary*)pkDict
-	       entityNamed: (NSString*)entityName
+- (id)objectWithPrimaryKey: (NSDictionary *)pkDict
+	       entityNamed: (NSString *)entityName
 {
   //OK
   id object = nil;
@@ -556,8 +557,8 @@ static NSString *EOMoreThanOneException = @"EOMoreThanOneException";
   return object;
 }
 
-- (id)rawRowsForEntityNamed: (NSString*)entityName
-	    qualifierFormat: (NSString*)format,...
+- (NSArray *)rawRowsForEntityNamed: (NSString *)entityName
+		   qualifierFormat: (NSString *)format,...
 {
   EOQualifier *qualifier;
   EOFetchSpecification *fetchSpec;
@@ -582,9 +583,9 @@ static NSString *EOMoreThanOneException = @"EOMoreThanOneException";
   return results;
 }
 
-- (id)rawRowsMatchingValue: (id)value
-		    forKey: (NSString*)key
-	       entityNamed: (NSString*)entityName
+- (NSArray *)rawRowsMatchingValue: (id)value
+			   forKey: (NSString *)key
+		      entityNamed: (NSString *)entityName
 {
   NSDictionary *valueDict;
   NSArray *results;
@@ -592,7 +593,7 @@ static NSString *EOMoreThanOneException = @"EOMoreThanOneException";
   NSAssert([entityName length]>0,@"No entity name");
 
   if (!value)
-    value=[EONull null];
+    value=GDL2_EONull;
 
   valueDict = [NSDictionary dictionaryWithObject: value
                             forKey: key];
@@ -602,8 +603,8 @@ static NSString *EOMoreThanOneException = @"EOMoreThanOneException";
   return results;
 }
 
-- (id)rawRowsMatchingValues: (NSDictionary*)values
-		entityNamed: (NSString*)entityName
+- (NSArray *)rawRowsMatchingValues: (NSDictionary *)values
+		       entityNamed: (NSString *)entityName
 {
   EOQualifier *qualifier;
   EOFetchSpecification *fetchSpec;
@@ -623,8 +624,8 @@ static NSString *EOMoreThanOneException = @"EOMoreThanOneException";
   return results;
 }
 
-- (id)rawRowsWithSQL: (NSString*)sqlString
-	  modelNamed: (NSString*)name
+- (NSArray *)rawRowsWithSQL: (NSString *)sqlString
+		 modelNamed: (NSString *)name
 {
   EODatabaseContext *databaseContext;
   EODatabaseChannel *databaseChannel;
@@ -665,8 +666,8 @@ static NSString *EOMoreThanOneException = @"EOMoreThanOneException";
   return results;
 }
 
-- (id)rawRowsWithStoredProcedureNamed: (NSString*)name
-			    arguments: (NSDictionary*)args
+- (NSArray *)rawRowsWithStoredProcedureNamed: (NSString *)name
+				   arguments: (NSDictionary *)args
 {
   EODatabaseContext *databaseContext;
   EODatabaseChannel *databaseChannel;
@@ -709,8 +710,8 @@ static NSString *EOMoreThanOneException = @"EOMoreThanOneException";
   return results;
 }
 
-- (id)executeStoredProcedureNamed: (NSString*)name
-			arguments: (NSDictionary*)args
+- (NSDictionary *)executeStoredProcedureNamed: (NSString *)name
+				    arguments: (NSDictionary *)args
 {
   EODatabaseContext *databaseContext;
   EODatabaseChannel *databaseChannel;
@@ -748,8 +749,8 @@ static NSString *EOMoreThanOneException = @"EOMoreThanOneException";
   return returnValues;
 }
 
-- (id)objectFromRawRow: (NSDictionary*)row
-	   entityNamed: (NSString*)entityName
+- (id)objectFromRawRow: (NSDictionary *)row
+	   entityNamed: (NSString *)entityName
 {
   NSAssert([entityName length] > 0, @"No entity name");
 
@@ -757,7 +758,7 @@ static NSString *EOMoreThanOneException = @"EOMoreThanOneException";
                entityNamed: entityName];
 }
 
-- (id)databaseContextForModelNamed: (NSString*)name
+- (EODatabaseContext *)databaseContextForModelNamed: (NSString *)name;
 {
   EOModelGroup *modelGroup;
   EOModel *model;
@@ -815,7 +816,7 @@ connectionDictionaryOverrides: (NSDictionary *)overrides
   return object;
 }
 
-- (id)primaryKeyForObject: (id)object
+- (NSDictionary *)primaryKeyForObject: (id)object;
 {
   EOKeyGlobalID *gid;
   EOEntity *entity;
@@ -833,8 +834,8 @@ connectionDictionaryOverrides: (NSDictionary *)overrides
   return newDict;
 }
 
-- (id)destinationKeyForSourceObject: (id)object
-		  relationshipNamed: (NSString*)name
+- (NSDictionary *)destinationKeyForSourceObject: (id)object
+			      relationshipNamed: (NSString*)name
 {
   EODatabaseContext *databaseContext;
   EODatabase *database;
@@ -910,7 +911,7 @@ connectionDictionaryOverrides: (NSDictionary *)overrides
   return newInstance;
 }
 
-- (NSArray*)localInstancesOfObjects: (NSArray*)objects
+- (NSArray *)localInstancesOfObjects: (NSArray *)objects
 {
   NSMutableArray *array;
   int i, count = [objects count];
@@ -931,7 +932,7 @@ connectionDictionaryOverrides: (NSDictionary *)overrides
   return array;
 }
 
-- (EOModelGroup*)modelGroup
+- (EOModelGroup *)modelGroup
 {
   EOObjectStore *rootObjectStore;
   EOObjectStoreCoordinator *objectStoreCoordinator;
@@ -954,7 +955,7 @@ connectionDictionaryOverrides: (NSDictionary *)overrides
   return modelGroup;
 }
 
-- (EOEntity*)entityNamed: (NSString*)entityName
+- (EOEntity *)entityNamed: (NSString *)entityName
 {
   EOEntity *entity;
   EOModelGroup *modelGroup;
@@ -979,7 +980,7 @@ connectionDictionaryOverrides: (NSDictionary *)overrides
   return entity;
 }
 
-- (EOEntity*)entityForClass: (Class)classObject
+- (EOEntity *)entityForClass: (Class)classObject
 {
   EOModelGroup *modelGroup;
   NSArray *models;
@@ -1036,7 +1037,7 @@ connectionDictionaryOverrides: (NSDictionary *)overrides
     }
 
   if (!result)
-    [NSException raise: /*NSObjectNotAvailableException*/NSInvalidArgumentException
+    [NSException raise: NSObjectNotAvailableException
                  format: @"%@ could not find entity for class named %@",
                  NSStringFromSelector(_cmd), className];
 
@@ -1045,7 +1046,7 @@ connectionDictionaryOverrides: (NSDictionary *)overrides
   return result;
 }
 
-- (EOEntity*)entityForObject: (id)object
+- (EOEntity *)entityForObject: (id)object
 {
   EOClassDescription *classDesc;
   EOEntity *newEntity;
@@ -1096,7 +1097,7 @@ connectionDictionaryOverrides: (NSDictionary *)overrides
 
 @implementation EOObjectStoreCoordinator (EOModelGroup)
 
-- (id) modelGroup
+- (id)modelGroup
 {
   //Seems OK
   EOModelGroup *modelGroup;
@@ -1118,7 +1119,7 @@ connectionDictionaryOverrides: (NSDictionary *)overrides
   return modelGroup;
 }
 
-- (void) setModelGroup: (EOModelGroup*)modelGroup
+- (void)setModelGroup: (EOModelGroup *)modelGroup
 {
   NSMutableDictionary *userInfo;
 
