@@ -1,7 +1,7 @@
 /** 
    Postgres95Adaptor.m <title>Postgres95Adaptor</title>
 
-   Copyright (C) 2000 Free Software Foundation, Inc.
+   Copyright (C) 2000,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: February 2000
@@ -12,8 +12,8 @@
    Author: Manuel Guesdon <mguesdon@orange-concept.com>
    Date: October 2000
 
-   $Revision: 1.18 $
-   $Date: 2004/04/17 12:41:32 $
+   $Revision: 22184 $
+   $Date: 2005-12-19 00:26:53 +0100 (Mon, 19 Dez 2005) $
 
    <abstract></abstract>
 
@@ -33,13 +33,13 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
 #include "config.h"
 
-RCS_ID("$Id: Postgres95Adaptor.m,v 1.18 2004/04/17 12:41:32 ayers Exp $")
+RCS_ID("$Id: Postgres95Adaptor.m 22184 2005-12-18 23:26:53Z ratmice $")
 
 #ifdef GNUSTEP
 #include <Foundation/NSArray.h>
@@ -76,6 +76,7 @@ RCS_ID("$Id: Postgres95Adaptor.m,v 1.18 2004/04/17 12:41:32 ayers Exp $")
 #include <Postgres95EOAdaptor/Postgres95SQLExpression.h>
 #include <Postgres95EOAdaptor/Postgres95Values.h>
 
+#include "Postgres95Private.h"
 
 NSString *Postgres95Exception = @"Postgres95Exception";
 static int pgConnTotalAllocated = 0;
@@ -172,33 +173,49 @@ postgresClientVersion()
 
 
 /* 
-   The first 4 must correspond to EOAdaptorValueTypes:
-   EOAdaptorNumberType, EOAdaptorCharactersType,
-   EOAdaptorBytesType, EOAdaptorDateType
-*/
-static NSString *externalTypeNames[] = {
-#warning (stephane@sente.ch) Needs to be updated!!!
-  @"numeric", @"varchar",  @"bpchar", @"date",
-  @"boolean", @"bool", 
-  @"char", @"char2", @"char4", @"char8", @"char16", @"filename", 
-  @"reltime", @"time", @"tinterval", @"abstime", @"timestamp",
-  @"real", @"double precision", @"float4", @"float8", 
-  @"bigint", @"int8", @"int4", @"int2", 
-  @"oid", @"oid8", @"oidint2", @"oidint4", @"oidchar16",
-  @"decimal", @"cid", @"tid", @"xid",
-  nil
-};
-
-static NSString *internalTypeNames[] = {
-  @"NSNumber", @"NSString", @"NSData", @"NSCalendarDate",
-  @"NSNumber", @"NSNumber",
-  @"NSString", @"NSString", @"NSString", @"NSString", @"NSString", @"NSString",
-  @"NSCalendarDate", @"NSCalendarDate", @"NSCalendarDate", @"NSCalendarDate", @"NSCalendarDate",
-  @"NSNumber", @"NSNumber", @"NSNumber", @"NSNumber",
-  @"NSNumber", @"NSNumber", @"NSNumber", @"NSNumber",
-  @"NSNumber", @"NSNumber", @"NSNumber", @"NSNumber", @"NSNumber",
-  @"NSDecimalNumber", @"NSDecimalNumber", @"NSDecimalNumber", @"NSDecimalNumber",
-  nil
+   The first 4 must correspond to EOAdaptorValueTypes.
+   external type name	internal type name
+ */
+static NSString *typeNames[][2] = {
+{@"numeric",		@"NSNumber"},		/* EOAdaptorNumberType */
+{@"varchar",		@"NSString"}, 		/* EOAdaptorCharactersType */
+{@"bytea",		@"NSData"},		/* EOAdaptorBytesType */
+{@"date",		@"NSCalendarDate"},	/* EOAdaptorDateType */
+{@"boolean",		@"NSNumber"},
+{@"bool",		@"NSNumber"},
+{@"char",		@"NSString"},
+{@"char2",		@"NSString"},
+{@"char4",		@"NSString"},
+{@"char8",		@"NSString"},
+{@"char16",		@"NSString"},
+{@"filename",		@"NSString"},
+{@"reltime",		@"NSCalendarDate"},
+{@"time",		@"NSCalendarDate"},
+{@"tinterval",		@"NSCalendarDate"},
+{@"abstime",		@"NSCalendarDate"},
+{@"timestamp",		@"NSCalendarDate"},
+{@"real",		@"NSNumber"},
+{@"double precision", 	@"NSNumber"},
+{@"float4",		@"NSNumber"},
+{@"float8",		@"NSNumber"},
+{@"bigint", 		@"NSNumber"},
+{@"int8",		@"NSNumber"},
+{@"integer",		@"NSNumber"},
+{@"int4",		@"NSNumber"},
+{@"smallint",		@"NSNumber"},
+{@"int2",		@"NSNumber"},
+{@"oid",		@"NSNumber"},
+{@"oid8",		@"NSNumber"},
+{@"oidint2",		@"NSNumber"},
+{@"oidint4",		@"NSNumber"},
+{@"oidchar16",		@"NSNumber"},
+{@"serial",		@"NSNumber"},
+{@"serial8",		@"NSNumber"},
+{@"decimal",		@"NSDecimalNumber"},
+{@"cid",		@"NSDecimalNumber"},
+{@"tid",		@"NSDecimalNumber"},
+{@"xid",		@"NSDecimalNumber"},
+{@"bpchar",		@"NSData"}
 };
 
 + (NSDictionary *)externalToInternalTypeMap
@@ -207,11 +224,25 @@ static NSString *internalTypeNames[] = {
 
   if (!externalToInternalTypeMap)
   {
-    int i;
+    int i, c;
+    id *externalTypeNames;
+    id *internalTypeNames;
+    size_t size;
 
-    for (i = 0; externalTypeNames[i]; i++);
+    c = sizeof(typeNames) / sizeof(typeNames[0]);
+    size = sizeof(id) * c;
+    externalTypeNames = (id *)NSZoneMalloc([self zone], size); 
+    internalTypeNames = (id *)NSZoneMalloc([self zone], size); 
+
+    for (i = 0; i < c; i++)
+      {
+	externalTypeNames[i] = typeNames[i][0];
+	internalTypeNames[i] = typeNames[i][1];
+      }
 
     externalToInternalTypeMap = [[NSDictionary alloc] initWithObjects: internalTypeNames forKeys: externalTypeNames count: i];
+    NSZoneFree([self zone], externalTypeNames);
+    NSZoneFree([self zone], internalTypeNames);
   }
 
   return externalToInternalTypeMap;
@@ -233,16 +264,23 @@ static NSString *internalTypeNames[] = {
   // TODO
   EOAdaptorValueType value = [attribute adaptorValueType];
 
-  [attribute setExternalType: externalTypeNames[value]];
+  [attribute setExternalType: typeNames[value][0]];
 }
 
 + (void)assignExternalInfoForEntity: (EOEntity *)entity
 {
-  NSEnumerator *attributeEnumerator = [[entity attributes] objectEnumerator];
-  EOAttribute *attribute;
+  NSArray *attributes = [entity attributes];
+  EOAttribute *attribute = nil;
+  IMP arrayOAI = NULL;
+  static SEL selfAEIFA_SEL = @selector(assignExternalInfoForAttribute:);
+  IMP selfAEIFA_IMP = [self methodForSelector: selfAEIFA_SEL];
+  unsigned i, c;
 
-  while ((attribute = [attributeEnumerator nextObject]))
-    [self assignExternalInfoForAttribute: attribute];
+  for (i=0, c=[attributes count]; i < c; i++)
+    {
+      attribute = PSQLA_ObjectAtIndexWithImpPtr(attributes, &arrayOAI, i);
+      selfAEIFA_IMP(self, selfAEIFA_SEL, attribute);
+    }
 }
 
 /* Inherited methods */
@@ -269,14 +307,14 @@ static NSString *internalTypeNames[] = {
 - (BOOL)isValidQualifierType: (NSString *)typeName
                        model: (EOModel *)model
 {
-  int i;
+  int i,c;
   
-  for (i = 0; externalTypeNames[i]; i++)
+  for (i = 0, c = sizeof(typeNames) / sizeof(typeNames[0]); i < c; i++)
     {
       //TODO REMOVE
-      NSDebugMLog(@"externalTypeNames[i]=%@", externalTypeNames[i]);
+      NSDebugMLog(@"externalTypeNames[i]=%@", typeNames[i][0]);
 
-      if ([externalTypeNames[i] isEqualToString: typeName])
+      if ([typeNames[i][0] isEqualToString: typeName])
         return YES;
     }
   //TODO REMOVE
