@@ -1,7 +1,7 @@
-/* 
+/* -*-objc-*-
    Postgres95Channel.h
 
-   Copyright (C) 2000 Free Software Foundation, Inc.
+   Copyright (C) 2000,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: February 2000
@@ -24,7 +24,7 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 #ifndef __Postgres95Channel_h__
@@ -80,8 +80,7 @@
 - (void)_describeDatabaseTypes;
 
 - (BOOL)_evaluateExpression: (EOSQLExpression *)expression
-	     withAttributes: attrs;
-
+	     withAttributes: (NSArray*)attributes;
 @end
 
 @interface NSObject (Postgres95ChannelDelegate)
