@@ -1,7 +1,7 @@
 /** 
    Postgres95Channel.m <title>Postgres95Channel</title>
 
-   Copyright (C) 2000-2003 Free Software Foundation, Inc.
+   Copyright (C) 2000-2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: February 2000
@@ -12,8 +12,8 @@
    Author: Manuel Guesdon <mguesdon@orange-concept.com>
    Date: October 2000
 
-   $Revision: 1.20 $
-   $Date: 2004/03/19 16:41:19 $
+   $Revision: 21645 $
+   $Date: 2005-08-17 10:07:57 +0200 (Mit, 17 Aug 2005) $
 
    <abstract></abstract>
 
@@ -33,13 +33,13 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
 #include "config.h"
 
-RCS_ID("$Id: Postgres95Channel.m,v 1.20 2004/03/19 16:41:19 ayers Exp $")
+RCS_ID("$Id: Postgres95Channel.m 21645 2005-08-17 08:07:57Z ayers $")
 
 
 #ifdef GNUSTEP
@@ -77,6 +77,7 @@ RCS_ID("$Id: Postgres95Channel.m,v 1.20 2004/03/19 16:41:19 ayers Exp $")
 #include <Postgres95EOAdaptor/Postgres95Context.h>
 #include <Postgres95EOAdaptor/Postgres95Values.h>
 
+#include "Postgres95Private.h"
 
 static void __dummy_function_used_for_linking(void)
 {
@@ -107,15 +108,31 @@ pgResultDictionary(PGresult *pgResult)
 
   for (i = 1; i <= nfields; i++)
     {
-      char *fname;
-      fname = PQfname(pgResult, i);
-      [fields addObject: [NSDictionary dictionaryWithObjectsAndKeys:
-	[NSS_SWF:@"%s",  fname], @"PQfname",
-	[NSS_SWF:@"%d",  PQfnumber(pgResult, fname)], @"PQfnumber",
-	[NSS_SWF:@"%ud", PQftype(pgResult, i)], @"PQftype",
-	[NSS_SWF:@"%d",  PQfsize(pgResult, i)], @"PQfsize",
-	[NSS_SWF:@"%d",  PQfmod(pgResult, i)], @"PQfmod",
-	nil]];
+      NSString *fname;
+      NSNumber *fnumber;
+      NSNumber *ftype;
+      NSNumber *fsize;
+      NSNumber *fmod;
+      NSDictionary *dict;
+      char *cfname;
+
+      cfname = PQfname(pgResult, i);
+
+      fname = [NSString stringWithCString: cfname];
+      fnumber = [NSNumber numberWithInt: PQfnumber(pgResult, cfname)];
+      ftype = [NSNumber numberWithUnsignedInt: PQftype(pgResult, i)];
+      fsize = [NSNumber numberWithInt: PQfsize(pgResult, i)];
+      fmod = [NSNumber numberWithInt: PQfmod(pgResult, i)];
+
+      dict = [NSDictionary dictionaryWithObjectsAndKeys:
+			     fname, @"PQfname",
+			   fnumber, @"PQfnumber",
+			   ftype, @"PQftype",
+			   fsize, @"PQfsize",
+			   fmod, @"PQfmod",
+			   nil];
+
+      [fields addObject: dict];
     }
 
   for (i = 1; i <= ntuples; i++)
@@ -126,7 +143,9 @@ pgResultDictionary(PGresult *pgResult)
 	{
 	  NSString *tupleInfo;
 	  NSString *tupleKey;
-	  tupleKey = [NSS_SWF:@"%s", PQfname(pgResult, j)];
+
+	  tupleKey = [NSString stringWithCString: PQfname(pgResult, j)];
+
 	  if (PQgetisnull(pgResult, i, j))
 	    {
 	      tupleInfo = @"NULL";
@@ -162,6 +181,35 @@ pgResultDictionary(PGresult *pgResult)
 
 @implementation Postgres95Channel
 
++ (void) initialize
+{
+  static BOOL initialized=NO;
+  if (!initialized)
+    {
+      Class aClass=Nil;
+      PSQLA_PrivInit();
+
+      aClass=[Postgres95Values class]; // Force Initialize;      
+    };
+};
+
+/* Set DateStyle to use ISO format.  */
+- (void)_setDateStyle
+{
+  _pgResult = PQexec(_pgConn, 
+		     "SET DATESTYLE TO ISO");
+
+  if (_pgResult == NULL || PQresultStatus(_pgResult) != PGRES_COMMAND_OK)
+    {
+      _pgResult = NULL;
+      [NSException raise: Postgres95Exception
+		   format: @"cannot set date style to ISO."];
+    }
+  
+  PQclear(_pgResult);
+  _pgResult = NULL;
+}
+
 - (id) initWithAdaptorContext: (EOAdaptorContext *)adaptorContext
 {
   if ((self = [super initWithAdaptorContext: adaptorContext]))
@@ -213,6 +261,7 @@ pgResultDictionary(PGresult *pgResult)
 
   if (_pgConn)
     {
+      [self _setDateStyle];
       [self _readServerVersion];
       [self _describeDatabaseTypes];
     }
@@ -306,16 +355,20 @@ pgResultDictionary(PGresult *pgResult)
 
 - (NSArray*)lowLevelResultFieldNames: (PGresult*)res
 {
-  NSMutableArray *names = [NSMutableArray array];
   int nb = PQnfields(res);
+  NSMutableArray *names 
+    = AUTORELEASE([PSQLA_alloc(NSMutableArray) initWithCapacity: nb]);
   int i;
+  IMP namesAO=NULL; //addObject:
 
   for (i = 0; i < nb; i++)
     {
       char *szName = PQfname(res,i);
-      NSString *name = [NSString stringWithCString: szName];
-
-      [names addObject: name];
+      unsigned length = szName ? strlen(szName) : 0;
+      NSString *name = [(PSQLA_alloc(NSString)) initWithCString: szName
+						length: length];
+      PSQLA_AddObjectWithImpPtr(names,&namesAO,name);
+      RELEASE(name);
     }
 
   return names;
@@ -370,7 +423,7 @@ zone:zone
           int count = [_attributes count];
           id valueBuffer[100];
           id *values = NULL;
-          EONull *nullValue = (EONull *)[EONull null];
+          IMP attributesOAI=NULL; // objectAtIndex:
 
           NSDebugMLLog(@"gsdb", @"count=%d", count);
 
@@ -394,7 +447,7 @@ zone:zone
 
           for (i = 0; i < count; i++)
             {
-              EOAttribute *attr = [_attributes objectAtIndex: i];
+              EOAttribute *attr = PSQLA_ObjectAtIndexWithImpPtr(_attributes,&attributesOAI,i);
               int length = 0;
               const char *string = NULL;
 
@@ -402,7 +455,7 @@ zone:zone
 
               if (PQgetisnull(_pgResult, _currentResultRow, i))
                 {
-                  values[i] = RETAIN(nullValue); //to be compatible with others returned values
+                  values[i] = RETAIN(PSQLA_EONull); //to be compatible with others returned values
                 }
               else
                 {
@@ -419,27 +472,33 @@ zone:zone
                           string = [self _readBinaryDataRow: (Oid)atol(string)
                                          length:&length zone: zone];
                           //For efficiency reasons, the returned value is NOT autoreleased !
-                          values[i] = [Postgres95Values newValueForBytes: string
-                                                        length: length
-                                                        attribute: attr];
+                          values[i] = PSQLA_Postgres95Values_newValueForBytesLengthAttribute(string,length,attr);
                         }
                       else
                         {
-                          //For efficiency reasons, the returned value is NOT autoreleased !
-                          values[i] = [[NSNumber alloc]
-					initWithLong: atol(string)];
+                          // The documentatin states that for efficiency
+			  // reasons, the returned value is NOT autoreleased
+			  // yet in the case of GNUstep-base it would be more
+			  // efficient if the numberWithLong: method would be
+			  // used as we could often skip alloc / dealloc
+			  // and get a cached value.  We could use it and
+			  // send retain, or we could start maintaing our
+			  // own cache.
+                          values[i] = [PSQLA_alloc(NSNumber) initWithLong: atol(string)];
                         }
                     }
                   else
                     {
                       //For efficiency reasons, the returned value is NOT autoreleased !
-                      values[i] = [Postgres95Values newValueForBytes: string
-                                                    length: length
-                                                    attribute: attr];
+                      values[i] = PSQLA_Postgres95Values_newValueForBytesLengthAttribute(string,length,attr);
                     }
                 }
 
-              NSDebugMLLog(@"gsdb", @"value[%d]=%@", i, values[i]);
+              NSDebugMLLog(@"gsdb", @"value[%d] (%p)=%@ of class: %@", 
+                           i, values[i], values[i], [values[i] class]);
+
+              // We don't want to add nil value to dictionary !
+              NSAssert1(values[i],@"No value for attribute: %@",attr);
             }
 
           NSDebugMLLog(@"gsdb", @"values count=%d values=%p", count, values);
@@ -706,6 +765,10 @@ zone:zone
   NSEnumerator *enumerator = nil;
   NSString *attrName = nil;
   Postgres95Context *adaptorContext = nil;
+  IMP attrEnumNO=NULL; // nextObject
+  IMP rowOFK=NULL; // objectForKey:
+  IMP nrowSOFK=NULL; // setObject:forKey:
+  IMP nrowOFK=NULL; // objectForKey:
 
   EOFLOGObjectFnStart();
 
@@ -749,7 +812,7 @@ each key
 */
 
   enumerator = [row keyEnumerator];
-  while ((attrName = [enumerator nextObject]))
+  while ((attrName = PSQLA_NextObjectWithImpPtr(enumerator,&attrEnumNO)))
     {
       EOAttribute *attribute = nil;
       NSString *externalType = nil;
@@ -763,7 +826,7 @@ each key
       if (!attribute)
 	return; //???????????
 
-      value = [row objectForKey: attrName];
+      value = PSQLA_ObjectForKeyWithImpPtr(row,&rowOFK,attrName);
       NSDebugMLLog(@"gsdb", @"value=%@", value);
 
       externalType = [attribute externalType];
@@ -772,7 +835,7 @@ each key
       /* Insert the binary value into the binaryDataRow dictionary */
       if ([externalType isEqual: @"inversion"])
         {
-	  id binValue = [nrow objectForKey: attrName];
+	  id binValue = PSQLA_ObjectForKeyWithImpPtr(nrow,&nrowOFK,attrName);
 	  Oid binOid = [self _insertBinaryData: binValue 
 			     forAttribute: attribute];
 	  value = [NSNumber numberWithLong: binOid];
@@ -783,8 +846,7 @@ each key
           // [[adaptorContext adaptor] databaseEncoding]
         }
 
-      [nrow setObject: value
-	    forKey: attrName];      
+      PSQLA_SetObjectForKeyWithImpPtr(nrow,&nrowSOFK,value,attrName);
     }
   
   NSDebugMLLog(@"gsdb", @"nrow=%@", nrow);
@@ -881,7 +943,6 @@ each key
 
   EOFLOGObjectFnStart();
 
-  NSDebugMLog(@"TEST attributesToFetch=%@", [entity attributesToFetch]);
   NSDebugMLLog(@"gsdb",@"%@ -- %@ 0x%x: isFetchInProgress=%s",
 	       NSStringFromSelector(_cmd),
 	       NSStringFromClass([self class]),
@@ -985,6 +1046,7 @@ each key
   EOAttribute *attr = nil;
   Postgres95Context *adaptorContext = nil;
   unsigned long rows = 0;
+  IMP valuesOFK=NULL; // objectForKey:
 
   EOFLOGObjectFnStart();
   
@@ -1004,6 +1066,9 @@ each key
 
   if ([values count] > 0)
     {
+      IMP valueEnumNO=NULL; // nextObject
+      IMP mrowSOFK=NULL; // setObject:forKey;
+
       mrow = AUTORELEASE([values mutableCopyWithZone: [values zone]]);
 
       // Get EOAttributes involved in update operation
@@ -1012,7 +1077,7 @@ each key
       invAttributes = AUTORELEASE([[NSMutableArray alloc] initWithCapacity: [mrow count]]);
 
       enumerator = [values keyEnumerator];
-      while ((attrName = [enumerator nextObject]))
+      while ((attrName = PSQLA_NextObjectWithImpPtr(enumerator,&valueEnumNO)))
         {
           attr = [entity attributeNamed: attrName];
           externalType = [attr externalType];
@@ -1024,8 +1089,9 @@ each key
 			      [values objectForKey:attrName]]
                 forKey:attrName];
 */
-          [mrow setObject:[values objectForKey: attrName]
-                forKey: attrName];
+          PSQLA_SetObjectForKeyWithImpPtr(mrow,&mrowSOFK,
+                                        PSQLA_ObjectForKeyWithImpPtr(values,&valuesOFK,attrName),
+                                        attrName);
 
           if ([externalType isEqual: @"inversion"])
             [invAttributes addObject: attr];
@@ -1037,6 +1103,7 @@ each key
 
       if ([invAttributes count])
         {
+          IMP invAttributesNO=NULL; // nextObject
           // Select with update qualifier to see there is only one row
           // to be updated and to get the large objects (to be updatetd)
           // Oid from dataserver - there is a hack here based on the fact that
@@ -1063,8 +1130,8 @@ each key
 
           // Update the large objects and modify the row to update with Oid's
 
-          enumerator = [invAttributes objectEnumerator];
-          while ((attr = [enumerator nextObject]))
+          enumerator = [invAttributes objectEnumerator];          
+          while ((attr = PSQLA_NextObjectWithImpPtr(enumerator,&invAttributesNO)))
             {
               Oid oldOid;
               Oid newOid;
@@ -1076,8 +1143,9 @@ each key
               oldOid = [[dbRow objectForKey:attrName] longValue];
               newOid = [self _updateBinaryDataRow: oldOid data: data];
 
-              [mrow setObject: [NSNumber numberWithUnsignedLong: newOid]
-                    forKey: attrName];
+              PSQLA_SetObjectForKeyWithImpPtr(mrow,&mrowSOFK,
+                                            [NSNumber numberWithUnsignedLong: newOid],
+                                            attrName);
             }
         }
 
@@ -1333,28 +1401,29 @@ each key
 
   if (colsNumber == 0)
     {
-      [self setAttributesToFetch: [NSArray array]];
+      [self setAttributesToFetch: PSQLA_NSArray];
     }
   else if (!_attributes) //??
     {
       int i;
-      id *attributes = NULL;
+      id *attributes = NULL;      
+      IMP origAttributesOAI=NULL;
+      IMP oidToTypeNameOFK=NULL;
 
       attributes = alloca(colsNumber * sizeof(id));
 
       for (i = 0; i < colsNumber; i++)
         {
-          EOAttribute *attribute = AUTORELEASE([EOAttribute new]);
+	  EOAttribute *attribute 
+	    = AUTORELEASE([PSQLA_alloc(EOAttribute) init]);
           NSString *externalType;
           NSString *valueClass = @"NSString";
           NSString *valueType = nil;
-          NSDebugMLog(@"TEST attributesToFetch=%@",
-		      [[attribute entity] attributesToFetch]);
 
           if (_origAttributes)
             {
-              EOAttribute *origAttr = (EOAttribute *)[_origAttributes
-                                                       objectAtIndex: i];
+              EOAttribute *origAttr = (EOAttribute *)
+                PSQLA_ObjectAtIndexWithImpPtr(_origAttributes,&origAttributesOAI,i);
 
               [attribute setName: [origAttr name]];
               [attribute setColumnName: [origAttr columnName]];
@@ -1367,8 +1436,9 @@ each key
               NSNumber *externalTypeNumber;
 	      externalTypeNumber 
 		= [NSNumber numberWithLong: PQftype(_pgResult, i)];
-              externalType = [_oidToTypeName objectForKey: externalTypeNumber];
-
+              externalType = PSQLA_ObjectForKeyWithImpPtr(_oidToTypeName,
+                                                        &oidToTypeNameOFK,externalTypeNumber);
+              
               if (!externalType)
                 [NSException raise: Postgres95Exception
                              format: @"cannot find type for Oid = %d",
@@ -1378,6 +1448,7 @@ each key
               [attribute setColumnName: @"unknown"];
               [attribute setExternalType: externalType];
 
+              //TODO: Optimize ?
               if      ([externalType isEqual: @"bool"])
                 valueClass = @"NSNumber", valueType = @"c";
               else if ([externalType isEqual: @"char"])
@@ -1418,8 +1489,6 @@ each key
             }
 
           attributes[i] = attribute;
-          NSDebugMLog(@"TEST attributesToFetch=%@",
-		      [[attribute entity] attributesToFetch]);
         }
 
       [self setAttributesToFetch: AUTORELEASE([[NSArray alloc]
@@ -1437,8 +1506,9 @@ each key
 - (NSArray *)describeTableNames
 {
   int i, count;
-  NSMutableArray *results = [NSMutableArray array];
+  NSMutableArray *results = nil;
   char *tableSelect;
+  IMP resultsAO=NULL; // addObject:
 
   if (_pgVersion < 70300)
     {
@@ -1465,12 +1535,14 @@ each key
     }
 
   count = PQntuples(_pgResult);
+  results= AUTORELEASE([PSQLA_alloc(NSMutableArray) initWithCapacity: count]);
 
   for (i = 0; i < count; i++)
     {
       char *oid = PQgetvalue(_pgResult, i, 0);
 
-      [results addObject: [NSString stringWithUTF8String: oid]];
+      PSQLA_AddObjectWithImpPtr(results,&resultsAO,
+				[NSString stringWithUTF8String: oid]);
     }
 
   PQclear(_pgResult);
@@ -1495,7 +1567,8 @@ each key
   NSString *valueClass = @"NSString";
   NSString *valueType = nil;
   NSString *tableOid;
-  unsigned int n, c, k;
+  unsigned int n, k;
+  int count = 0;
 
   entity = AUTORELEASE([[EOEntity alloc] init]);
   [entity setName: tableName];
@@ -1529,58 +1602,78 @@ each key
 				     forKey: @"tableOid"]];
   stmt = [NSS_SWF: @"SELECT attname,typname,attnum "
 		 @"FROM pg_attribute LEFT JOIN pg_type ON atttypid = oid "
-		 @"WHERE attnum > 0 AND attrelid=%@", tableOid];
+		 @"WHERE attnum > 0 AND attrelid=%@"
+		 @"AND attisdropped IS FALSE", tableOid];
 
   EOAdaptorDebugLog(@"Postgres95Adaptor: execute command:\n%@", stmt);
   PQclear(_pgResult);
 
   _pgResult = PQexec(_pgConn, [stmt cString]);
+  count = PQntuples(_pgResult);
 
-  for (n = 0, c = PQntuples(_pgResult); n < c; n++)
+  if (count>0)
     {
-      NSString *columnName;
-      NSString *externalType;
+      for (n = 0; n < count; n++)
+        {
+          NSString *columnName;
+          NSString *externalType;
+	  char *name;
+	  unsigned length;
 
-      externalType = [NSString stringWithCString: PQgetvalue(_pgResult,n,1)];
-      if ([externalType isEqual: @"bool"])
-	valueClass = @"NSNumber", valueType = @"c";
-      else if ([externalType isEqual: @"char"])
-	valueClass = @"NSNumber", valueType = @"c";
-      else if ([externalType isEqual: @"dt"])
-	valueClass = @"NSCalendarDate", valueType = nil;
-      else if ([externalType isEqual: @"date"])
-	valueClass = @"NSCalendarDate", valueType = nil;
-      else if ([externalType isEqual: @"time"])
-	valueClass = @"NSCalendarDate", valueType = nil;
-      else if ([externalType isEqual: @"float4"])
-	valueClass = @"NSNumber", valueType = @"f";
-      else if ([externalType isEqual: @"float8"])
-	valueClass = @"NSNumber", valueType = @"d";
-      else if ([externalType isEqual: @"int2"])
-	valueClass = @"NSNumber", valueType = @"i";
-      else if ([externalType isEqual: @"int4"])
-	valueClass = @"NSNumber", valueType = @"i";
-      else if ([externalType isEqual: @"int8"])
-	valueClass = @"NSNumber", valueType = @"l";
-      else if ([externalType isEqual: @"oid"])
-	valueClass = @"NSNumber", valueType = @"l";
-      else if ([externalType isEqual: @"varchar"])
-	valueClass = @"NSString", valueType = nil;
-      else if ([externalType isEqual: @"bpchar"])
-	valueClass = @"NSString", valueType = nil;
-      else if ([externalType isEqual: @"text"])
-	valueClass = @"NSString", valueType = nil;
+	  name = PQgetvalue(_pgResult,n,1);
+	  length = name ? strlen(name) : 0;
+          
+          externalType = [(PSQLA_alloc(NSString)) initWithCString: name
+						  length: length];
+          
+          //TODO optimize ?
+          if ([externalType isEqual: @"bool"])
+            valueClass = @"NSNumber", valueType = @"c";
+          else if ([externalType isEqual: @"char"])
+            valueClass = @"NSNumber", valueType = @"c";
+          else if ([externalType isEqual: @"dt"])
+            valueClass = @"NSCalendarDate", valueType = nil;
+          else if ([externalType isEqual: @"date"])
+            valueClass = @"NSCalendarDate", valueType = nil;
+          else if ([externalType isEqual: @"time"])
+            valueClass = @"NSCalendarDate", valueType = nil;
+          else if ([externalType isEqual: @"float4"])
+            valueClass = @"NSNumber", valueType = @"f";
+          else if ([externalType isEqual: @"float8"])
+            valueClass = @"NSNumber", valueType = @"d";
+          else if ([externalType isEqual: @"int2"])
+            valueClass = @"NSNumber", valueType = @"i";
+          else if ([externalType isEqual: @"int4"])
+            valueClass = @"NSNumber", valueType = @"i";
+          else if ([externalType isEqual: @"int8"])
+            valueClass = @"NSNumber", valueType = @"l";
+          else if ([externalType isEqual: @"oid"])
+            valueClass = @"NSNumber", valueType = @"l";
+          else if ([externalType isEqual: @"varchar"])
+            valueClass = @"NSString", valueType = nil;
+          else if ([externalType isEqual: @"bpchar"])
+            valueClass = @"NSString", valueType = nil;
+          else if ([externalType isEqual: @"text"])
+            valueClass = @"NSString", valueType = nil;
+          
+	  name = PQgetvalue(_pgResult, n, 0);
+	  length = name ? strlen(name) : 0;
+          columnName = [(PSQLA_alloc(NSString)) initWithCString: name
+						length: length];
 
-      attribute = AUTORELEASE([EOAttribute new]);
-      columnName 
-	= [NSString stringWithCString: PQgetvalue(_pgResult, n, 0)];
-      [attribute setName: columnName];
-      [attribute setColumnName: columnName];
-      [attribute setExternalType: externalType];
-      [attribute setValueType: valueType];
-      [attribute setValueClassName: valueClass];
-      [entity addAttribute: attribute];
-    }
+	  attribute = [PSQLA_alloc(EOAttribute) init];
+          [attribute setName: columnName];
+          [attribute setColumnName: columnName];
+          [attribute setExternalType: externalType];
+          [attribute setValueType: valueType];
+          [attribute setValueClassName: valueClass];
+          [entity addAttribute: attribute];
+
+	  RELEASE(externalType);
+	  RELEASE(attribute);
+	  RELEASE(columnName);
+        }
+    };
 
   PQclear(_pgResult);
 
@@ -1593,9 +1686,11 @@ each key
   _pgResult = PQexec(_pgConn,[stmt cString]);
   if (PQntuples(_pgResult))
     {
-      NSString *pkAttNum;
-      pkAttNum = [NSString stringWithCString: PQgetvalue(_pgResult,0,0)];
-      pkAttNum = [pkAttNum stringByReplacingString:@" " withString: @", "];
+      NSString *pkAttNum 
+	= [NSString stringWithCString: PQgetvalue(_pgResult,0,0)];
+      pkAttNum = [pkAttNum stringByReplacingString:@" "
+                           withString: @", "];
+
       stmt = [NSS_SWF: @"SELECT attname FROM pg_attribute "
 		     @"WHERE attrelid='%@' and attnum in (%@)",
 		     tableOid, pkAttNum];
@@ -1606,22 +1701,31 @@ each key
  
       if (PQntuples(_pgResult))
 	{
-	  NSArray *pkeys = AUTORELEASE([NSArray new]);
-	  for (k = 0, c = PQntuples(_pgResult); k < c; k++)
+	  NSMutableArray *pkeys;
+          count = PQntuples(_pgResult);
+	  pkeys = [PSQLA_alloc(NSMutableArray) initWithCapacity: count];
+	  for (k = 0; k < count; k++)
 	    {
-	      const char *cName;
+	      //TODO: Optimize, it's probably faster to use a mutable
+	      // string here instead of alloc/dealloing new strings.
 	      NSString *name;
+	      const char *cName;
+	      unsigned length;
 
 	      cName = PQgetvalue(_pgResult,k,0);
-	      name  = [NSString stringWithCString: cName];
+	      length = cName ? strlen(cName) : 0;
+	      name  = [(PSQLA_alloc(NSString)) initWithCString: cName
+					       length: length];
 	      attribute = [entity attributeNamed: name];
 	      NSDebugMLLog(@"adaptor", @"pk(%d) name: %@", k, name); 
 
-	      pkeys = [pkeys arrayByAddingObject: attribute];
+	      [pkeys addObject: attribute];
+	      RELEASE(name);
 	    }
 
 	  NSDebugMLLog(@"adaptor", @"pkeys %@", pkeys);
 	  [entity setPrimaryKeyAttributes: pkeys];
+	  RELEASE(pkeys);
 	}
     }
   /* </primary key stuff> */
@@ -1658,8 +1762,13 @@ each key
       EORelationship *relationship;
       NSSet          *dstPKSet;
       NSMutableSet   *dstAttribNames;
+      char           *name;
+      unsigned        length;
 
-      fkString = [NSString stringWithCString: PQgetvalue(_pgResult,i,0)];
+      name = PQgetvalue(_pgResult,i,0);
+      length = name ? strlen(name) : 0;
+      fkString = AUTORELEASE([(PSQLA_alloc(NSString)) initWithCString: name
+						      length: length]);
       NSDebugMLLog(@"adaptor", @"foreign key: %@\n",fkString);
 
       fkComp = [fkString componentsSeparatedByString: @"\\000"];
@@ -1730,10 +1839,13 @@ each key
 
 - (EOModel *)describeModelWithTableNames: (NSArray *)tableNames
 {
-  EOModel   *model;
-  EOAdaptor *adaptor;
-  NSArray *entityNames;
-  unsigned int i;
+  EOModel   *model=nil;
+  EOAdaptor *adaptor=nil;
+  EOEntity  *entity=nil;
+  NSArray   *entityNames=nil;
+  unsigned int i=0;
+  int tableNamesCount=[tableNames count];
+  int entityNamesCount=0;
 
   adaptor = [[self adaptorContext] adaptor];
   model = AUTORELEASE([[EOModel alloc] init]);
@@ -1741,7 +1853,7 @@ each key
   [model setAdaptorName: [adaptor name]];
   [model setConnectionDictionary: [adaptor connectionDictionary]];
 
-  for (i = 0; i < [tableNames count]; i++)
+  for (i = 0; i < tableNamesCount; i++)
     {
       NSAutoreleasePool *pool = [NSAutoreleasePool new];
       NSString *name;
@@ -1762,11 +1874,11 @@ each key
 
   /* <foreign key stuff> */
   entityNames = [model entityNames];
-  for (i = 0; i < [entityNames count]; i++)
+  entityNamesCount=[entityNames count];
+  for (i = 0; i < entityNamesCount; i++)
     {
       NSAutoreleasePool *pool = [NSAutoreleasePool new];
       NSString *entityName;
-      EOEntity *entity;
 
       NS_DURING
 	entityName = [entityNames objectAtIndex:i];
@@ -1782,6 +1894,20 @@ each key
 
       [pool release];
     }
+
+  for (i=0; i < entityNamesCount; i++)
+    {
+      NSAutoreleasePool *pool = [NSAutoreleasePool new];
+      NSMutableArray *classProperties;
+
+      entity = [model entityNamed:[entityNames objectAtIndex:i]];
+      classProperties = [NSMutableArray arrayWithArray:[entity attributes]]; 
+      [classProperties removeObjectsInArray: [entity primaryKeyAttributes]];
+      [entity setClassProperties: classProperties];
+
+      [pool release];
+    }
+
   [model beautifyNames];
   return model;
 }
@@ -1861,7 +1987,7 @@ each key
 
   expr = AUTORELEASE([[[_adaptorContext adaptor] expressionClass] new]);
   sequenceName = [NSString stringWithFormat: primaryKeySequenceNameFormat,
-			   [entity externalName]];
+			   [entity primaryKeyRootName]];
   sequenceName = [expr sqlStringForSchemaObjectName: sequenceName];
   sqlString = [NSString stringWithFormat: @"SELECT nextval('%@')",
 			sequenceName];
@@ -1881,13 +2007,12 @@ each key
     }
   else
     {
+      EOAttribute *attr;
       string = PQgetvalue(_pgResult, _currentResultRow, 0);
       length = PQgetlength(_pgResult, _currentResultRow, 0);
       
-      pkValue = AUTORELEASE([Postgres95Values newValueForBytes: string
-                                   length: length
-                                   attribute: [_pkAttributeArray
-						objectAtIndex: 0]]);
+      attr = [_pkAttributeArray objectAtIndex: 0];
+      pkValue = AUTORELEASE(PSQLA_Postgres95Values_newValueForBytesLengthAttribute(string,length,attr));
 
       NSAssert(pkValue, @"no pk value");
       key = [[entity primaryKeyAttributeNames] objectAtIndex: 0];
