@@ -1,13 +1,13 @@
 /** 
    Postgres95SQLExpression.m <title>Postgres95SQLExpression</title>
 
-   Copyright (C) 2000-2002 Free Software Foundation, Inc.
+   Copyright (C) 2000-2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: February 2000
 
-   $Revision: 1.18 $
-   $Date: 2004/03/29 14:40:15 $
+   $Revision: 22681 $
+   $Date: 2006-03-19 19:03:50 +0100 (Son, 19 Mär 2006) $
 
    <abstract></abstract>
 
@@ -28,21 +28,25 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
 #include "config.h"
 
-RCS_ID("$Id: Postgres95SQLExpression.m,v 1.18 2004/03/29 14:40:15 ayers Exp $")
+RCS_ID("$Id: Postgres95SQLExpression.m 22681 2006-03-19 18:03:50Z ratmice $")
 
 #ifdef GNUSTEP
 #include <Foundation/NSUtilities.h>
 #include <Foundation/NSException.h>
+#include <Foundation/NSDebug.h>
 #else
 #include <Foundation/Foundation.h>
 #endif
 
+#include <Foundation/NSDecimal.h>
+#include <Foundation/NSDecimalNumber.h>
+
 #ifndef GNUSTEP
 #include <GNUstepBase/GNUstep.h>
 #include <GNUstepBase/GSCategories.h>
@@ -57,19 +61,37 @@ RCS_ID("$Id: Postgres95SQLExpression.m,v 1.18 2004/03/29 14:40:15 ayers Exp $")
 #include <EOAccess/EOModel.h>
 #include <EOAccess/EOSchemaGeneration.h>
 
-#include <Postgres95EOAdaptor/Postgres95SQLExpression.h>
-#include <Postgres95EOAdaptor/Postgres95Adaptor.h>
-#include <Postgres95EOAdaptor/Postgres95Values.h>
+#include "Postgres95SQLExpression.h"
+#include "Postgres95Adaptor.h"
+#include "Postgres95Values.h"
 
-@interface EOSQLExpression (Privat)
-//Ayers: Review (Don't rely on privat method)
-- (NSString*) _aliasForRelatedAttribute: (EOAttribute *)attr
-		       relationshipPath: (NSString *)keyPath;
+#include "Postgres95Compatibility.h"
+#include "Postgres95Private.h"
 
+/* These methods are undocumented but exist in GDL2 and WO4.5. 
+   Ayers: Review (Don't rely on them) */
+@interface EOEntity (Undocumented)
+- (EORelationship *)relationshipForPath: (NSString *)path;
 @end
 
+@interface EOSQLExpression (PrivatUndocumented)
+- (NSString*) _aliasForRelatedAttribute: (EOAttribute *)attr
+		       relationshipPath: (NSString *)keyPath;
+@end
+
+
 @implementation Postgres95SQLExpression
 
++ (void) initialize
+{
+  static BOOL initialized=NO;
+  if (!initialized)
+    {
+      initialized=YES;
+      PSQLA_PrivInit();
+    };
+};
+
 + (NSString *)formatValue: (id)value
              forAttribute: (EOAttribute *)attribute
 {
@@ -127,9 +149,81 @@ RCS_ID("$Id: Postgres95SQLExpression.m,v 1.18 2004/03/29 14:40:15 ayers Exp $")
     {
       EOFLOGObjectLevelArgs(@"EOSQLExpression",
 			    @"float case - value=%@ class=%@",
-			    value, [value class]);      
+			    value, [value class]);
 
-      formatted = [NSString stringWithFormat: @"%@", value];
+      if (_isNilOrEONull(value))
+        formatted=@"NULL";
+      else 
+        {
+          unsigned short precision=[attribute precision];
+          short scale=[attribute scale];
+          EOFLOGObjectLevelArgs(@"EOSQLExpression",
+                                @"float case - value=%@ class=%@ precision=%d scale=%d",
+                                value, [value class],precision,scale);
+          // As far as I understand, we need to try to do complex things if precision!=0 or scale!=0 
+          if (precision==0 && scale==0)
+            {
+              // just convert it to string...
+              formatted = [NSString stringWithFormat: @"%@", value];
+            }
+          else
+            {
+              NSDecimalNumber* decimalValue=nil;
+              if ([value isKindOfClass: PSQLA_NSDecimalNumberClass] == NO)
+                {
+                  if ([value isKindOfClass: PSQLA_NSStringClass] == YES)
+                    {
+                      decimalValue
+			= AUTORELEASE([PSQLA_alloc(NSDecimalNumber) initWithString:value]);
+                      EOFLOGObjectLevelArgs(@"EOSQLExpression",
+                                            @"float case - value [%@]=%@ ==> decimalValue=%@",
+                                            value,[value class],decimalValue);
+                    }
+                  else if ([value respondsToSelector: @selector(doubleValue)])
+                    {
+                      decimalValue
+			= AUTORELEASE([PSQLA_alloc(NSDecimalNumber) initWithDouble:[value doubleValue]]);
+                      EOFLOGObjectLevelArgs(@"EOSQLExpression",
+                                            @"float case - value [%@]=%@ ==> decimalValue=%@",
+                                            value,[value class],decimalValue);
+                    }
+                  else if ([value respondsToSelector: @selector(floatValue)])
+                    {
+                      decimalValue
+			= AUTORELEASE([PSQLA_alloc(NSDecimalNumber) initWithFloat:[value floatValue]]);
+                      EOFLOGObjectLevelArgs(@"EOSQLExpression",
+                                            @"float case - value [%@]=%@ ==> decimalValue=%@",
+                                            value,[value class],decimalValue);
+                    }
+                  else if ([value respondsToSelector: @selector(intValue)])
+                    {
+                      decimalValue
+			= AUTORELEASE([PSQLA_alloc(NSDecimalNumber) initWithInt:[value intValue]]);
+                      EOFLOGObjectLevelArgs(@"EOSQLExpression",
+                                            @"float case - value [%@]=%@ ==> decimalValue=%@",
+                                            value,[value class],decimalValue);
+                    };
+                  if (decimalValue)
+                    {
+                      NSDecimal decimal;
+                      NSDecimalNumberHandler* handler=[NSDecimalNumberHandler decimalNumberHandlerWithRoundingMode:NSRoundPlain // Is Plain OK ?
+                                                                              scale:scale
+                                                                              raiseOnExactness:YES
+                                                                              raiseOnOverflow:YES
+                                                                              raiseOnUnderflow:YES
+                                                                              raiseOnDivideByZero:YES];
+                      decimalValue=[decimalValue decimalNumberByRoundingAccordingToBehavior:handler];
+                      decimal=[decimalValue decimalValue];
+                      formatted=NSDecimalString(&decimal,nil);
+                    }
+                  else
+                    {
+                      // Not supported type: just convert it to string...
+                      formatted = [NSString stringWithFormat: @"%@", value];
+                    };
+                };
+            };
+        };
 
       // value was for example 0 length string
       if ([formatted length] == 0)
@@ -141,7 +235,7 @@ RCS_ID("$Id: Postgres95SQLExpression.m,v 1.18 2004/03/29 14:40:15 ayers Exp $")
 			    @"BOOL case - value=%@ class=%@",
 			    value, [value class]);          
 
-      if ([value isKindOfClass: [NSNumber class]] == YES)
+      if ([value isKindOfClass: PSQLA_NSNumberClass] == YES)
         {
           BOOL boolValue = [value boolValue];
 
@@ -201,19 +295,42 @@ RCS_ID("$Id: Postgres95SQLExpression.m,v 1.18 2004/03/29 14:40:15 ayers Exp $")
 		    value, value, [value class]);
         }
       // Value can also be a string...
-      if ([value isKindOfClass:[NSDate class]])
+      if ([value isKindOfClass: PSQLA_NSDateClass])
         {
-          formatted = [NSString stringWithFormat: @"'%@'",
-                                [value
-                                  descriptionWithCalendarFormat:
-                                    [NSCalendarDate postgres95Format]//@"%d/%m/%Y %H:%M:%S"
-                                  timeZone: nil
-                                  locale: nil]];
+	  NSString *format;
+
+	  format = [value descriptionWithCalendarFormat: 
+			    PSQLA_postgresCalendarFormat
+			  timeZone: nil
+			  locale: nil];
+
+          formatted = [NSString stringWithFormat: @"'%@'", format];
+                                  
         }
       else
         formatted = [NSString stringWithFormat: @"'%@'",value];
     }
-  else
+  else if ([externalType isEqualToString: @"bytea"]) 
+    {
+      unsigned char *escapedString;
+      size_t newLength;
+
+      EOFLOGObjectLevelArgs(@"EOSQLExpression",
+			    @"bytea case - value=%@ class=%@",
+			    value, [value class]);
+
+      escapedString = PQescapeBytea ((unsigned char *)[value bytes],
+				     [value length],
+				     &newLength);
+
+      /* Note that the value returned in newLength is unreliable.  */
+      formatted = [NSString stringWithFormat: @"'%s'", escapedString];
+      if (escapedString)
+	{
+	  PQfreemem (escapedString);
+	}
+    }
+  else // String...
     {
       int length = 0;
 
@@ -432,7 +549,8 @@ RCS_ID("$Id: Postgres95SQLExpression.m,v 1.18 2004/03/29 14:40:15 ayers Exp $")
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"entity=%@", entity);
   EOFLOGObjectLevelArgs(@"EOSQLExpression", @"_aliasesByRelationshipPath=%@",
                         _aliasesByRelationshipPath);
-      
+  EOFLOGObjectLevelArgs(@"EOSQLExpression", @"_flags.hasOuterJoin=%d",
+                        _flags.hasOuterJoin);
 
   contextStackCount=[_contextStack count];
   if (contextStackCount>1 && _flags.hasOuterJoin)
@@ -470,7 +588,7 @@ RCS_ID("$Id: Postgres95SQLExpression.m,v 1.18 2004/03/29 14:40:15 ayers Exp $")
               
               EOFLOGObjectLevelArgs(@"EOSQLExpression", 
 				    @"entitiesString=%@", entitiesString);
-              
+
               if (useAliases)
                 [entitiesString appendFormat: @" %@",
                                 [_aliasesByRelationshipPath
@@ -658,6 +776,185 @@ RCS_ID("$Id: Postgres95SQLExpression.m,v 1.18 2004/03/29 14:40:15 ayers Exp $")
   return finalEntitiesString;
 }
 
+// Postgres like seems buggy:
+// see http://www.postgresql.org/docs/7.4/interactive/functions-matching.html
+// for exemple "ab" doesn't match [_]b
+
++ (NSString *)sqlPatternFromShellPattern: (NSString *)pattern
+{
+  NSString* sqlPattern=nil;
+  int patternLength=[pattern length];
+  if (patternLength==0)
+    sqlPattern=pattern;
+  else
+    {
+      const char *s, *p, *init = [pattern cString];
+      NSString *tmp;
+      NSMutableString *str = [NSMutableString stringWithCapacity:
+                                                patternLength];
+      IMP appendStringIMP = [str methodForSelector:@selector(appendString:)];
+      
+      for (s = p = init; *s; s++)
+        {
+          switch (*s)
+            {
+            case '*':
+              if (s != p)
+		{
+		  tmp = [(PSQLA_alloc(NSString)) initWithCString: p
+						 length: s-p];
+		  PSQLA_AppendStringWithImp(str, appendStringIMP, tmp);
+		  [tmp release];
+		}
+              [str appendString: @"%"];
+              p = s+1;
+              break;
+            case '?':
+              if (s != p)
+		{
+		  tmp = [(PSQLA_alloc(NSString)) initWithCString: p
+						 length: s-p];
+		  PSQLA_AppendStringWithImp(str, appendStringIMP, tmp);
+		  [tmp release];
+		}
+              (*appendStringIMP)(str,@selector(appendString:),@"_");
+              p = s+1;
+              break;
+            case '%':
+              if (s != p)
+		{
+		  tmp = [(PSQLA_alloc(NSString)) initWithCString: p
+						 length: s-p];
+		  PSQLA_AppendStringWithImp(str, appendStringIMP, tmp);
+		  [tmp release];
+		}
+              if (s != init && *(s-1) == '[' && *(s+1) == ']')
+                {
+                  (*appendStringIMP)(str,@selector(appendString:),@"%]");
+                  p = s+2; s++;
+                }
+              else
+                {
+                  (*appendStringIMP)(str,@selector(appendString:),@"[%]");
+                  p = s+1;
+                }
+              break;
+/*Postgresql doesn't want [_] but want _
+            case '_':
+              if (s != p)
+                (*appendStringIMP)(str,@selector(appendString:),
+                                   (*stringWithCString_lengthIMP)
+                                   (PSQLA_NSStringClass,PSQLA_stringWithCString_lengthSEL,p,s-p));
+              
+              if (s != init && *(s-1) == '[' && *(s+1) == ']')
+                {
+                  (*appendStringIMP)(str,@selector(appendString:),@"_]");
+                  p = s+2; p++;
+                }
+              else
+                {
+                  (*appendStringIMP)(str,@selector(appendString:),@"[_]");
+                  p = s+1;
+                }
+              break;
+*/
+            }
+        }
+      
+      if (*p)
+        (*appendStringIMP)(str,@selector(appendString:),[NSString stringWithCString:p]);
+
+      sqlPattern=str;
+    };
+
+  EOFLOGObjectLevelArgs(@"EOSQLExpression", @"pattern=%@ => %@",
+			pattern,sqlPattern);
+
+  return sqlPattern;
+}
+
++ (NSString *)sqlPatternFromShellPattern: (NSString *)pattern
+                     withEscapeCharacter: (unichar)escapeCharacter
+{
+  const char *s, *p, *init = [pattern cString];
+  NSString *tmp;
+  NSMutableString *str = [NSMutableString stringWithCapacity:
+					    [pattern length]];
+  IMP appendStringIMP = [str methodForSelector:@selector(appendString:)];
+
+  for (s = p = init; *s; s++)
+    {
+      switch (*s)
+        {
+	case '*':
+	  if (s != p)
+	    {
+	      tmp = [(PSQLA_alloc(NSString)) initWithCString: p
+					     length: s-p];
+	      PSQLA_AppendStringWithImp(str, appendStringIMP, tmp);
+	      [tmp release];
+	    }
+	  PSQLA_AppendStringWithImp(str,appendStringIMP,@"%");
+	  p = s+1;
+	  break;
+	case '?':
+	  if (s != p)
+	    {
+	      tmp = [(PSQLA_alloc(NSString)) initWithCString: p
+					     length: s-p];
+	      PSQLA_AppendStringWithImp(str, appendStringIMP, tmp);
+	      [tmp release];
+	    }
+	  PSQLA_AppendStringWithImp(str,appendStringIMP,@"_");
+	  p = s+1;
+	  break;
+	case '%':
+	  if (s != p)
+	    {
+	      tmp = [(PSQLA_alloc(NSString)) initWithCString: p
+					     length: s-p];
+	      PSQLA_AppendStringWithImp(str, appendStringIMP, tmp);
+	      [tmp release];
+	    }
+	  if (s != init && *(s-1) == '[' && *(s+1) == ']')
+	    {
+	      PSQLA_AppendStringWithImp(str,appendStringIMP,@"%]");
+	      p = s+2; s++;
+	    }
+	  else
+	    {
+	      PSQLA_AppendStringWithImp(str,appendStringIMP,@"[%]");
+	      p = s+1;
+	    }
+	  break;
+/*Postgresql doesn't want [_] but want _
+	case '_':
+	  if (s != p)
+	    PSQLA_AppendStringWithImp(str,appendStringIMP,
+            (*stringWithCString_lengthIMP)
+            (PSQLA_NSStringClass,PSQLA_stringWithCString_lengthSEL,p,s-p));
+	  
+	  if (s != init && *(s-1) == '[' && *(s+1) == ']')
+	    {
+	      PSQLA_AppendStringWithImp(str,appendStringIMP,@"_]");
+	      p = s+2; p++;
+	    }
+	  else
+	    {
+	      PSQLA_AppendStringWithImp(str,appendStringIMP,@"[_]");
+	      p = s+1;
+	    }
+	  break;
+*/
+        }
+    }
+
+  if (*p)
+    PSQLA_AppendStringWithImp(str,appendStringIMP,[NSString stringWithCString:p]);
+
+  return str;
+}
+
 - (NSString *)columnTypeStringForAttribute:(EOAttribute *)attribute
 {
   NSString *extType = [attribute externalType];
