@@ -1,7 +1,7 @@
 /* 
    Postgres95Values.h
 
-   Copyright (C) 2000 Free Software Foundation, Inc.
+   Copyright (C) 2000,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it
    Date: February 2000
@@ -21,7 +21,7 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 #ifndef __Postgres95Values_h__
@@ -41,9 +41,6 @@
 @class Postgres95Channel;
 
 
-extern NSString *Postgres95CalendarFormat;
-
-
 @interface Postgres95Values:NSObject
 {
 }
@@ -71,11 +68,4 @@ extern NSString *Postgres95CalendarFormat;
 
 @end
 
-@interface NSCalendarDate (Postgres95ValueCreation)
-
-+ (void)setPostgres95Format: (NSString *)_format;
-+ (NSString *)postgres95Format;
-
-@end
-
 #endif /* __Postgres95Values_h__ */
