@@ -1,13 +1,13 @@
 /** 
    Postgres95Values.m
 
-   Copyright (C) 2000-2003 Free Software Foundation, Inc.
+   Copyright (C) 2000-2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it
    Date: February 2000
 
-   $Revision: 1.13 $
-   $Date: 2004/03/19 16:41:19 $
+   $Revision: 22681 $
+   $Date: 2006-03-19 19:03:50 +0100 (Son, 19 Mär 2006) $
 
    <abstract></abstract>
 
@@ -27,14 +27,13 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
 #include "config.h"
 
-RCS_ID("$Id: Postgres95Values.m,v 1.13 2004/03/19 16:41:19 ayers Exp $")
-
+RCS_ID("$Id: Postgres95Values.m 22681 2006-03-19 18:03:50Z ratmice $")
 
 #ifdef GNUSTEP
 #include <Foundation/NSData.h>
@@ -55,22 +54,45 @@ RCS_ID("$Id: Postgres95Values.m,v 1.13 2004/03/19 16:41:19 ayers Exp $")
 
 
 #include <EOAccess/EOAttribute.h>
-#include <EOControl/EONSAddOns.h>
 
-#include <Postgres95EOAdaptor/Postgres95Adaptor.h>
-#include <Postgres95EOAdaptor/Postgres95Channel.h>
-#include <Postgres95EOAdaptor/Postgres95Values.h>
+#include "Postgres95EOAdaptor/Postgres95Adaptor.h"
+#include "Postgres95EOAdaptor/Postgres95Channel.h"
+#include "Postgres95EOAdaptor/Postgres95Values.h"
 
+#include "Postgres95Compatibility.h"
+#include "Postgres95Private.h"
+
+#include <stdlib.h>
 
 void __postgres95_values_linking_function (void)
 {
 }
 
-//NSString *Postgres95CalendarFormat = @"%m-%d-%Y %H:%M:%S %Z";
-NSString *Postgres95CalendarFormat = nil;
+static BOOL attrRespondsToValueClass = NO;
+static BOOL attrRespondsToValueTypeChar = NO;
+static NSStringEncoding LPSQLA_StringDefaultCStringEncoding;
+
+@interface EOAttribute (private)
+- (Class)_valueClass;
+- (char)_valueTypeChar;
+@end
 
 @implementation Postgres95Values
 
++ (void) initialize
+{
+  static BOOL initialized=NO;
+  if (!initialized)
+    {
+      PSQLA_PrivInit();
+
+      attrRespondsToValueClass
+	= [EOAttribute instancesRespondToSelector: @selector(_valueClass)];
+      attrRespondsToValueTypeChar
+	= [EOAttribute instancesRespondToSelector: @selector(_valueTypeChar)];
+      LPSQLA_StringDefaultCStringEncoding = [NSString defaultCStringEncoding];
+    }
+};
 
 + (id)newValueForBytes: (const void *)bytes
                 length: (int)length
@@ -94,59 +116,105 @@ NSString *Postgres95CalendarFormat = nil;
       return [self newValueForDateType: bytes
 		   length: length
 		   attribute: attribute];
+    default:
+      NSAssert2(NO,
+                @"Bad (%d) adaptor type for attribute : %@",
+                (int)[attribute adaptorValueType],attribute);
+      return nil;
     }
-
-    return nil;
 }
 
 /**
 For efficiency reasons, the returned value is NOT autoreleased !
+bytes is null terminated (cf Postgresql doc) and length is equivalent 
+to strlen(bytes)
 **/
 + (id)newValueForNumberType: (const void *)bytes
                      length: (int)length
                   attribute: (EOAttribute *)attribute
-{
-  NSString *str = nil;
+{  
   id value = nil;
+  NSString* externalType=nil;
+  
+  externalType=[attribute externalType];
 
-  if ([[attribute externalType] isEqualToString: @"bool"])
+  if (length==1 // avoid -isEqualToString if we can :-)
+      && [externalType isEqualToString: @"bool"])
     {
       if (((char *)bytes)[0] == 't' && ((char *)bytes)[1] == 0)
-	return [[NSNumber alloc] initWithBool:YES];
-      if (((char *)bytes)[0] == 'f' && ((char *)bytes)[1] == 0)
-	return [[NSNumber alloc] initWithBool:NO];
+	value=RETAIN(PSQLA_NSNumberBool_Yes);
+      else if (((char *)bytes)[0] == 'f' && ((char *)bytes)[1] == 0)
+	value=RETAIN(PSQLA_NSNumberBool_No);
+      else
+        NSAssert1(NO,@"Bad boolean: %@",[NSString stringWithCString:bytes
+                                                  length:length]);
     }
-
-  str = [[NSString alloc] initWithCString:(char *)bytes length:length];
-
-  if ([[attribute valueClassName] isEqualToString: @"NSDecimalNumber"])
-    value = [[NSDecimalNumber alloc] initWithString: str];
-  else if ([[attribute valueType] isEqualToString: @"i"])
-    value = [[NSNumber alloc] initWithInt: [str intValue]];
-  else if ([[attribute valueType] isEqualToString: @"I"])
-    value = [[NSNumber alloc] initWithUnsignedInt: [str unsignedIntValue]];
-  else if ([[attribute valueType] isEqualToString: @"c"])
-    value = [[NSNumber alloc] initWithChar: [str intValue]];
-  else if ([[attribute valueType] isEqualToString: @"C"])
-    value = [[NSNumber alloc] numberWithUnsignedChar: [str unsignedIntValue]];
-  else if ([[attribute valueType] isEqualToString: @"s"])
-    value = [[NSNumber alloc] initWithShort: [str shortValue]];
-  else if ([[attribute valueType] isEqualToString: @"S"])
-    value = [[NSNumber alloc] initWithUnsignedShort: [str unsignedShortValue]];
-  else if ([[attribute valueType] isEqualToString: @"l"])
-    value = [[NSNumber alloc] initWithLong: [str longValue]];
-  else if ([[attribute valueType] isEqualToString: @"L"])
-    value = [[NSNumber alloc] initWithUnsignedLong: [str unsignedLongValue]];
-  else if ([[attribute valueType] isEqualToString: @"u"])
-    value = [[NSNumber alloc] initWithLongLong: [str longLongValue]];
-  else if ([[attribute valueType] isEqualToString: @"U"])
-    value = [[NSNumber alloc] initWithUnsignedLongLong: [str unsignedLongLongValue]];
-  else if ([[attribute valueType] isEqualToString: @"f"])
-    value = [[NSNumber alloc] initWithFloat: [str floatValue]];
   else
-    value = [[NSNumber alloc] initWithDouble: [str doubleValue]];
+    {
+      Class valueClass = attrRespondsToValueClass 
+	? [attribute _valueClass] 
+	: NSClassFromString ([attribute valueClassName]);
 
-  [str release];
+      if (valueClass==PSQLA_NSDecimalNumberClass)
+        {
+	  //TODO: Optimize without creating NSString instance
+          NSString* str = [PSQLA_alloc(NSString) initWithCString:bytes
+				                 length:length];
+          
+          value = [PSQLA_alloc(NSDecimalNumber) initWithString: str];
+
+          RELEASE(str);
+        }
+      else
+        {
+          char valueTypeChar = attrRespondsToValueTypeChar
+	    ? [attribute _valueTypeChar]
+	    : [[attribute valueType] cString][0];
+          switch(valueTypeChar)
+            {
+            case 'i':
+              value = [PSQLA_alloc(NSNumber) initWithInt: atoi(bytes)];
+              break;
+            case 'I':
+              value = [PSQLA_alloc(NSNumber) initWithUnsignedInt:(unsigned int)atol(bytes)];
+              break;
+            case 'c':
+              value = [PSQLA_alloc(NSNumber) initWithChar: atoi(bytes)];
+              break;
+            case 'C':
+              value = [PSQLA_alloc(NSNumber) initWithUnsignedChar: (unsigned char)atoi(bytes)];
+              break;
+            case 's':
+              value = [PSQLA_alloc(NSNumber) initWithShort: (short)atoi(bytes)];
+              break;
+            case 'S':
+              value = [PSQLA_alloc(NSNumber) initWithUnsignedShort: (unsigned short)atoi(bytes)];
+              break;
+            case 'l':
+              value = [PSQLA_alloc(NSNumber) initWithLong: atol(bytes)];
+              break;
+            case 'L':
+              value = [PSQLA_alloc(NSNumber) initWithUnsignedLong:strtoul(bytes,NULL,10)];
+              break;
+            case 'u':
+              value = [PSQLA_alloc(NSNumber) initWithLongLong:atoll(bytes)];
+              break;
+            case 'U':
+              value = [PSQLA_alloc(NSNumber) initWithUnsignedLongLong:strtoull(bytes,NULL,10)];
+              break;
+            case 'f':
+              value = [PSQLA_alloc(NSNumber) initWithFloat: (float)strtod(bytes,NULL)];
+              break;
+            case 'd':
+            case '\0':
+              value = [PSQLA_alloc(NSNumber) initWithDouble: strtod(bytes,NULL)];
+              break;
+            default:
+              NSAssert2(NO,@"Unknown attribute valueTypeChar: %c for attribute: %@",
+                        valueTypeChar,attribute);
+            };
+        };
+    };
 
   return value;
 }
@@ -160,7 +228,7 @@ For efficiency reasons, the returned value is NOT autoreleased !
 {
   return [attribute newValueForBytes: bytes
 		    length: length
-		    encoding: [NSString defaultCStringEncoding]];
+		    encoding: LPSQLA_StringDefaultCStringEncoding];
 }
 
 /**
@@ -170,8 +238,23 @@ For efficiency reasons, the returned value is NOT autoreleased !
                     length: (int)length
                  attribute: (EOAttribute *)attribute
 {
-  return [attribute newValueForBytes: bytes
-		    length: length];
+  size_t newLength = length;
+  unsigned char *decodedBytes = 0;
+  id data = nil;
+
+  if ([[attribute externalType] isEqualToString: @"bytea"])
+    {
+      decodedBytes = PQunescapeBytea((unsigned char *)bytes, &newLength);
+      bytes = decodedBytes;
+    }
+
+  data = [attribute newValueForBytes: bytes
+		    length: newLength];
+  if (decodedBytes)
+    {
+      PQfreemem (decodedBytes);
+    }
+  return data;
 }
 
 
@@ -182,17 +265,26 @@ For efficiency reasons, the returned value is NOT autoreleased !
                    length: (int)length
                 attribute: (EOAttribute *)attribute
 {
-  id d;
-  NSString *str = [NSString stringWithCString: bytes length: length];
-  NSString *format = [NSCalendarDate postgres95Format];
+  id date=nil;
+  NSString *str = [PSQLA_alloc(NSString) initWithCString:(const char *)bytes
+                                         length:length];
 
-  d = [[NSCalendarDate alloc] initWithString: str
-			      calendarFormat: format];
-  // TODO server TZ ?
+  NSDebugMLLog(@"gsdb",@"str=%@ format=%@",str,PSQLA_postgresCalendarFormat);  
 
-  NSDebugMLLog(@"gsdb",@"str=%@ d=%@ format=%@",str,d,format);  
+  date = [PSQLA_alloc(NSCalendarDate) initWithString: str
+	       calendarFormat: PSQLA_postgresCalendarFormat];
 
-  return d;
+  NSDebugMLLog(@"gsdb",@"str=%@ d=%@ dtz=%@ format=%@",
+	       str,date,[date timeZone],PSQLA_postgresCalendarFormat);  
+
+  //We may have some 'invalid' date so it's better to stop here
+  NSAssert2(date,
+            @"No date created for string '%@' for attribute: %@",
+            str,attribute);
+
+  RELEASE(str);
+
+  return date;
 }
 
 
@@ -280,55 +372,19 @@ if ([type isEqual:@"bytea"])
 @end // NSData (Postgres95ValueCreation)
 
 */
+
 @implementation NSCalendarDate (Postgres95ValueCreation)
-/*
-- stringValueForPostgres95Type:(NSString*)type
-  attribute:(EOAttribute*)attribute
-{
-    NSString* externalType = [attribute externalType];
-	if (!CALENDAR_FORMAT)
-	  [NSCalendarDate setPostgres95Format:[NSString stringWithCString:"%b %d %Y %I:%M%p %Z"]];
-  
-    if ([externalType isEqualToString:@"abstime"]) {
-	id tz = [attribute serverTimeZone];
-	id date;
-
-	if (tz) {
-	    date = [[self copy] autorelease];
-	    [date setTimeZone:tz];
-	}
-	else
-	    date = self;
-
-	return [NSString stringWithFormat:@"'%@'",
-		    [date descriptionWithCalendarFormat:CALENDAR_FORMAT]];
-    }
-
-    THROW([[DataTypeMappingNotSupportedException alloc]
-	    initWithFormat:@"Postgres95 cannot map NSCalendarDate in "
-			    @"attribute %@ to external type %@",
-			    [attribute name], externalType]);
-    return nil;
-}
-*/
 
 + (NSString*)postgres95Format
 {
-  if (!Postgres95CalendarFormat)
-    //Mirko: @"%d/%m/%Y %H:%M:%S.00 %Z"
-    [NSCalendarDate setPostgres95Format: [NSString stringWithCString: "%m-%d-%Y %H:%M:%S %Z"]]; //"%b %d %Y %I:%M%p %Z"]];
-
-  return Postgres95CalendarFormat;
+  NSLog(@"%@ - is deprecated.  The adaptor always uses ISO format.",
+        NSStringFromSelector(_cmd));
+  return PSQLA_postgresCalendarFormat;
 }
 
-+ (void)setPostgres95Format: (NSString*)_format
-{
-  ASSIGN(Postgres95CalendarFormat, _format); 
-}
-
-
 @end // NSCalendarDate (Postgres95ValueCreation)
 
+
 /*
 @implementation EONull (Postgres95ValueCreation)
 
