@@ -1,7 +1,7 @@
 /** 
    EOAndQualifier.m <title>EOAndQualifier Class</title>
 
-   Copyright (C) 2000-2002 Free Software Foundation, Inc.
+   Copyright (C) 2000-2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: February 2000
@@ -9,8 +9,8 @@
    Author: Manuel Guesdon <mguesdon@orange-concept.com>
    Date: January 2002
 
-   $Revision: 1.15 $
-   $Date: 2004/03/19 16:41:19 $
+   $Revision: 21766 $
+   $Date: 2005-10-02 15:56:00 +0200 (Son, 02 Okt 2005) $
 
    <abstract></abstract>
 
@@ -30,13 +30,13 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
 #include "config.h"
 
-RCS_ID("$Id: EOAndQualifier.m,v 1.15 2004/03/19 16:41:19 ayers Exp $")
+RCS_ID("$Id: EOAndQualifier.m 21766 2005-10-02 13:56:00Z ayers $")
 
 #ifdef GNUSTEP
 #include <Foundation/NSDictionary.h>
@@ -204,9 +204,13 @@ RCS_ID("$Id: EOAndQualifier.m,v 1.15 2004/03/19 16:41:19 ayers Exp $")
   return resultQualifier;
 }
 
-- (id) initWithKeyValueUnarchiver: (id)param0
+- (id) initWithKeyValueUnarchiver: (id) archiver
 {
-  return [self notImplemented: _cmd]; //TODO
+  if ((self = [super init])) {
+    id qualifierArray = [archiver decodeObjectForKey:@"qualifiers"];
+    ASSIGN (_qualifiers, qualifierArray);
+  }
+  return self;
 }
 
 - (void) encodeWithKeyValueArchiver: (id)param0
@@ -227,7 +231,19 @@ RCS_ID("$Id: EOAndQualifier.m,v 1.15 2004/03/19 16:41:19 ayers Exp $")
   return nil;//TODO
 }
 
-- (id) description
+- (NSString *) description
+{
+  NSString *dscr;
+  
+  dscr = [NSString stringWithFormat: @"(%@)",
+		   [_qualifiers componentsJoinedByString: @" AND "]];
+
+  return dscr;
+}
+/**
+ * Deprecated debug description.  Expect it to be removed.
+ */
+- (NSString *) debugDescription
 {
   NSString *dscr;
 
@@ -239,4 +255,14 @@ RCS_ID("$Id: EOAndQualifier.m,v 1.15 2004/03/19 16:41:19 ayers Exp $")
   return dscr;
 }
 
+- (void)addQualifierKeysToSet: (NSMutableSet *)keys
+{
+  EOQualifier *qual;
+  unsigned int i,n;
+  for (i=0, n=[_qualifiers count]; i < n; i++)
+    {
+      qual = [_qualifiers objectAtIndex:i];
+      [qual addQualifierKeysToSet: keys];
+    }
+}
 @end
