@@ -1,7 +1,7 @@
 /* -*-objc-*-
    EOCheapArray.h
 
-   Copyright (C) 2000 Free Software Foundation, Inc.
+   Copyright (C) 2000,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Manuel Guesdon <mguesdon@orange-concept.com>
    Date: Sep 2000
@@ -21,7 +21,7 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 #ifndef __EOCheapArray_h__
@@ -43,7 +43,7 @@
   unsigned int _refcount;
 }
 
-- (id)initWithArray: (id)array;
+- (id)initWithArray: (NSArray *)array;
 - (id)initWithObjects: (id *)objects
 		count: (unsigned int)count;
 - (void)release;
@@ -71,7 +71,7 @@
 		count: (unsigned int)count;
 - (id)initWithArray: (NSArray *)array;
 - (void)dealloc;
-- (id)shallowCopy;
+- (NSArray *)shallowCopy;
 - (void)_setCopy: (id)param0;
 - (void)_mutate;
 - (unsigned int)count;
