@@ -1,13 +1,13 @@
 /** 
    EODetailDataSource.m <title>EODetailDataSource Class</title>
 
-   Copyright (C) 2000 Free Software Foundation, Inc.
+   Copyright (C) 2000,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: July 2000
 
-   $Revision: 1.11 $
-   $Date: 2004/03/19 16:41:20 $
+   $Revision: 22681 $
+   $Date: 2006-03-19 19:03:50 +0100 (Son, 19 Mär 2006) $
 
    <abstract></abstract>
 
@@ -27,13 +27,13 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
 #include "config.h"
 
-RCS_ID("$Id: EODetailDataSource.m,v 1.11 2004/03/19 16:41:20 ayers Exp $")
+RCS_ID("$Id: EODetailDataSource.m 22681 2006-03-19 18:03:50Z ratmice $")
 
 #ifdef GNUSTEP
 #include <Foundation/NSObject.h>
@@ -138,8 +138,15 @@ RCS_ID("$Id: EODetailDataSource.m,v 1.11 2004/03/19 16:41:20 ayers Exp $")
 
 - (EOClassDescription *)masterClassDescription
 {
-  return [EOClassDescription classDescriptionForEntityName:
-			       _masterClassDescriptionName];
+  if (_masterClassDescriptionName)
+    {
+      return [EOClassDescription classDescriptionForEntityName:
+				   _masterClassDescriptionName];
+    }
+  else
+    {
+      return [_masterDataSource classDescriptionForObjects];
+    }
 }
 
 - (void)setMasterClassDescription: (EOClassDescription *)classDescription
@@ -196,7 +203,12 @@ RCS_ID("$Id: EODetailDataSource.m,v 1.11 2004/03/19 16:41:20 ayers Exp $")
 
 - (EOEditingContext *)editingContext
 {
-  return [_masterObject editingContext];
+  /*
+   * 4.5 documented as _masterObject or nil 
+   * in 5.x this is documented as returning the master data source or nil
+   * seems to be a documentation error in 4.5
+   */
+  return [_masterDataSource editingContext];
 }
 
 - (NSArray *)fetchObjects
