@@ -1,7 +1,7 @@
 /* -*-objc-*-
    EOEditingContext.h
 
-   Copyright (C) 2000-2002 Free Software Foundation, Inc.
+   Copyright (C) 2000-2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Date: June 2000
 
@@ -20,11 +20,11 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
-#ifndef	__EOEditingContext_h__
-#define	__EOEditingContext_h__
+#ifndef	__EOControl_EOEditingContext_h__
+#define	__EOControl_EOEditingContext_h__
 
 #ifdef GNUSTEP
 #include <Foundation/NSHashTable.h>
@@ -47,11 +47,13 @@
 @class NSAutoreleasePool;
 @class NSUndoManager;
 
+@class EOSharedEditingContext;
 
 @interface EOEditingContext : EOObjectStore <EOObserving>
 {
   EOObjectStore *_objectStore;
   NSUndoManager *_undoManager;
+
   NSHashTable *_unprocessedChanges;
   NSHashTable *_unprocessedDeletes;
   NSHashTable *_unprocessedInserts;
@@ -59,7 +61,7 @@
   NSHashTable *_deletedObjects;
   NSHashTable *_changedObjects;
 
-  NSMapTable *_objectsById;
+  NSMapTable *_globalIDsByObject;
   NSMapTable *_objectsByGID;
   NSMutableDictionary *_snapshotsByGID;
   NSMutableDictionary *_eventSnapshotsByGID;
@@ -68,6 +70,7 @@
   NSMutableArray *_editors;
   id _messageHandler;
   unsigned short _undoTransactionID;
+
   struct {
     unsigned registeredForCallback:1;
     unsigned propagatesDeletesAtEndOfEvent:1;
@@ -80,8 +83,11 @@
     unsigned registeredUndoTransactionID:1;
     unsigned retainsAllRegisteredObjects:1;
     unsigned lockUsingParent:1;
-    unsigned unused:5;
+    unsigned ignoreSharedContextNotifications:1;
+    unsigned isSharedContextLocked:1;
+    unsigned unused:3;
   } _flags;
+
   struct {
     unsigned willRunLoginPanel:1;
     unsigned shouldFetchObjects:1;
@@ -93,12 +99,17 @@
     unsigned willSaveChanges:1;
   } _delegateRespondsTo;
   
-  NSRecursiveLock*_lock;
+  NSRecursiveLock *_lock;
+  EOSharedEditingContext *_sharedContext;
   int _lockCount;
   id _notificationQueue;
   NSAutoreleasePool * _lockPool;
+  NSTimeInterval fetchTimestamp;
 }
 
++ (void)setDefaultFetchTimestampLag: (NSTimeInterval)lag;
++ (NSTimeInterval)defaultFetchTimestampLag;
+
 + (void)setInstancesRetainRegisteredObjects: (BOOL)flag;
 + (BOOL)instancesRetainRegisteredObjects;
 
@@ -109,8 +120,6 @@
 - (void)insertObject: (id)object;
 - (void)insertObject: (id)object
         withGlobalID: (EOGlobalID *)gid;
-- (void)_insertObject: (id)object
-         withGlobalID: (EOGlobalID *)gid;
 
 -(void)setLevelsOfUndo: (int)levels;
 
@@ -135,8 +144,6 @@
 - (void)setUndoManager: (NSUndoManager *)undoManager;
 - (NSUndoManager *)undoManager;
 
-- (void)_observeUndoManagerNotifications;
-
 - (void)objectWillChange: (id)object;
 
 - (void)recordObject: (id)object
@@ -152,9 +159,6 @@
 - (NSArray *)insertedObjects;
 - (NSArray *)deletedObjects;
 
-- (void)_processDeletedObjects;
-- (void)_processOwnedObjectsUsingChangeTable: (NSHashTable *)changeTable
-				 deleteTable: (NSHashTable *)deleteTable;
 - (void)propagatesDeletesUsingTable: (NSHashTable *)deleteTable;
 - (void)validateDeletesUsingTable: (NSHashTable *)deleteTable;
 - (BOOL)validateTable: (NSHashTable *)table
@@ -164,7 +168,6 @@
 
 
 - (void)processRecentChanges;
-- (void)_registerClearStateWithUndoManager;
 
 - (BOOL)propagatesDeletesAtEndOfEvent;
 - (void)setPropagatesDeletesAtEndOfEvent: (BOOL)propagatesDeletesAtEndOfEvent;
@@ -175,6 +178,9 @@
 - (BOOL)locksObjectsBeforeFirstModification;
 - (void)setLocksObjectsBeforeFirstModification: (BOOL)yn;
 
+- (EOSharedEditingContext *)sharedEditingContext;
+- (void)setSharedEditingContext:(EOSharedEditingContext *)sharedEditingContext;
+
 /** Returns a dictionary containing a snapshot of object 
 that reflects its committed values (last values putted in 
 the database; i.e. values before changes were made on the 
@@ -284,9 +290,8 @@ shouldInvalidateObject: (id)object
 shouldFetchObjectsDescribedByFetchSpecification: (EOFetchSpecification *)fetchSpecification;
 
 - (BOOL)editingContext: (EOEditingContext *)editingContext
-shouldMergeChangedObject: (id)object;
-
-- (void)didMergeChangedObjectsInEditingContext: (EOEditingContext *)editingContext;
+shouldMergeChangesForObject: (id)object;
+- (void)editingContextDidMergeChanges: (EOEditingContext *)editingContext;
 
 @end
 
@@ -364,14 +369,4 @@ shouldContinueFetchingWithCurrentObjectCount: (unsigned)count
 
 @end
 
-/*
- * Information
- */
-@interface EOEditingContext (EOEditingContextInfo)
-
-- (NSDictionary *)unprocessedInfo;
-- (NSDictionary *)pendingInfo;
-
-@end
-
 #endif
