@@ -1,7 +1,7 @@
 /* -*-objc-*-
    EOFault.h
 
-   Copyright (C) 1996-2000 Free Software Foundation, Inc.
+   Copyright (C) 1996-2000,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mircea Oancea <mircea@jupiter.elcom.pub.ro>
    Date: 1996
@@ -24,7 +24,7 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 #ifndef	__EOFault_h__
@@ -54,6 +54,7 @@
 
 @interface EOFault
 {
+@public
   Class isa;
   EOFaultHandler *_handler;
 }
