@@ -1,7 +1,7 @@
 /** 
    EOFault.m <title>EOFault Class</title>
 
-   Copyright (C) 1996-2002 Free Software Foundation, Inc.
+   Copyright (C) 1996-2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mircea Oancea <mircea@jupiter.elcom.pub.ro>
    Date: 1996
@@ -9,8 +9,8 @@
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: June 2000
 
-   $Revision: 1.21 $
-   $Date: 2004/03/19 16:41:20 $
+   $Revision: 21645 $
+   $Date: 2005-08-17 10:07:57 +0200 (Mit, 17 Aug 2005) $
 
    <abstract></abstract>
 
@@ -30,13 +30,13 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
 #include "config.h"
 
-RCS_ID("$Id: EOFault.m,v 1.21 2004/03/19 16:41:20 ayers Exp $")
+RCS_ID("$Id: EOFault.m 21645 2005-08-17 08:07:57Z ayers $")
 
 #ifdef GNUSTEP
 #include <Foundation/NSObject.h>
@@ -67,6 +67,8 @@ RCS_ID("$Id: EOFault.m,v 1.21 2004/03/19 16:41:20 ayers Exp $")
 #include <EOControl/EOEditingContext.h>
 #include <EOControl/EODebug.h>
 
+#include <limits.h>
+
 /*
  * EOFault class
  */
@@ -140,9 +142,27 @@ static Class EOFaultClass = NULL;
 
 + (BOOL)respondsToSelector: (SEL)sel
 {
-  return (GSGetInstanceMethod(self, sel) != (GSMethod)0);
+  return (GSGetMethod(self, sel, NO, YES) != (GSMethod)0);
 }
 
+/**
+ * Returns a pointer to the C function implementing the method used
+ * to respond to messages with aSelector by instances of the receiving
+ * class.
+ * <br />Raises NSInvalidArgumentException if given a null selector.
+ *
+ * It's a temporary fix to support NSAutoreleasePool optimization
+ */
++ (IMP) instanceMethodForSelector: (SEL)aSelector
+{
+  if (aSelector == 0)
+    [NSException raise: NSInvalidArgumentException
+		format: @"%@ null selector given", NSStringFromSelector(_cmd)];
+  /*
+   *	Since 'self' is an class, get_imp() will get the instance method.
+   */
+  return get_imp((Class)self, aSelector);
+}
 
 // Fault class methods
 
@@ -170,6 +190,7 @@ static Class EOFaultClass = NULL;
 
 + (BOOL)isFault: (id)object
 {
+  //See also EOPrivat.h
 //  NSDebugFLLog(@"gsdb",@"object=%p",object);
 
   if (object == nil)
@@ -263,7 +284,7 @@ static Class EOFaultClass = NULL;
 
 // Fault Instance methods
 
-- superclass
+- (Class) superclass
 {
   return [[_handler targetClass] superclass];
 }
@@ -326,7 +347,8 @@ static Class EOFaultClass = NULL;
   NSDebugFLLog(@"gsdb", @"class=%@ aSelector=%@", class,
 	       NSStringFromSelector(aSelector));
 
-  respondsToSelector = (GSGetInstanceMethod(class, aSelector) != (GSMethod)0);
+  respondsToSelector 
+    = (GSGetMethod(class, aSelector, YES, YES) != (GSMethod)0);
   NSDebugFLLog(@"gsdb", @"STOP self=%p", self);
 
   return respondsToSelector;
@@ -488,6 +510,10 @@ static Class EOFaultClass = NULL;
 
 - (void)forwardInvocation: (NSInvocation *)invocation
 {
+  NSDebugFLLog(@"gsdb",@"invocation selector=%@ target: %p",
+               NSStringFromSelector([invocation selector]),
+               [invocation target]);
+
   if ([_handler shouldPerformInvocation: invocation])
     [_handler completeInitializationOfObject: self];
 
