@@ -1,7 +1,7 @@
 /* -*-objc-*-
    EOFetchSpecification.h
 
-   Copyright (C) 2000 Free Software Foundation, Inc.
+   Copyright (C) 2000,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: February 2000
@@ -21,7 +21,7 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 #ifndef __EOFetchSpecification_h__
@@ -60,7 +60,7 @@
     unsigned locksObjects:1;
     unsigned refreshesRefetchedObjects:1;
     unsigned promptsAfterFetchLimit:1;
-    unsigned allVariablesRequiredFromBindings:1;
+    unsigned requiresAllQualifierBindingVariables:1;
     unsigned reserved:26;
   }_flags;
 }
@@ -74,8 +74,6 @@
 		  isDeep: (BOOL)isDeep
 		   hints: (NSDictionary *)hints;
 
-- (EOFetchSpecification *)fetchSpecificationByApplyingBindings: (id)bindings;
-
 - (EOFetchSpecification *)fetchSpecificationWithQualifierBindings: (NSDictionary *)bindings;
 
 + (EOFetchSpecification *)fetchSpecificationNamed: (NSString *)name
@@ -129,8 +127,8 @@
 - (void)setPromptsAfterFetchLimit: (BOOL)promptsAfterFetchLimit;
 - (BOOL)promptsAfterFetchLimit;
 
-- (void)setAllVariablesRequiredFromBindings: (BOOL)allVariablesRequired;
-- (BOOL)allVariablesRequiredFromBindings;
+- (void)setRequiresAllQualifierBindingVariables: (BOOL)flag;
+- (BOOL)requiresAllQualifierBindingVariables;
 
 - (void)setPrefetchingRelationshipKeyPaths: (NSArray *)prefetchingRelationshipKeys;
 - (NSArray *)prefetchingRelationshipKeyPaths;
@@ -148,9 +146,6 @@
 - (NSArray *)rawRowKeyPaths;
 - (void)setRawRowKeyPaths: (NSArray *)rawRowKeyPaths;
 
-- (BOOL)requiresAllQualifierBindingVariables;
-- (void)setRequiresAllQualifierBindingVariables: (BOOL)flag;
-
 @end
 
 GDL2CONTROL_EXPORT NSString *EOPrefetchingRelationshipHintKey;
