@@ -1,13 +1,13 @@
 /** 
    EOFetchSpecification.m <title>EOFetchSpecification</title>
 
-   Copyright (C) 2000 Free Software Foundation, Inc.
+   Copyright (C) 2000,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: February 2000
 
-   $Revision: 1.16 $
-   $Date: 2004/03/19 16:41:20 $
+   $Revision: 21645 $
+   $Date: 2005-08-17 10:07:57 +0200 (Mit, 17 Aug 2005) $
 
    <abstract></abstract>
 
@@ -27,13 +27,13 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
 #include "config.h"
 
-RCS_ID("$Id: EOFetchSpecification.m,v 1.16 2004/03/19 16:41:20 ayers Exp $")
+RCS_ID("$Id: EOFetchSpecification.m 21645 2005-08-17 08:07:57Z ayers $")
 
 #ifdef GNUSTEP
 #include <Foundation/NSCoder.h>
@@ -56,6 +56,13 @@ RCS_ID("$Id: EOFetchSpecification.m,v 1.16 2004/03/19 16:41:20 ayers Exp $")
 #include <EOControl/EONSAddOns.h>
 #include <EOControl/EOQualifier.h>
 
+#include <EOControl/EODeprecated.h>
+
+NSString *EOPrefetchingRelationshipHintKey = @"EOPrefetchingRelationshipHintKey";
+NSString *EOFetchLimitHintKey = @"EOFetchLimitHintKey";
+NSString *EOPromptAfterFetchLimitHintKey = @"EOPromptAfterFetchLimitHintKey";
+
+
 @interface NSObject (EOAccess)
  /* EOEntity.h */
 - (EOFetchSpecification *)fetchSpecificationNamed: (NSString *)fetchSpecName;
@@ -136,16 +143,15 @@ RCS_ID("$Id: EOFetchSpecification.m,v 1.16 2004/03/19 16:41:20 ayers Exp $")
   return self;
 }
 
-- (EOFetchSpecification *)fetchSpecificationByApplyingBindings: (id)bindings
-{
-  [self notImplemented:_cmd];
-  return nil;
-}
-
 - (EOFetchSpecification *)fetchSpecificationWithQualifierBindings: (NSDictionary *)bindings
 {
-  [self notImplemented: _cmd];
-  return nil;
+  EOQualifier *qualifier;
+  BOOL flag = [self requiresAllQualifierBindingVariables];
+
+  qualifier = [[self qualifier] qualifierWithBindings: bindings
+				requiresAllVariables: flag];
+  [self setQualifier: qualifier];
+  return self;
 }
 
 + (EOFetchSpecification *)fetchSpecificationNamed: (NSString *)name
@@ -277,7 +283,7 @@ setRequiresAllQualifierBindingVariables:requiresAllQualifierBindingVariables
         [unarchiver decodeBoolForKey: @"refreshesRefetchedObjects"];
       _flags.promptsAfterFetchLimit = 
         [unarchiver decodeBoolForKey: @"promptsAfterFetchLimit"];
-      _flags.allVariablesRequiredFromBindings = 
+      _flags.requiresAllQualifierBindingVariables = 
         [unarchiver decodeBoolForKey: @"requiresAllQualifierBindingVariables"];
     }
 
@@ -300,21 +306,21 @@ setRequiresAllQualifierBindingVariables:requiresAllQualifierBindingVariables
             forKey:@"prefetchingRelationshipKeyPaths"];
   [archiver encodeInt:_fetchLimit
             forKey:@"fetchLimit"];
-  [archiver encodeBool:_flags.usesDistinct
+  [archiver encodeBool:_flags.usesDistinct ? YES : NO
             forKey:@"usesDistinct"];
-  [archiver encodeBool:_flags.isDeep
+  [archiver encodeBool:_flags.isDeep ? YES : NO
             forKey:@"isDeep"];
-  [archiver encodeBool:_flags.locksObjects
+  [archiver encodeBool:_flags.locksObjects ? YES : NO
             forKey:@"locksObjects"];
-  [archiver encodeBool:_flags.refreshesRefetchedObjects
+  [archiver encodeBool:_flags.refreshesRefetchedObjects ? YES : NO
             forKey:@"refreshesRefetchedObjects"];
-  [archiver encodeBool:_flags.promptsAfterFetchLimit
+  [archiver encodeBool:_flags.promptsAfterFetchLimit ? YES : NO
             forKey:@"promptsAfterFetchLimit"];
-  [archiver encodeBool:_flags.refreshesRefetchedObjects
+  [archiver encodeBool:_flags.refreshesRefetchedObjects ? YES : NO
             forKey:@"refreshesRefetchedObjects"];
-  [archiver encodeBool:_flags.promptsAfterFetchLimit
+  [archiver encodeBool:_flags.promptsAfterFetchLimit ? YES : NO
             forKey:@"promptsAfterFetchLimit"];
-  [archiver encodeBool:_flags.allVariablesRequiredFromBindings
+  [archiver encodeBool:_flags.requiresAllQualifierBindingVariables ? YES : NO
             forKey:@"requiresAllQualifierBindingVariables"];
 }
 
@@ -348,7 +354,7 @@ setRequiresAllQualifierBindingVariables:requiresAllQualifierBindingVariables
   [desc appendString: [NSString stringWithFormat: @"promptsAfterFetchLimit = %s;\n", 
 				_flags.promptsAfterFetchLimit ? "YES" : "NO"]];
   [desc appendString: [NSString stringWithFormat: @"requiresAllQualifierBindingVariables = %s;\n", 
-				_flags.allVariablesRequiredFromBindings ? "YES" : "NO"]];
+				_flags.requiresAllQualifierBindingVariables ? "YES" : "NO"]];
   [desc appendString: @"}"];
 
   return desc;
@@ -386,10 +392,10 @@ setRequiresAllQualifierBindingVariables:requiresAllQualifierBindingVariables
   return _qualifier;
 }
 
-- (void)setUsesDistinct: (BOOL)usesDistinct
+- (void)setUsesDistinct: (BOOL)flag
 {
   [self willChange];
-  _flags.usesDistinct = usesDistinct;
+  _flags.usesDistinct = flag ? YES : NO;
 }
 
 - (BOOL)usesDistinct
@@ -400,7 +406,7 @@ setRequiresAllQualifierBindingVariables:requiresAllQualifierBindingVariables
 - (void)setIsDeep: (BOOL)isDeep
 {
   [self willChange];
-  _flags.isDeep = isDeep;
+  _flags.isDeep = isDeep ? YES : NO;
 }
 
 - (BOOL)isDeep
@@ -411,7 +417,7 @@ setRequiresAllQualifierBindingVariables:requiresAllQualifierBindingVariables
 - (void)setLocksObjects: (BOOL)locksObjects
 {
   [self willChange];  
-  _flags.locksObjects = locksObjects;
+  _flags.locksObjects = locksObjects ? YES : NO;
 }
 
 - (BOOL)locksObjects
@@ -422,7 +428,7 @@ setRequiresAllQualifierBindingVariables:requiresAllQualifierBindingVariables
 - (void)setRefreshesRefetchedObjects: (BOOL)refreshesRefetchedObjects
 {
   [self willChange];  
-  _flags.refreshesRefetchedObjects = refreshesRefetchedObjects;
+  _flags.refreshesRefetchedObjects = refreshesRefetchedObjects ? YES : NO;
 }
 
 - (BOOL)refreshesRefetchedObjects
@@ -444,7 +450,7 @@ setRequiresAllQualifierBindingVariables:requiresAllQualifierBindingVariables
 - (void)setPromptsAfterFetchLimit: (BOOL)promptsAfterFetchLimit
 {
   [self willChange];  
-  _flags.promptsAfterFetchLimit = promptsAfterFetchLimit;
+  _flags.promptsAfterFetchLimit = promptsAfterFetchLimit ? YES : NO;
 }
 
 - (BOOL)promptsAfterFetchLimit
@@ -452,14 +458,14 @@ setRequiresAllQualifierBindingVariables:requiresAllQualifierBindingVariables
   return _flags.promptsAfterFetchLimit;
 }
 
-- (void)setAllVariablesRequiredFromBindings: (BOOL)allVariablesRequired
+- (void)setRequiresAllQualifierBindingVariables: (BOOL)flag
 {
-  _flags.allVariablesRequiredFromBindings = allVariablesRequired;
+  _flags.requiresAllQualifierBindingVariables = flag ? YES : NO;
 }
 
-- (BOOL)allVariablesRequiredFromBindings
+- (BOOL)requiresAllQualifierBindingVariables
 {
-  return _flags.allVariablesRequiredFromBindings;
+  return _flags.requiresAllQualifierBindingVariables;
 }
 
 - (void)setPrefetchingRelationshipKeyPaths: (NSArray *)prefetchingRelationshipKeys
@@ -537,20 +543,20 @@ setRequiresAllQualifierBindingVariables:requiresAllQualifierBindingVariables
       if (fetchLimit != 0)
         {
           [mutableHints setObject: [NSNumber numberWithInt: fetchLimit]
-			forKey: @"EOFetchLimitHintKey"];
+			forKey: EOFetchLimitHintKey];
         }
 
       if (promptsAfterFetchLimit)
         {
 	  [mutableHints setObject: [NSNumber numberWithBool:
 					       promptsAfterFetchLimit]
-			forKey: @"EOPromptAfterFetchLimitHintKey"];
+			forKey: EOPromptAfterFetchLimitHintKey];
         }
 
       if ([prefetchingRelationshipKeyPaths count] > 0)
         {
 	  [mutableHints setObject: prefetchingRelationshipKeyPaths
-			forKey: @"EOPrefetchingRelationshipHintKey"];
+			forKey: EOPrefetchingRelationshipHintKey];
         }
     }
 
@@ -568,16 +574,20 @@ setRequiresAllQualifierBindingVariables:requiresAllQualifierBindingVariables
   ASSIGN(_rawAttributeKeys, rawRowKeyPaths);
 }
 
-- (BOOL)requiresAllQualifierBindingVariables
+@end
+
+@implementation EOFetchSpecification (deprecated)
+- (BOOL)allVariablesRequiredFromBindings
 {
-  NSDebugMLLog(@"gsdb", @"self=%p", self);
-  return _flags.allVariablesRequiredFromBindings;
+  NSLog(@"DEPRECATED: Use requiresAllQualifierBindingVariables");
+  return [self requiresAllQualifierBindingVariables];
 }
 
-- (void)setRequiresAllQualifierBindingVariables: (BOOL)flag
+- (void)setAllVariablesRequiredFromBindings: (BOOL)flag
 {
-  [self willChange];  
-  [self notImplemented: _cmd];
+  NSLog(@"DEPRECATED: Use setRequiresAllQualifierBindingVariables:");
+  [self setRequiresAllQualifierBindingVariables: flag];
 }
 
 @end
+
