@@ -1,13 +1,13 @@
 /** 
    EOGlobalID.m <title>EOGlobalID</title>
 
-   Copyright (C) 2000-2003 Free Software Foundation, Inc.
+   Copyright (C) 2000-2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: February 2000
 
-   $Revision: 1.12 $
-   $Date: 2004/04/18 08:16:23 $
+   $Revision: 22681 $
+   $Date: 2006-03-19 19:03:50 +0100 (Son, 19 Mär 2006) $
 
    <abstract></abstract>
 
@@ -27,13 +27,13 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
 #include "config.h"
 
-RCS_ID("$Id: EOGlobalID.m,v 1.12 2004/04/18 08:16:23 ayers Exp $")
+RCS_ID("$Id: EOGlobalID.m 22681 2006-03-19 18:03:50Z ratmice $")
 
 #ifdef GNUSTEP
 #include <Foundation/NSCoder.h>
@@ -87,9 +87,7 @@ NSString *EOGlobalIDChangedNotification = @"EOGlobalIDChangedNotification";
 
 - (id)copyWithZone: (NSZone *)zone
 {
-  EOGlobalID *gid = [[[self class] alloc] init];
-
-  return gid;
+  return RETAIN(self);
 }
 
 @end
@@ -99,7 +97,6 @@ NSString *EOGlobalIDChangedNotification = @"EOGlobalIDChangedNotification";
 
 static unsigned short sequence = (unsigned short)-1;
 
-
 + (EOTemporaryGlobalID *)temporaryGlobalID
 {
   return [[[self alloc] init] autorelease];
@@ -223,14 +220,28 @@ static unsigned short sequence = (unsigned short)-1;
   return self;
 }
 
-- (id)copyWithZone: (NSZone *)zone
+- (NSString *)description
 {
-  EOTemporaryGlobalID *gid = [super copyWithZone:zone];
+  unsigned char dst[(EOUniqueBinaryKeyLength<<1)   /* 2 x buffer */
+		    + (EOUniqueBinaryKeyLength>>2) /* + 1 space per 4 byte */
+		    + 1];                          /* + terminator */
+  unsigned int i,j;
 
-  gid->_refCount = _refCount;
-  memcpy(gid->_bytes, _bytes, sizeof(_bytes));
+  #define num2char(num) ((num) < 0xa ? ((num)+'0') : ((num)+0x57))
+  for (i = 0, j = 0; i < EOUniqueBinaryKeyLength; i++, j++)
+    {
+      dst[j] = num2char((_bytes[i]>>4) & 0x0f);
+      dst[++j] = num2char(_bytes[i] & 0x0f);
 
-  return gid;
+      /* Insert a space per 4 bytes.  */
+      if ((i & 3) == 3 && i < EOUniqueBinaryKeyLength-1)
+	{
+	  dst[++j] = ' ';
+	}
+    }
+  dst[j] = 0;
+  return [NSString stringWithFormat: @"<%s %s>",
+		   GSClassNameFromObject(self), dst];
 }
 
 @end
