@@ -1,13 +1,13 @@
 /** 
    EOKeyComparisonQualifier.m <title>EOKeyComparisonQualifier</title>
 
-   Copyright (C) 2000 Free Software Foundation, Inc.
+   Copyright (C) 2000,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: February 2000
 
-   $Revision: 1.14 $
-   $Date: 2004/03/19 16:41:20 $
+   $Revision: 21766 $
+   $Date: 2005-10-02 15:56:00 +0200 (Son, 02 Okt 2005) $
 
    <abstract></abstract>
 
@@ -27,13 +27,13 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
 #include "config.h"
 
-RCS_ID("$Id: EOKeyComparisonQualifier.m,v 1.14 2004/03/19 16:41:20 ayers Exp $")
+RCS_ID("$Id: EOKeyComparisonQualifier.m 21766 2005-10-02 13:56:00Z ayers $")
 
 #ifdef GNUSTEP
 #include <Foundation/NSDictionary.h>
@@ -236,6 +236,22 @@ RCS_ID("$Id: EOKeyComparisonQualifier.m,v 1.14 2004/03/19 16:41:20 ayers Exp $")
  * Returns a human readable representation of the receiver.
  */
 - (NSString *) description
+{
+  NSString *selectorString;
+  selectorString = [isa stringForOperatorSelector: _selector];
+  if (selectorString == nil)
+    {
+      selectorString = NSStringFromSelector(_selector);
+    }
+  return [NSString stringWithFormat:@"(%@ %@ %@)",
+		   _leftKey,
+		   selectorString,
+		   _rightKey];
+}
+/**
+ * Deprecated debug description.  Expect it to be removed.
+ */
+- (NSString *) debugDescription
 {
   NSString *selectorString;
   selectorString = [isa stringForOperatorSelector: _selector];
@@ -251,5 +267,30 @@ RCS_ID("$Id: EOKeyComparisonQualifier.m,v 1.14 2004/03/19 16:41:20 ayers Exp $")
 		   _rightKey];
 }
 
+- (void)addQualifierKeysToSet: (NSMutableSet *)keys
+{
+  [keys addObject: _leftKey];
+}
+
+- (id) initWithKeyValueUnarchiver: (EOKeyValueUnarchiver*)unarchiver
+{
+  EOFLOGObjectFnStartOrCond(@"EOQualifier");
+  
+  if ((self = [self init]))
+  {
+    NSString *selectorName = [unarchiver decodeObjectForKey: @"selectorName"];
+    
+    if (selectorName) 
+      _selector = NSSelectorFromString(selectorName);
+    
+    ASSIGN(_leftKey, [unarchiver decodeObjectForKey: @"leftKey"]);
+    ASSIGN(_rightKey, [unarchiver decodeObjectForKey: @"rightKey"]);
+  }
+  
+  EOFLOGObjectFnStopOrCond(@"EOQualifier");
+  
+  return self;
+}
+
 @end
 
