@@ -1,13 +1,13 @@
 /** 
    EOKeyGlobalID.m <title>EOKeyGlobalID Class</title>
 
-   Copyright (C) 2000-2002 Free Software Foundation, Inc.
+   Copyright (C) 2000-2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: June 2000
 
-   $Revision: 1.11 $
-   $Date: 2004/03/19 16:41:20 $
+   $Revision: 22179 $
+   $Date: 2005-12-18 23:34:54 +0100 (Son, 18 Dez 2005) $
 
    <abstract></abstract>
 
@@ -27,13 +27,13 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
 #include "config.h"
 
-RCS_ID("$Id: EOKeyGlobalID.m,v 1.11 2004/03/19 16:41:20 ayers Exp $")
+RCS_ID("$Id: EOKeyGlobalID.m 22179 2005-12-18 22:34:54Z ayers $")
 
 #ifdef GNUSTEP
 #include <Foundation/NSString.h>
@@ -54,9 +54,20 @@ RCS_ID("$Id: EOKeyGlobalID.m,v 1.11 2004/03/19 16:41:20 ayers Exp $")
 #include <EOControl/EODebug.h>
 #include <EOControl/EONull.h>
 
+#include "EOPrivate.h"
+
 
 @implementation EOKeyGlobalID
 
++ (void) initialize
+{
+  static BOOL initialized=NO;
+  if (!initialized)
+    {
+      GDL2_PrivateInit();
+    };
+};
+
 + (id)globalIDWithEntityName: (NSString *)entityName
 			keys: (id *)keys
 		    keyCount: (unsigned)count
@@ -118,7 +129,7 @@ RCS_ID("$Id: EOKeyGlobalID.m,v 1.11 2004/03/19 16:41:20 ayers Exp $")
   return [NSArray arrayWithObjects: _keyValues count: _keyCount];
 }
 
-- (BOOL)isEqual: other
+- (BOOL)isEqual: (id)other
 {
   unsigned short oCount;
   int i;
@@ -150,9 +161,9 @@ RCS_ID("$Id: EOKeyGlobalID.m,v 1.11 2004/03/19 16:41:20 ayers Exp $")
   unsigned int hash = 0;
 
   for (i = 0; i < _keyCount; i++)
-    hash += [_keyValues[i] hash];
+    hash ^= [_keyValues[i] hash];
 
-  hash += [_entityName hash];
+  hash ^= [_entityName hash];
 
   return hash;
 }
@@ -177,25 +188,6 @@ RCS_ID("$Id: EOKeyGlobalID.m,v 1.11 2004/03/19 16:41:20 ayers Exp $")
   return self;
 }
 
-- (id)copyWithZone: (NSZone *)zone
-{
-  EOKeyGlobalID *gid = [super copyWithZone: zone];
-  int i;
-
-  ASSIGN(gid->_entityName, _entityName);
-  gid->_keyCount = _keyCount;
-
-  gid->_keyValues = NSZoneMalloc(zone, _keyCount * sizeof(id));
-
-  for (i = 0; i < _keyCount; i++)
-    {
-      gid->_keyValues[i] = nil;
-      ASSIGN(gid->_keyValues[i], _keyValues[i]);
-    }
-
-  return gid;
-}
-
 - (BOOL) isFinal
 {
 //  [self notImplemented:_cmd]; //TODO
@@ -229,7 +221,7 @@ RCS_ID("$Id: EOKeyGlobalID.m,v 1.11 2004/03/19 16:41:20 ayers Exp $")
   BOOL areNulls = YES;
 
   for (i = 0; areNulls && i < _keyCount; i++)
-    areNulls = isNilOrEONull(_keyValues[i]);
+    areNulls = _isNilOrEONull(_keyValues[i]);
 
   return areNulls;
 }
