@@ -1,7 +1,7 @@
 /* -*-objc-*-
    EOKeyValueArchiver.h
 
-   Copyright (C) 2000 Free Software Foundation, Inc.
+   Copyright (C) 2000,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: February 2000
@@ -21,7 +21,7 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 #ifndef __EOKeyValueArchiving_h__
@@ -60,13 +60,6 @@
 - (void)encodeInt: (int)intValue
            forKey: (NSString *)key;
 
-- (void)_encodeDictionary: (id)dictionary
-		   forKey: (id)key;
-- (void)_encodeObjects: (id)objects
-		forKey: (id)key;
-- (void)_encodeValue: (id)value
-	      forKey: (id)key;
-
 - (NSDictionary *)dictionary;
 
 - (void)setDelegate: (id)delegate;
