@@ -1,7 +1,7 @@
 /* -*-objc-*-
    EOKeyValueCoding.h
 
-   Copyright (C) 2000 Free Software Foundation, Inc.
+   Copyright (C) 2000,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: February 2000
@@ -24,7 +24,7 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 #ifndef __EOKeyValueCoding_h__
@@ -58,7 +58,9 @@
 - (id)valueForKey: (NSString *)key;
 
 /**
- * Unimplemented here.  Relies on NSKeyValueCoding.
+ * Overrides the implementation of gnustep-base/Foundation this method
+ * is currently deprecated in favor of setValue:forKey: yet we aim
+ * to maintain WebObjects 4.5 compatibility.
  */
 - (void)takeValue: (id)value forKey: (NSString *)key;
 
@@ -103,14 +105,16 @@
 
 /**
  * This method is invoked by the EOKeyValueCoding mechanism when an attempt
- * is made to set an null value for a scalar attribute.  This implementation
- * raises an NSInvalidArgument exception. <br/>
- * The NSKeyValueCoding -unableToSetNilForKey: is overriden to invoke this
+ * is made to set an null value for a scalar attribute.
+ * Contrary to the TOC of the documentation, this method is called
+ * unableToSetNilForKey: and not unableToSetNullForKey:<br/>
+ * This implementation raises an NSInvalidArgument exception. <br/>
+ * The NSKeyValueCoding -setNilValueForKey: is overriden to invoke this
  * method instead.  We manipulate the runtime to insure that our implementation
  * of unableToSetNilForKey: is used in favor of the one in gnustep-base or
  * Foundation.
  */
-- (void)unableToSetNullForKey: (NSString *)key;
+- (void)unableToSetNilForKey: (NSString *)key;
 
 @end
 
@@ -122,7 +126,9 @@
 - (id)valueForKeyPath: (NSString *)keyPath;
 
 /**
- * Unimplemented here.  Relies on NSKeyValueCoding.
+ * Overrides the implementation of gnustep-base/Foundation this method
+ * is currently deprecated in favor of setValue:forKeyPath: yet we aim
+ * to maintain WebObjects 4.5 compatibility.
  */
 - (void)takeValue: (id)value forKeyPath: (NSString *)keyPath;
 
@@ -132,7 +138,9 @@
 - (NSDictionary *)valuesForKeys: (NSArray *)keys;
 
 /**
- * Unimplemented here.  Relies on NSKeyValueCoding.
+ * Overrides the implementation of gnustep-base/Foundation this method
+ * is currently deprecated in favor of setValue:forKeyPath: yet we aim
+ * to maintain WebObjects 4.5 compatibility.
  */
 - (void)takeValuesFromDictionary: (NSDictionary *)dictionary; 
 
