@@ -1,7 +1,7 @@
 /** 
    EOKeyValueCoding.m <title>EOKeyValueCoding</title>
 
-   Copyright (C) 1996-2002, 2003 Free Software Foundation, Inc.
+   Copyright (C) 1996-2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mircea Oancea <mircea@jupiter.elcom.pub.ro>
    Date: November 1996
@@ -34,13 +34,13 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
 #include "config.h"
 
-RCS_ID("$Id: EOKeyValueCoding.m,v 1.17 2004/03/19 16:41:20 ayers Exp $")
+RCS_ID("$Id: EOKeyValueCoding.m 22805 2006-04-24 06:38:15Z ayers $")
 
 #ifdef GNUSTEP
 #include <Foundation/NSObject.h>
@@ -71,83 +71,57 @@ RCS_ID("$Id: EOKeyValueCoding.m,v 1.17 2004/03/19 16:41:20 ayers Exp $")
 
 #include <GNUstepBase/GSObjCRuntime.h>
 
-static EONull *null = nil;
-static SEL     oaiSel;
+#include "EOPrivate.h"
+
 static BOOL    strictWO;
+static BOOL initialized=NO;
 
 static inline void
 initialize(void)
 {
-  if (null == nil)
+  if (!initialized)
     {
-      null     = [EONull null];
-      oaiSel   = @selector(objectAtIndex:);
+      initialized=YES;
       strictWO = GSUseStrictWO451Compatibility(nil);
+      GDL2_PrivateInit();
     }
 }
 
 /* This macro is only used locally in defined places so for the sake
    of efficiency, we don't use the do {} while (0) pattern.  */
-#define INITIALIZE if (null == nil) initialize();
+#define INITIALIZE if (!initialized) initialize();
 
-/*
- * This dummy class exists to provide a replacement implementation for
- * NSObject -unableToSetNilForKey:, which calls -unableToSetNullForKey:
- * as defined in WO4.5.  We need this mechanism as a category cannot
- * reliably override the category in gnustep-base or Foundation.
- */
-@interface NilToNull : NSObject
-@end
-@interface NilToNull (SurrpressWarning)
-- (void) unableToSetNullForKey: (NSString *)key;
-@end
-@implementation NilToNull
-+ (void)load
-{
-  Class cls;
-  SEL sel;
-  IMP imp;
-  GSMethod method;
-
-  imp = NULL;
-  sel = @selector(unableToSetNilForKey:);
-  cls = GSClassFromName("NSObject");
-
-  method = GSGetInstanceMethodNotInherited(self, sel);
-  if (method != METHOD_NULL)
-    {
-      imp = method->method_imp;
-    }
-  else
-    {
-      fprintf(stderr,
-	      "%s: Could not find method unableToSetNilForKey: in NilToNil!\n",
-	      __FILE__);
-      abort();
-    }
-
-  method = GSGetInstanceMethodNotInherited(cls, sel);
-  if (method != METHOD_NULL)
-    {
-      method->method_imp = imp;
-    }
-  else
-    {
-      fprintf(stderr,
-	      "%s: Could not find method unableToSetNilForKey: in NSObject!\n",
-	      __FILE__);
-      abort();
-    }
-  GSFlushMethodCacheForClass(cls);
-}
-
-- (void) unableToSetNilForKey: (NSString *)key
-{
-  [self unableToSetNullForKey: key];
-}
-@end
 
 @implementation NSObject (_EOKeyValueCodingCompatibility)
+
+- (void)GDL2KVCNSObjectICategoryID
+{
+}
+
++ (void)load
+{
+  GDL2_ActivateCategory("NSObject",
+			@selector(GDL2KVCNSObjectICategoryID), YES);
+}
+
+
+/* This is what -base(add) will call.  It should invoke what the API
+   specifies should be overridden.  */
+- (void) setNilValueForKey: (NSString*)aKey
+{
+  [self unableToSetNilForKey: aKey];
+}
+
+
+/* This is what should be overridden according to the API.*/
+- (void) unableToSetNilForKey: (NSString *)key
+{
+  [NSException raise: NSInvalidArgumentException
+	       format: @"%@ -- %@ 0x%x: Given nil value to set for key \"%@\"",
+	       NSStringFromSelector(_cmd), NSStringFromClass([self class]), 
+	       self, key];
+}
+
 /* See EODeprecated.h. */
 + (void) flushClassKeyBindings
 {
@@ -158,14 +132,93 @@ initialize(void)
 {
 }
 
-/* See header file for documentation. */
-- (void) unableToSetNullForKey: (NSString *)key
+- (void) takeValue: (id)anObject forKey: (NSString*)aKey
 {
-  [NSException raise: NSInvalidArgumentException
-	       format: @"%@ -- %@ 0x%x: Given nil value to set for key \"%@\"",
-	       NSStringFromSelector(_cmd), NSStringFromClass([self class]), 
-	       self, key];
+  SEL		sel = 0;
+  const char	*type = 0;
+  int		off;
+  unsigned	size = [aKey length];
+
+  if (size > 0)
+    {
+      const char	*name;
+      char		buf[size+6];
+      char		lo;
+      char		hi;
+
+      strcpy(buf, "_set");
+      [aKey getCString: &buf[4]];
+      lo = buf[4];
+      hi = islower(lo) ? toupper(lo) : lo;
+      buf[4] = hi;
+      buf[size+4] = ':';
+      buf[size+5] = '\0';
+
+      name = &buf[1];	// setKey:
+      type = NULL;
+      sel = GSSelectorFromName(name);
+      if (sel == 0 || [self respondsToSelector: sel] == NO)
+	{
+	  name = buf;	// _setKey:
+	  sel = GSSelectorFromName(name);
+	  if (sel == 0 || [self respondsToSelector: sel] == NO)
+	    {
+	      sel = 0;
+	      if ([[self class] accessInstanceVariablesDirectly] == YES)
+		{
+		  buf[size+4] = '\0';
+		  buf[3] = '_';
+		  buf[4] = lo;
+		  name = &buf[4];	// key
+		  if (GSObjCFindVariable(self, name, &type, &size, &off) == NO)
+		    {
+		      name = &buf[3];	// _key
+		      GSObjCFindVariable(self, name, &type, &size, &off);
+		    }
+		}
+	    }
+	}
+    }
+  GSObjCSetVal(self, [aKey UTF8String], anObject, sel, type, size, off);
 }
+
+
+- (void) takeValue: (id)anObject forKeyPath: (NSString*)aKey
+{
+  NSRange	r = [aKey rangeOfString: @"."];
+
+  if (r.length == 0)
+    {
+      [self takeValue: anObject forKey: aKey];
+    }
+  else
+    {
+      NSString	*key = [aKey substringToIndex: r.location];
+      NSString	*path = [aKey substringFromIndex: NSMaxRange(r)];
+
+      [[self valueForKey: key] takeValue: anObject forKeyPath: path];
+    }
+}
+
+
+- (void) takeValuesFromDictionary: (NSDictionary*)aDictionary
+{
+  NSEnumerator	*enumerator = [aDictionary keyEnumerator];
+  NSNull	*null = [NSNull null];
+  NSString	*key;
+
+  while ((key = [enumerator nextObject]) != nil)
+    {
+      id obj = [aDictionary objectForKey: key];
+
+      if (obj == null)
+	{
+	  obj = nil;
+	}
+      [self takeValue: obj forKey: key];
+    }
+}
+
 @end
 
 
@@ -184,6 +237,16 @@ initialize(void)
 
 @implementation NSArray (EOKeyValueCoding)
 
+- (void)GDL2KVCNSArrayICategoryID
+{
+}
+
++ (void)load
+{
+  GDL2_ActivateCategory("NSArray",
+			@selector(GDL2KVCNSArrayICategoryID), YES);
+}
+
 /**
  * EOKeyValueCoding protocol<br/>
  * This overrides NSObjects implementation of this method.
@@ -191,7 +254,8 @@ initialize(void)
  * returned by invoking [NSObject-valueForKey:]
  * for each item in the receiver, substituting EONull for nil.
  * Keys formated like "@function.someKey" are resolved by invoking
- * [NSArray-computeFuncionWithKey:] "someKey" on the reviever.
+ * [NSArray-computeFuncionWithKey:] "someKey" on the receiver.
+ * If the key is omitted, the function will be called with nil.
  * The following functions are supported by default:
  * <list>
  *  <item>@sum   -> -computeSumForKey:</item>
@@ -200,10 +264,12 @@ initialize(void)
  *  <item>@min   -> -computeMinForKey:</item>
  *  <item>@count -> -computeCountForKey:</item>
  * </list>
- * As a special case the @count function does not require a key,
- * in fact, any key supplied is ignored.
- * As another special case the key "count" is not forwarded to each object
- * of the receiver but returns the number of objects of the receiver.<br/>
+ * Computational components generally expect a key to be passed to
+ * the function.  This is not mandatory in which case 'nil' will be supplied.
+ * (i.e. you may use "@myFuncWhichCanHandleNil" as a key.)
+ * As a special case the key "count" does not actually invoke
+ * computeCountForKey: on receiver but returns the number of objects of 
+ * the receiver.<br/>
  * There is no special handling of EONull.  Therefore expect exceptions
  * on EONull not responding to decimalValue and compare: when the are
  * used with this mechanism. 
@@ -222,27 +288,40 @@ initialize(void)
   else if ([key hasPrefix:@"@"])
     {
       NSString *selStr;
+      NSString *attrStr;
       SEL       sel;
       NSRange   r;
 
       r = [key rangeOfString:@"."];
-      NSAssert(r.location!=NSNotFound,
-	       @"Invalid computational key structure");
-      r.length   = r.location - 1; /* set length of key (w/o @) */
-      r.location = 1;              /* remove leading '@' */
-      selStr = [NSString stringWithFormat: @"compute%@ForKey:",
-		   [[key substringWithRange: r] capitalizedString]];
-      sel = NSSelectorFromString(selStr);
-      NSAssert(sel!=NULL,@"Invalid computational key");
+      if (r.location == NSNotFound)
+	{
+	  r.length   = [key length] - 1; /* set length of key (w/o @) */
+	  r.location = 1;                /* remove leading '@' */
+	  attrStr = nil;
+	}
+      else
+	{
+	  r.length  = r.location - 1;    /* set length of key (w/o @) */
+	  r.location = 1;                /* remove leading '@' */
+                                         /* skip located '.' */
+	  attrStr = [key substringFromIndex: NSMaxRange(r) + 1];
+	}
 
-      result = [self performSelector: sel               /* skip located '.' */
-		     withObject: [key substringFromIndex: NSMaxRange(r) + 1]];
+      selStr = [NSString stringWithFormat: @"compute%@ForKey:",
+		   [[key substringWithRange: r] initialCapitalizedString]];
+      sel = NSSelectorFromString(selStr);
+      NSAssert2(sel!=NULL,@"Invalid computational key: '%@' Selector: '%@'",
+                key,
+                selStr);
+
+      result = [self performSelector: sel
+		     withObject: attrStr];
     }
   else
     {
       result = [self resultsOfPerformingSelector: @selector(valueForKey:)
 		     withObject: key
-		     defaultResult: null];
+		     defaultResult: GDL2_EONull];
     }
 
   EOFLOGObjectFnStopCond(@"EOKVC");
@@ -253,19 +332,17 @@ initialize(void)
  * EOKeyValueCoding protocol<br/>
  * Returns the object returned by invoking [NSObject-valueForKeyPath:]
  * on the object returned by invoking [NSObject-valueForKey:]
- * on the reciever with the first key component supplied by the key path,
+ * on the receiver with the first key component supplied by the key path,
  * with rest of the key path.<br/>
  * If the first component starts with "@", the first component includes the key
  * of the computational key component and as the form "@function.key".
  * If there is only one key component, this method invokes 
  * [NSObject-valueForKey:] in the receiver with that component.
- * All computational components are expected to specifiy a key with the
- * exception of @count, in which case the key maybe omitted.
  * Unlike the reference implementation GDL2 allows you to continue the keyPath
- * in a meaningfull way after @count but the path must then contain a key as
+ * in a meaningful way after @count but the path must then contain a key as
  * the computational key structure implies.
  * (i.e. you may use "@count.self.decimalValue") The actual key "self" is
- * infact ignored during the computation, but the formal structure must be
+ * in fact ignored during the computation, but the formal structure must be
  * maintained.<br/>
  * It should be mentioned that the reference implementation
  * would return the result of "@count" independent
@@ -281,15 +358,14 @@ initialize(void)
 
   EOFLOGObjectFnStartCond(@"EOKVC");
   r = [keyPath rangeOfString: @"."];
-  if ([keyPath hasPrefix: @"@"] == YES &&
-      [keyPath isEqualToString: @"@count"] == NO)
+  if ([keyPath hasPrefix: @"@"] == YES
+      && [keyPath isEqualToString: @"@count"] == NO
+      && r.location != NSNotFound)
     {
       NSRange rr;
       unsigned length;
 
       length = [keyPath length];
-      NSAssert1(r.location!=NSNotFound && length > r.location,
-		@"invalid computational keyPath:%@", keyPath);
 
       rr.location = NSMaxRange(r);
       rr.length   = length - rr.location;
@@ -321,26 +397,30 @@ initialize(void)
  */
 - (id)computeSumForKey: (NSString *)key
 {
-  NSDecimalNumber *ret;
+  NSDecimalNumber *ret=nil;
   NSDecimal        result, left, right;
   NSRoundingMode   mode;
-  unsigned int     i, count;
-  IMP              oai;
+  unsigned int     count;
 
   INITIALIZE;
 
   EOFLOGObjectFnStartCond(@"EOKVC");
+
   mode = [[NSDecimalNumber defaultBehavior] roundingMode];
-  oai = [self methodForSelector: oaiSel];
   count = [self count];
   NSDecimalFromComponents(&result, 0, 0, NO);
 
-  for (i=0; i<count; i++)
+  if (count>0)
     {
-      left = result;
-      right = [[(*oai)(self, oaiSel, i) valueForKey: key] decimalValue];
-      NSDecimalAdd(&result, &left, &right, mode);
-    }
+      unsigned int i=0;
+      IMP oaiIMP = [self methodForSelector: @selector(objectAtIndex:)];
+      for (i=0; i<count; i++)
+        {
+          left = result;
+          right = [[GDL2_ObjectAtIndexWithImp(self,oaiIMP,i) valueForKey: key] decimalValue];
+          NSDecimalAdd(&result, &left, &right, mode);
+        }
+    };
         
   ret = [NSDecimalNumber decimalNumberWithDecimal: result];
   EOFLOGObjectFnStopCond(@"EOKVC");
@@ -355,26 +435,30 @@ initialize(void)
  */
 - (id)computeAvgForKey: (NSString *)key
 {
-  NSDecimalNumber *ret;
+  NSDecimalNumber *ret = nil;
   NSDecimal        result, left, right;
   NSRoundingMode   mode;
-  unsigned int     i, count;
-  IMP              oai;
+  unsigned int     count = 0;
 
   INITIALIZE;
 
   EOFLOGObjectFnStartCond(@"EOKVC");
   mode = [[NSDecimalNumber defaultBehavior] roundingMode];
-  oai = [self methodForSelector: oaiSel];
   count = [self count];
   NSDecimalFromComponents(&result, 0, 0, NO);
 
-  for (i=0; i<count; i++)
+  if (count>0)
     {
-      left = result;
-      right = [[(*oai)(self, oaiSel, i) valueForKey: key] decimalValue];
-      NSDecimalAdd(&result, &left, &right, mode);
-    }
+      unsigned int i=0;
+      IMP oaiIMP = [self methodForSelector: @selector(objectAtIndex:)];
+      
+      for (i=0; i<count; i++)
+        {
+          left = result;
+          right = [[GDL2_ObjectAtIndexWithImp(self,oaiIMP,i) valueForKey: key] decimalValue];
+          NSDecimalAdd(&result, &left, &right, mode);
+        }
+    };
 
   left  = result;
   NSDecimalFromComponents(&right, (unsigned long long) count, 0, NO);
@@ -399,33 +483,34 @@ initialize(void)
 
 - (id)computeMaxForKey: (NSString *)key
 {
-  id           result, resultVal;
-  unsigned int i, count;;
+  id result=nil;
+  id resultVal=nil;
+  unsigned int count=0;
 
   INITIALIZE;
 
   EOFLOGObjectFnStartCond(@"EOKVC");
-  result    = nil;
-  resultVal = nil;
   count     = [self count];
 
   if (count > 0)
     {
-      id                 current,currentVal;
-      IMP                oai;
+      unsigned int i=0;
+      id           current = nil;
+      id	   currentVal = nil;
+      IMP          oaiIMP = [self methodForSelector: @selector(objectAtIndex:)];
 
-      oai = [self methodForSelector: oaiSel];
-      for(i=0; i<count && (resultVal == nil || resultVal == null); i++)
+      for(i=0; i<count && (resultVal == nil || resultVal == GDL2_EONull); i++)
 	{
-	  result    = (*oai)(self, oaiSel, i);
+	  result    = GDL2_ObjectAtIndexWithImp(self,oaiIMP,i);
 	  resultVal = [result valueForKey: key];
 	}          
       for (; i<count; i++)
 	{
-	  current    = (*oai)(self, oaiSel, i);
+	  current    = GDL2_ObjectAtIndexWithImp(self,oaiIMP,i);
 	  currentVal = [current valueForKey: key];
 
-	  if (currentVal == nil || currentVal == null) continue;
+	  if (currentVal == nil || currentVal == GDL2_EONull)
+            continue;
 	  
 	  if ([(NSObject *)resultVal compare: currentVal] == NSOrderedAscending)
 	    {
@@ -441,33 +526,33 @@ initialize(void)
 
 - (id)computeMinForKey: (NSString *)key
 {
-  id             result, resultVal;
-  unsigned int   i, count;
+  id result=nil;
+  id resultVal=nil;
+  unsigned int   count = 0;
 
   INITIALIZE;
 
   EOFLOGObjectFnStartCond(@"EOKVC");
-  result    = nil;
-  resultVal = nil;
   count     = [self count];
 
   if (count > 0)
     {
-      id  current, currentVal;
-      IMP oai;
+      id current=nil;
+      id currentVal=nil;
+      unsigned int i = 0;
+      IMP oaiIMP = [self methodForSelector: @selector(objectAtIndex:)];
 
-      oai = [self methodForSelector: oaiSel];
-      for(i=0; i<count && (resultVal == nil || resultVal == null); i++)
+      for(i=0; i<count && (resultVal == nil || resultVal == GDL2_EONull); i++)
 	{
-	  result    = (*oai)(self, oaiSel, i);
+	  result    = GDL2_ObjectAtIndexWithImp(self,oaiIMP,i);
 	  resultVal = [result valueForKey: key];
 	}          
       for (; i<count; i++)
 	{
-	  current    = (*oai)(self, oaiSel, i);
+	  current    = GDL2_ObjectAtIndexWithImp(self,oaiIMP,i);
 	  currentVal = [current valueForKey: key];
 
-	  if (currentVal == nil || currentVal == null) continue;
+	  if (currentVal == nil || currentVal == GDL2_EONull) continue;
 
 	  if ([(NSObject *)resultVal compare: currentVal] == NSOrderedDescending)
 	    {
@@ -486,6 +571,16 @@ initialize(void)
 
 @implementation NSDictionary (EOKeyValueCoding)
 
+- (void)GDL2KVCNSDictionaryICategoryID
+{
+}
+
++ (void)load
+{
+  GDL2_ActivateCategory("NSDictionary",
+			@selector(GDL2KVCNSDictionaryICategoryID), YES);
+}
+
 /**
  * Returns the object stored in the dictionary for this key.
  * Unlike Foundation, this method may return objects for keys other than
@@ -784,6 +879,16 @@ initialize(void)
 
 @implementation NSMutableDictionary (EOKVCGNUstepExtensions)
 
+- (void)GDL2KVCNSMutableDictionaryICategoryID
+{
+}
+
++ (void)load
+{
+  GDL2_ActivateCategory("NSMutableDictionary",
+			@selector(GDL2KVCNSMutableDictionaryICategoryID), YES);
+}
+
 /**
  * Method to augment the NSKeyValueCoding implementation
  * to account for added functionality such as quoted key paths.
@@ -838,9 +943,11 @@ initialize(void)
 					mutableCopy] autorelease];
       NSMutableString *key = [NSMutableString string];
 
+      unsigned keyPathArrayCount = [keyPathArray count];
+
       //EOFLOGObjectLevelArgs(@"EOKVC", @"keyPathArray=%@", keyPathArray);
 
-      while ([keyPathArray count] > 0)
+      while (keyPathArrayCount > 0)
         {
           id tmpKey;
 
@@ -850,6 +957,7 @@ initialize(void)
           //EOFLOGObjectLevelArgs(@"EOKVC", @"tmpKey=%@", tmpKey);
 
           [keyPathArray removeObjectAtIndex: 0];
+          keyPathArrayCount--;
 
           if ([key length] > 0)
             [key appendString: @"."];
@@ -871,7 +979,7 @@ initialize(void)
       //EOFLOGObjectLevelArgs(@"EOKVC",@"left keyPathArray=\"%@\"",
       //             keyPathArray);
 
-      if ([keyPathArray count] > 0)
+      if (keyPathArrayCount > 0)
         {
           id obj = [self objectForKey: key];
 
@@ -936,9 +1044,11 @@ initialize(void)
 					mutableCopy] autorelease];
       NSMutableString *key = [NSMutableString string];
 
+      int keyPathArrayCount=[keyPathArray count];
+
       //EOFLOGObjectLevelArgs(@"EOKVC", @"keyPathArray=%@", keyPathArray);
 
-      while ([keyPathArray count] > 0)
+      while (keyPathArrayCount > 0)
         {
           id tmpKey;
 
@@ -948,6 +1058,7 @@ initialize(void)
           //EOFLOGObjectLevelArgs(@"EOKVC", @"tmpKey=%@", tmpKey);
 
           [keyPathArray removeObjectAtIndex: 0];
+          keyPathArrayCount--;
 
           if ([key length] > 0)
             [key appendString: @"."];
@@ -968,7 +1079,7 @@ initialize(void)
       //EOFLOGObjectLevelArgs(@"EOKVC",@"left keyPathArray=\"%@\"",
       //             keyPathArray);
 
-      if ([keyPathArray count] > 0)
+      if (keyPathArrayCount > 0)
         {
           id obj = [self objectForKey: key];
 
@@ -1054,7 +1165,7 @@ initialize(void)
 
       if (val == nil)
 	{
-	  val = null;
+	  val = GDL2_EONull;
 	}
 
       [newKeyPaths addObject: keyPath];
@@ -1172,7 +1283,7 @@ initialize(void)
       NS_ENDHANDLER;
         
       if (val == nil)
-	val = null;
+	val = GDL2_EONull;
       
       [newKeyPaths addObject: keyPath];
       [newVals addObject: val];
