@@ -9,8 +9,8 @@
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: February 2000
 
-   $Revision: 1.4 $
-   $Date: 2002/12/31 16:25:10 $
+   $Revision: 21645 $
+   $Date: 2005-08-17 10:07:57 +0200 (Mit, 17 Aug 2005) $
 
    <abstract></abstract>
 
@@ -30,13 +30,13 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
 #include "config.h"
 
-RCS_ID("$Id: EOKeyValueCodingBase.m,v 1.4 2002/12/31 16:25:10 mirko Exp $")
+RCS_ID("$Id: EOKeyValueCodingBase.m 21645 2005-08-17 08:07:57Z ayers $")
 
 #import <Foundation/NSObject.h>
 #import <Foundation/NSArray.h>
