@@ -1,7 +1,7 @@
 /** 
    EOKeyValueQualifier.m <title>EOKeyValueQualifier Class</title>
 
-   Copyright (C) 2000-2002 Free Software Foundation, Inc.
+   Copyright (C) 2000-2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: February 2000
@@ -9,8 +9,8 @@
    Author: Manuel Guesdon <mguesdon@orange-concept.com>
    Date: November 2001
 
-   $Revision: 1.16 $
-   $Date: 2004/03/19 16:41:20 $
+   $Revision: 21766 $
+   $Date: 2005-10-02 15:56:00 +0200 (Son, 02 Okt 2005) $
 
    <abstract></abstract>
 
@@ -30,17 +30,18 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
 #include "config.h"
 
-RCS_ID("$Id: EOKeyValueQualifier.m,v 1.16 2004/03/19 16:41:20 ayers Exp $")
+RCS_ID("$Id: EOKeyValueQualifier.m 21766 2005-10-02 13:56:00Z ayers $")
 
 #ifdef GNUSTEP
 #include <Foundation/NSString.h>
 #include <Foundation/NSDictionary.h>
+#include <Foundation/NSSet.h>
 #include <Foundation/NSObjCRuntime.h>
 #include <Foundation/NSException.h>
 #else
@@ -61,6 +62,8 @@ RCS_ID("$Id: EOKeyValueQualifier.m,v 1.16 2004/03/19 16:41:20 ayers Exp $")
 #include <EOControl/EONull.h>
 #include <EOControl/EODebug.h>
 
+#include "EOPrivate.h"
+
 /*
   This declaration is needed by the compiler to state that
   eventhough we know not all objects respond to -compare:,
@@ -80,10 +83,16 @@ RCS_ID("$Id: EOKeyValueQualifier.m,v 1.16 2004/03/19 16:41:20 ayers Exp $")
 @end
 
 @implementation EOKeyValueQualifier
-static EONull *null = nil;
+
 + (void)initialize
 {
-  null = [EONull null];
+  static BOOL initialized=NO;
+  if (!initialized)
+    {
+      initialized=YES;
+
+      GDL2_PrivateInit();
+    };
 }
 
 /**
@@ -120,7 +129,7 @@ static EONull *null = nil;
       ASSIGNCOPY(_key, key);
       if (value == nil)
 	{
-	  value = null;
+	  value = GDL2_EONull;
 	}
       ASSIGN(_value, value);
     }
@@ -199,59 +208,99 @@ static EONull *null = nil;
  */
 - (BOOL) evaluateWithObject: (id)object
 {
-  NSObject *val;
+  NSObject *objectValue;
+  NSObject *selfValue=_value;
   BOOL (*imp)(id, SEL, id);
 
-  val = [object valueForKey: _key];
+  objectValue = [object valueForKey: _key];
 
-  if (val == nil)
+  if (objectValue == nil)
     {
-      val = null;
+      objectValue = GDL2_EONull;
+    }
+  if (selfValue == nil)
+    {
+      selfValue = GDL2_EONull;
     }
 
-  imp = (BOOL (*)(id, SEL, id))[val methodForSelector: _selector];
+  imp = (BOOL (*)(id, SEL, id))[objectValue methodForSelector: _selector];
   if (imp != NULL)
     {
-      return (*imp) (val, _selector, _value);
+      return (*imp) (objectValue, _selector, selfValue);
     }
   if (sel_eq(_selector, EOQualifierOperatorEqual) == YES)
     {
-      return [val isEqual: _value];
+      return [objectValue isEqual: selfValue];
     }
   else if (sel_eq(_selector, EOQualifierOperatorNotEqual) == YES)
     {
-      return ([val isEqual: _value]?NO:YES);
+      return ([objectValue isEqual: selfValue]?NO:YES);
     }
   else if (sel_eq(_selector, EOQualifierOperatorLessThan) == YES)
     {
-      return [val compare: _value] == NSOrderedAscending;
+      if (objectValue==GDL2_EONull)
+        return ((selfValue==GDL2_EONull) ? NO : YES);
+      else if (selfValue==GDL2_EONull)
+        return NO;
+      else
+        return [objectValue compare: selfValue] == NSOrderedAscending;
     }
   else if (sel_eq(_selector, EOQualifierOperatorGreaterThan) == YES)
     {
-      return [val compare: _value] == NSOrderedDescending;
+      if (objectValue==GDL2_EONull)
+        return NO;
+      else if (selfValue==GDL2_EONull)
+        return YES;
+      else
+        return [objectValue compare: selfValue] == NSOrderedDescending;
     }
   else if (sel_eq(_selector, EOQualifierOperatorLessThanOrEqualTo) == YES)
     {
-      return [val compare: _value] != NSOrderedDescending;
+      if (objectValue==GDL2_EONull)
+        return YES;
+      else if (selfValue==GDL2_EONull)
+        return NO;
+      else
+        return [objectValue compare: selfValue] != NSOrderedDescending;
     }
   else if (sel_eq(_selector, EOQualifierOperatorGreaterThanOrEqualTo) == YES)
     {
-      return [val compare: _value] != NSOrderedAscending;
+      if (objectValue==GDL2_EONull)
+        return ((selfValue==GDL2_EONull) ? YES : NO);
+      else if (selfValue==GDL2_EONull)
+        return YES;
+      else
+        return [objectValue compare: selfValue] != NSOrderedAscending;
     }
   else if (sel_eq(_selector, EOQualifierOperatorContains) == YES)
     {
-      return [(id)val rangeOfString: _value].location != NSNotFound;
+      //Philosophical question: does nil contains nil ??
+
+      if (objectValue==GDL2_EONull) // Let's say nil does contain nothing (even not nil)
+        return NO;
+      else if (selfValue==GDL2_EONull) // Let's say nil is contained by nothing
+        return NO;
+      else
+        return [(NSString*)objectValue rangeOfString: 
+                             (NSString*)selfValue].location != NSNotFound;
     }
   else if (sel_eq(_selector, EOQualifierOperatorLike) == YES)
     {
       NSEmitTODO();  //TODO
-      return [val isEqual: _value] == NSOrderedSame;
+      //How to handle nil like ?
+      return [objectValue isEqual: selfValue];
     }
   else if (sel_eq(_selector, EOQualifierOperatorCaseInsensitiveLike) == YES)
     {
       NSEmitTODO();  //TODO
-      return [[(id)val uppercaseString] isEqual: [_value uppercaseString]]
-	== NSOrderedSame;
+      //How to handle nil like ?
+      if (objectValue==GDL2_EONull)
+        return ((selfValue==GDL2_EONull) ? YES : NO);
+      else if (selfValue==GDL2_EONull)
+        return NO;
+      else
+        return [(id)objectValue caseInsensitiveCompare: 
+                      (NSString*)selfValue] == NSOrderedSame;
     }
   /*Ayers (09-02-2002): Maybe we should raise instead of returning NO.*/
   return NO;
@@ -261,6 +310,22 @@ static EONull *null = nil;
  * Returns a human readable representation of the receiver.
  */
 - (NSString *) description
+{
+  NSString *selectorString;
+  selectorString = [isa stringForOperatorSelector: _selector];
+  if (selectorString == nil)
+    {
+      selectorString = NSStringFromSelector(_selector);
+    }
+  return [NSString stringWithFormat:@"(%@ %@ '%@')",
+		   _key,
+		   selectorString,
+		   _value];
+}
+/**
+ * Deprecated debug description.  Expect it to be removed.
+ */
+- (NSString *) debugDescription
 {
   NSString *selectorString;
   selectorString = [isa stringForOperatorSelector: _selector];
@@ -319,6 +384,10 @@ static EONull *null = nil;
   return [self notImplemented: _cmd]; //TODO
 }
 
+- (void)addQualifierKeysToSet: (NSMutableSet *)keys
+{
+  [keys addObject: _key];
+}
 @end
 
 @implementation EOKeyValueQualifier (Privat)
