@@ -1,7 +1,7 @@
 /* -*-objc-*-
    EOMultipleKnownKeyDictionary.h
 
-   Copyright (C) 2000 Free Software Foundation, Inc.
+   Copyright (C) 2000,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Manuel Guesdon <mguesdon@orange-concept.com>
    Date: October 2000
@@ -21,7 +21,7 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 #ifndef	__EOMultipleKnownKeyDictionary_h__
@@ -43,7 +43,8 @@
 @class NSMutableDictionary;
 
 @class EOMutableKnownKeyDictionary;
-
+@class EOMKKDArrayMapping;
+@class EOMKKDSubsetMapping;
 
 @interface EOMKKDInitializer : GCObject
 {
@@ -66,11 +67,11 @@
 - (id)objectForIndex: (unsigned int)index
 	  dictionary: (NSDictionary *)dictionary;
 - (unsigned int)indexForKey: (id)key;
-- (id)arrayMappingForKeys: (id)keys;
-- (id)subsetMappingForSourceDictionaryInitializer: (EOMKKDInitializer *)init
-				       sourceKeys: (NSArray *)sourceKeys
-				  destinationKeys: (NSArray *)destinationKeys;
-- (id)subsetMappingForSourceDictionaryInitializer: (id)param0;
+- (EOMKKDArrayMapping *)arrayMappingForKeys: (NSArray *)keys;
+- (EOMKKDSubsetMapping *)subsetMappingForSourceDictionaryInitializer: (EOMKKDInitializer *)init
+							  sourceKeys: (NSArray *)sourceKeys
+						     destinationKeys: (NSArray *)destinationKeys;
+- (EOMKKDSubsetMapping *)subsetMappingForSourceDictionaryInitializer: (EOMKKDInitializer *)param0;
 - (id *)keys;
 - (BOOL)hasKey: (id)key;
 
@@ -159,8 +160,8 @@
 - (unsigned int)count;
 - (id)objectForKey: (id)key;
 - (void)setObject: (id)object
-	   forKey: (NSString *)key;
-- (void)removeObjectForKey: (NSString *)key;
+	   forKey: (id)key;
+- (void)removeObjectForKey: (id)key;
 - (BOOL)containsObjectsNotIdenticalTo: (id)object;
 - (void)addEntriesFromDictionary: (NSDictionary *)dictionary;
 - (NSEnumerator *)keyEnumerator;
