@@ -1,13 +1,13 @@
 /**
    EEOMutableKnownKeyDictionary.m <title>EEOMutableKnownKeyDictionary</title>
 
-   Copyright (C) 2000-2002 Free Software Foundation, Inc.
+   Copyright (C) 2000-2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Manuel Guesdon <mguesdon@orange-concept.com>
    Date: October 2000
 
-   $Revision: 1.12 $
-   $Date: 2004/03/19 16:41:20 $
+   $Revision: 23472 $
+   $Date: 2006-09-12 21:36:24 +0200 (Die, 12 Sep 2006) $
 
    <abstract></abstract>
 
@@ -27,13 +27,13 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
 #include "config.h"
 
-RCS_ID("$Id: EOMutableKnownKeyDictionary.m,v 1.12 2004/03/19 16:41:20 ayers Exp $")
+RCS_ID("$Id: EOMutableKnownKeyDictionary.m 23472 2006-09-12 19:36:24Z ayers $")
 
 #ifdef GNUSTEP
 #include <Foundation/NSArray.h>
@@ -57,9 +57,20 @@ RCS_ID("$Id: EOMutableKnownKeyDictionary.m,v 1.12 2004/03/19 16:41:20 ayers Exp
 #include <EOControl/EODebug.h>
 #include <EOControl/EONull.h>
 
+#include "EOPrivate.h"
 
 @implementation EOMKKDInitializer
 
++ (void)initialize
+{
+  static BOOL initialized=NO;
+  if (!initialized)
+    {
+      initialized=YES;
+      GDL2_PrivateInit();
+    }
+}
+
 + (EOMKKDInitializer*)initializerFromKeyArray: (NSArray*)keys
 {
   EOMKKDInitializer *initializer = [[self newWithKeyArray: keys] autorelease];
@@ -224,7 +235,7 @@ RCS_ID("$Id: EOMutableKnownKeyDictionary.m,v 1.12 2004/03/19 16:41:20 ayers Exp
   return [dictionary objectForKey: key];
 }
 
-- (unsigned int) indexForKey: (NSString*)key
+- (unsigned int) indexForKey: (id)key
 {
   void *index = NSMapGet(_keyToIndex, (const void *)key);
 
@@ -236,15 +247,14 @@ RCS_ID("$Id: EOMutableKnownKeyDictionary.m,v 1.12 2004/03/19 16:41:20 ayers Exp
 
 - (BOOL)hasKey: (id)key
 {
-  return ([self indexForKey: key] != NSNotFound);
+  return (EOMKKDInitializer_indexForKeyWithImpPtr(self,NULL,key) != NSNotFound);
 }
 
 - (EOMKKDArrayMapping*) arrayMappingForKeys: (NSArray*)keys
 {
   int selfKeyCount = [keys count];
   int keyCount = [keys count];
-  EOMKKDArrayMapping *arrayMapping;
-  int i;
+  EOMKKDArrayMapping *arrayMapping = nil;
 
   NSAssert(keyCount <= selfKeyCount, @"key count greater than our key count");
 
@@ -252,13 +262,21 @@ RCS_ID("$Id: EOMutableKnownKeyDictionary.m,v 1.12 2004/03/19 16:41:20 ayers Exp
 				      destinationDescription: self
 				      zone: [self zone]] autorelease];  
 
-  for (i = 0; i < keyCount; i++)
+  if (keyCount>0)
     {
-      NSString *key = [keys objectAtIndex: i];
-      int destinationIndex = [self indexForKey:key];
+      int i=0;
+      GDL2IMP_UINT indexForKeyIMP=NULL;
+      IMP objectAtIndexIMP=[keys methodForSelector:@selector(objectAtIndex:)];
 
-      arrayMapping->_destinationOffsetForArrayIndex[i] = destinationIndex + 1;
-    }
+      for (i = 0; i < keyCount; i++)
+        {
+          NSString *key = GDL2_ObjectAtIndexWithImp(keys,objectAtIndexIMP,i);
+          int destinationIndex =  EOMKKDInitializer_indexForKeyWithImpPtr(self,&indexForKeyIMP,key);
+          
+          
+          arrayMapping->_destinationOffsetForArrayIndex[i] = destinationIndex + 1;
+        }
+    };
 
   return arrayMapping;
 }
@@ -269,8 +287,7 @@ RCS_ID("$Id: EOMutableKnownKeyDictionary.m,v 1.12 2004/03/19 16:41:20 ayers Exp
 {
   unsigned int selfKeyCount = [self count];
   unsigned int keyCount = [destinationKeys count];
-  EOMKKDSubsetMapping *subsetMapping;
-  int i;
+  EOMKKDSubsetMapping *subsetMapping = nil;
 
   NSAssert([sourceKeys count] == keyCount, @"Source and destination keys count are different");
   NSAssert(keyCount <= selfKeyCount, @"key count greater than our key count");
@@ -285,38 +302,55 @@ RCS_ID("$Id: EOMutableKnownKeyDictionary.m,v 1.12 2004/03/19 16:41:20 ayers Exp
   EOFLOGObjectLevelArgs(@"EOMKKD", @"sourceKeys=%@", sourceKeys);
   EOFLOGObjectLevelArgs(@"EOMKKD", @"destinationKeys=%@", destinationKeys);
 
-  for (i = 0; i < keyCount; i++)
-    {
-      NSString *sourceKey;
-      NSString *destinationKey;
-      int destinationIndex;
-      int sourceIndex;
+  if (keyCount>0)
+      {
+        int i;
+        GDL2IMP_UINT selfIndexForKeyIMP=NULL;
+        GDL2IMP_UINT sourceInitializerIndexForKeyIMP=NULL;
+        IMP sourceObjectAtIndexIMP=[sourceKeys methodForSelector:@selector(objectAtIndex:)];
+        IMP destinationObjectAtIndexIMP=[destinationKeys methodForSelector:@selector(objectAtIndex:)];
 
-      sourceKey = [sourceKeys objectAtIndex: i];
-      EOFLOGObjectLevelArgs(@"EOMKKD", @"sourceKey=%@", sourceKey);
-
-      destinationKey = [destinationKeys objectAtIndex: i];
-      EOFLOGObjectLevelArgs(@"EOMKKD", @"destinationKey=%@", destinationKey);
-
-      destinationIndex = [self indexForKey: destinationKey];
-      EOFLOGObjectLevelArgs(@"EOMKKD", @"destinationIndex=%d",
-			    destinationIndex);
-
-      sourceIndex = [sourceInitializer indexForKey: sourceKey];
-      EOFLOGObjectLevelArgs(@"EOMKKD", @"sourceIndex=%d", sourceIndex);
-
-      NSAssert2(destinationIndex != NSNotFound,
-                @"Destination Key %@ not found in %@",
-                destinationKey,
-                self);
-      NSAssert2(sourceIndex != NSNotFound,
-                @"Source Key %@ not found in %@",
-                sourceKey,
-                sourceInitializer);
-
-      subsetMapping->_sourceOffsetForDestinationOffset[destinationIndex]
-	= sourceIndex + 1;
-    }
+        for (i = 0; i < keyCount; i++)
+          {
+            NSString *sourceKey = nil;
+            NSString *destinationKey = nil;
+            int destinationIndex = 0;
+            int sourceIndex = 0;
+            
+            sourceKey = 
+              GDL2_ObjectAtIndexWithImp(sourceKeys,sourceObjectAtIndexIMP,i);
+            EOFLOGObjectLevelArgs(@"EOMKKD", @"sourceKey=%@", sourceKey);
+            
+            destinationKey = 
+              GDL2_ObjectAtIndexWithImp(destinationKeys,destinationObjectAtIndexIMP,i);
+            EOFLOGObjectLevelArgs(@"EOMKKD", @"destinationKey=%@", destinationKey);
+            
+            destinationIndex =  
+              EOMKKDInitializer_indexForKeyWithImpPtr(self,
+                                                      &selfIndexForKeyIMP,
+                                                      destinationKey);
+            EOFLOGObjectLevelArgs(@"EOMKKD", @"destinationIndex=%d",
+                                  destinationIndex);
+            
+            sourceIndex = 
+              EOMKKDInitializer_indexForKeyWithImpPtr(sourceInitializer,
+                                                      &sourceInitializerIndexForKeyIMP,
+                                                      sourceKey);
+            EOFLOGObjectLevelArgs(@"EOMKKD", @"sourceIndex=%d", sourceIndex);
+            
+            NSAssert2(destinationIndex != NSNotFound,
+                      @"Destination Key %@ not found in %@",
+                      destinationKey,
+                      self);
+            NSAssert2(sourceIndex != NSNotFound,
+                      @"Source Key %@ not found in %@",
+                      sourceKey,
+                      sourceInitializer);
+            
+            subsetMapping->_sourceOffsetForDestinationOffset[destinationIndex]
+              = sourceIndex + 1;
+          }
+      };
 
   return subsetMapping;
 }
@@ -329,22 +363,28 @@ RCS_ID("$Id: EOMutableKnownKeyDictionary.m,v 1.12 2004/03/19 16:41:20 ayers Exp
 					  sourceDescription: sourceInitializer
 					  destinationDescription: self
 					  zone: [self zone]] autorelease];
-  int i;
 
-  for (i = 0; i < keyCount; i++)
+  if (keyCount>0)
     {
-      NSString *key;
-      int index;
-
-      key = _keys[i];
-      EOFLOGObjectLevelArgs(@"EOMKKD", @"key=%@", key);
-
-      index = [sourceInitializer indexForKey: key];
-      EOFLOGObjectLevelArgs(@"EOMKKD", @"index=%d", index);
-
-      subsetMapping->_sourceOffsetForDestinationOffset[i]
-	= (index == NSNotFound ? 0 : index + 1);
-    }
+      int i=0;
+      GDL2IMP_UINT indexForKeyIMP=NULL;
+  
+      for (i = 0; i < keyCount; i++)
+        {
+          NSString *key;
+          int index;
+          
+          key = _keys[i];
+          EOFLOGObjectLevelArgs(@"EOMKKD", @"key=%@", key);
+          
+          index = EOMKKDInitializer_indexForKeyWithImpPtr(sourceInitializer,
+                                                          &indexForKeyIMP,key);
+          EOFLOGObjectLevelArgs(@"EOMKKD", @"index=%d", index);
+          
+          subsetMapping->_sourceOffsetForDestinationOffset[i]
+            = (index == NSNotFound ? 0 : index + 1);
+        }
+    };
 
   return subsetMapping;
 }
@@ -485,16 +525,6 @@ RCS_ID("$Id: EOMutableKnownKeyDictionary.m,v 1.12 2004/03/19 16:41:20 ayers Exp
 
 @implementation EOMKKDArrayMapping
 
-+ (id)dictionaryFromDictionary: (NSDictionary *)dict
-		 subsetMapping: (EOMKKDSubsetMapping *)subsetMapping
-{
-#warning (stephane@sente.ch) Method is really not implemented!
-#warning (Ayers 28-03-2003) Either remove this method from array or use EOMutableKnownKeyDictionary instead of self 
-  return [[self newDictionaryFromDictionary: dict
-                subsetMapping: subsetMapping
-                zone: NULL] autorelease];
-}
-
 + (id)newInstanceWithKeyCount: (unsigned int)keyCount
        destinationDescription: (EOMKKDInitializer*)destination
 			 zone: (NSZone*)zone
@@ -536,6 +566,16 @@ RCS_ID("$Id: EOMutableKnownKeyDictionary.m,v 1.12 2004/03/19 16:41:20 ayers Exp
 
 @implementation EOMutableKnownKeyDictionary
 
++ (void)initialize
+{
+  static BOOL initialized=NO;
+  if (!initialized)
+    {
+      initialized=YES;
+      GDL2_PrivateInit();
+    }
+}
+
 + (id)dictionaryFromDictionary: (NSDictionary *)dict
 		 subsetMapping: (EOMKKDSubsetMapping *)subsetMapping
 {
@@ -795,7 +835,7 @@ RCS_ID("$Id: EOMutableKnownKeyDictionary.m,v 1.12 2004/03/19 16:41:20 ayers Exp
   return [_MKKDInitializer count];
 }
 
-- (id) objectForKey: (NSString*)key
+- (id) objectForKey: (id)key
 {
   id object = nil;
   unsigned int index;
@@ -803,7 +843,7 @@ RCS_ID("$Id: EOMutableKnownKeyDictionary.m,v 1.12 2004/03/19 16:41:20 ayers Exp
 //  EOFLOGObjectFnStart();
   NSAssert(_MKKDInitializer, @"No _MKKDInitializer");
 
-  index = [_MKKDInitializer indexForKey: key];
+  index = EOMKKDInitializer_indexForKeyWithImpPtr(_MKKDInitializer,NULL,key);
 
 //  EOFLOGObjectLevelArgs(@"EOMKKD", @"index=%d", index);
 
@@ -826,13 +866,13 @@ RCS_ID("$Id: EOMutableKnownKeyDictionary.m,v 1.12 2004/03/19 16:41:20 ayers Exp
 }
 
 - (void) setObject: (id)object
-            forKey: (NSString*)key
+            forKey: (id)key
 {
   unsigned int index;
 
   NSAssert(_MKKDInitializer, @"No _MKKDInitializer");
 
-  index = [_MKKDInitializer indexForKey: key];
+  index = EOMKKDInitializer_indexForKeyWithImpPtr(_MKKDInitializer,NULL,key);
 
   if (index == NSNotFound)
     {
@@ -851,13 +891,13 @@ RCS_ID("$Id: EOMutableKnownKeyDictionary.m,v 1.12 2004/03/19 16:41:20 ayers Exp
     }
 }
 
-- (void) removeObjectForKey: (NSString*)key
+- (void) removeObjectForKey: (id)key
 {
   unsigned int index;
 
   NSAssert(_MKKDInitializer, @"No _MKKDInitializer");
 
-  index = [_MKKDInitializer indexForKey: key];
+  index = EOMKKDInitializer_indexForKeyWithImpPtr(_MKKDInitializer,NULL,key);
 
   if (index == NSNotFound)
     {
@@ -883,9 +923,9 @@ RCS_ID("$Id: EOMutableKnownKeyDictionary.m,v 1.12 2004/03/19 16:41:20 ayers Exp
     {
       if (_values[i] != object)
         {
-          if (isNilOrEONull(_values[i]))
-            result =! isNilOrEONull(object);
-          else if (isNilOrEONull(object))
+          if (_isNilOrEONull(_values[i]))
+            result =! _isNilOrEONull(object);
+          else if (_isNilOrEONull(object))
             result = YES;
           else
             result = ![_values[i] isEqual: object];
@@ -900,11 +940,12 @@ RCS_ID("$Id: EOMutableKnownKeyDictionary.m,v 1.12 2004/03/19 16:41:20 ayers Exp
 - (void)addEntriesFromDictionary: (NSDictionary*)dictionary
 {
   NSEnumerator *e = [dictionary keyEnumerator];
-  id key;
+  id key=nil;
+  IMP indexForKeyIMP=NULL;
 
   while ((key = [e nextObject]))
     {
-      if (![self objectForKey: key]) //Don't overwrite already present values ?
+      if (!EOMKKD_objectForKeyWithImpPtr(self,&indexForKeyIMP,key)) //Don't overwrite already present values ?
         {
           [self setObject: [dictionary objectForKey: key]
                 forKey: key];
