@@ -1,13 +1,13 @@
 /** 
    EONSAddOns.m <title>EONSAddOns</title>
 
-   Copyright (C) 2000-2002 Free Software Foundation, Inc.
+   Copyright (C) 2000-2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Manuel Guesdon <mguesdon@orange-concept.com>
    Date: October 2000
 
-   $Revision: 1.14 $
-   $Date: 2004/03/19 16:41:20 $
+   $Revision: 21645 $
+   $Date: 2005-08-17 10:07:57 +0200 (Mit, 17 Aug 2005) $
 
    <abstract></abstract>
 
@@ -27,13 +27,13 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
 #include "config.h"
 
-RCS_ID("$Id: EONSAddOns.m,v 1.14 2004/03/19 16:41:20 ayers Exp $")
+RCS_ID("$Id: EONSAddOns.m 21645 2005-08-17 08:07:57Z ayers $")
 
 #ifdef GNUSTEP
 #include <Foundation/NSString.h>
@@ -54,11 +54,17 @@ RCS_ID("$Id: EONSAddOns.m,v 1.14 2004/03/19 16:41:20 ayers Exp $")
 #include <GNUstepBase/GSCategories.h>
 #include <GNUstepBase/GSObjCRuntime.h>
 #endif
+#include <GNUstepBase/Unicode.h>
 #include <GNUstepBase/GSLock.h>
 
 #include <EOControl/EONSAddOns.h>
 #include <EOControl/EODebug.h>
 
+#include <limits.h>
+
+#include "EOPrivate.h"
+
+
 static NSRecursiveLock *local_lock = nil;
 static BOOL GSStrictWO451Flag = NO;
 
@@ -89,6 +95,61 @@ GSUseStrictWO451Compatibility (NSString *key)
   return GSStrictWO451Flag;
 }
 
+void
+GDL2_DumpMethodList(Class cls, SEL sel, BOOL isInstance)
+{
+  void *iterator = 0;
+  GSMethodList mList;
+  
+  fprintf(stderr,"List for :%s %s (inst:%d)\n",
+	      GSNameFromClass(cls), GSNameFromSelector(sel), isInstance);
+  while ((mList = GSMethodListForSelector(cls, sel,
+					  &iterator, isInstance)))
+    {
+      GSMethod meth = GSMethodFromList(mList, sel, NO);
+      IMP imp = meth->method_imp;
+
+      fprintf(stderr,"List: %p Meth: %p Imp: %p\n",
+	      mList, meth, imp);
+    }
+  fprintf(stderr,"List finished\n"); fflush(stderr);
+}
+
+void
+GDL2_ActivateCategory(const char *className, SEL sel, BOOL isInstance)
+{
+  Class cls;
+  GSMethodList mList;
+
+  cls = GSClassFromName(className);
+  mList = GSMethodListForSelector(cls, sel, 0, isInstance);
+
+  GSRemoveMethodList(cls, mList, isInstance);
+  GSAddMethodList(cls, mList, isInstance);
+
+  GSFlushMethodCacheForClass(cls);
+}
+
+void
+GDL2_ActivateAllGDL2Categories(void)
+{
+  /* EOKeyValueCoding */
+  GDL2_ActivateCategory("NSObject",
+                        @selector(GDL2KVCNSObjectICategoryID), YES);
+  GDL2_ActivateCategory("NSArray",
+			@selector(GDL2KVCNSArrayICategoryID), YES);
+  GDL2_ActivateCategory("NSDictionary",
+                        @selector(GDL2KVCNSDictionaryICategoryID), YES);
+  GDL2_ActivateCategory("NSMutableDictionary",
+                        @selector(GDL2KVCNSMutableDictionaryICategoryID), YES);
+
+  /* EOClassDescription */
+  GDL2_ActivateCategory("NSObject",
+                        @selector(GDL2CDNSObjectICategoryID), YES);
+
+}
+
+
 @implementation NSObject (NSObjectPerformingSelector)
 
 - (NSArray*)resultsOfPerformingSelector: (SEL)sel
@@ -131,7 +192,7 @@ GSUseStrictWO451Compatibility (NSString *key)
                         object,
                         sel_get_name(sel));
 
-              [results addObject: result]; //TODO What to do if nil ??
+              [results addObject: result];
             }
         }
       NS_HANDLER
@@ -140,7 +201,7 @@ GSUseStrictWO451Compatibility (NSString *key)
                     object,
                     [object class],
                     NSStringFromSelector(sel));
-          NSLog(@"%@ (%@)",localException,[localException reason]);
+          NSLog(@"%@ %@",localException,[localException userInfo]);
           [localException raise];
         }
       NS_ENDHANDLER;
@@ -519,6 +580,22 @@ GSUseStrictWO451Compatibility (NSString *key)
 }
 @end
 
+@implementation NSString (Extensions)
+- (NSString *)initialCapitalizedString
+{
+  unichar *chars;
+  unsigned int length = [self length];
+
+  chars = objc_malloc(length * sizeof(unichar));
+  [self getCharacters: chars];
+  chars[0]=uni_toupper(chars[0]);
+
+  return AUTORELEASE([[NSString alloc] initWithCharactersNoCopy: chars
+				       length: length
+				       freeWhenDone: YES]);
+}
+@end
+
 @implementation NSString (StringToNumber)
 -(unsigned int)unsignedIntValue
 {
