@@ -1,13 +1,13 @@
 /** 
-   EOOrQualifier.m <title>EOOrQualifier</title>
+   EOOrQualifier.m <title>EONotQualifier</title>
 
-   Copyright (C) 2000-2002 Free Software Foundation, Inc.
+   Copyright (C) 2000-2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: February 2000
 
-   $Revision: 1.11 $
-   $Date: 2004/03/19 16:41:20 $
+   $Revision: 21645 $
+   $Date: 2005-08-17 10:07:57 +0200 (Mit, 17 Aug 2005) $
 
    <abstract></abstract>
 
@@ -27,13 +27,13 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
 #include "config.h"
 
-RCS_ID("$Id: EONotQualifier.m,v 1.11 2004/03/19 16:41:20 ayers Exp $")
+RCS_ID("$Id: EONotQualifier.m 21645 2005-08-17 08:07:57Z ayers $")
 
 #ifdef GNUSTEP
 #include <Foundation/NSDictionary.h>
@@ -118,5 +118,9 @@ RCS_ID("$Id: EONotQualifier.m,v 1.11 2004/03/19 16:41:20 ayers Exp $")
   [_qualifier _addBindingsToDictionary:dictionary];
 }
 
+- (void)addQualifierKeysToSet: (NSMutableSet *)keys
+{
+  [_qualifier addQualifierKeysToSet: keys];
+}
 
 @end
