@@ -1,7 +1,7 @@
 /* -*-objc-*-
    EONull.h
 
-   Copyright (C) 1996-2002, 2003 Free Software Foundation, Inc.
+   Copyright (C) 1996-2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mircea Oancea <mircea@jupiter.elcom.pub.ro>
    Date: 1996
@@ -21,7 +21,7 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 #ifndef __EONull_h__
@@ -43,12 +43,5 @@
 - (NSString *)sqlString;
 @end
 
-@interface NSObject (EONullTest)
-- (BOOL)isEONull;
-- (BOOL)isNotEONull;
-@end
-
-GDL2CONTROL_EXPORT BOOL isNilOrEONull(id v);
-
 #endif /* __EONull_h__ */
 
