@@ -1,7 +1,7 @@
 /** 
    EONull.m <title>EONull Class</title>
 
-   Copyright (C) 1996-2002 Free Software Foundation, Inc.
+   Copyright (C) 1996-2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mircea Oancea <mircea@jupiter.elcom.pub.ro>
    Date: 1996
@@ -9,8 +9,8 @@
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: February 2000
 
-   $Revision: 1.10 $
-   $Date: 2004/03/19 16:41:20 $
+   $Revision: 21645 $
+   $Date: 2005-08-17 10:07:57 +0200 (Mit, 17 Aug 2005) $
 
    <abstract></abstract>
 
@@ -30,13 +30,13 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
 #include "config.h"
 
-RCS_ID("$Id: EONull.m,v 1.10 2004/03/19 16:41:20 ayers Exp $")
+RCS_ID("$Id: EONull.m 21645 2005-08-17 08:07:57Z ayers $")
 
 #ifdef GNUSTEP
 #include <Foundation/NSString.h>
@@ -71,28 +71,9 @@ RCS_ID("$Id: EONull.m,v 1.10 2004/03/19 16:41:20 ayers Exp $")
 @end
 
 
-@implementation NSObject (EONull)
-
-- (BOOL)isEONull
-{  
-  return ((id)self == [NSNull null]);
-}
-
-- (BOOL)isNotEONull
-{
-  return ![self isEONull];
-}
-
-@end
-
-BOOL isNilOrEONull(id v)
-{
-  return ((!v) || [v isEONull]);
-}
-
 /*
  * We keep this class to support NSClassFromString() which
- * scripting libraries my depend on.  Note that this is
+ * scripting libraries may depend on.  Note that this is
  * not a fail-safe implementation.  You should rely on
  * [EONull+null] and pointer comparison.  Do not rely on
  * [obj isKindOfClass: NSClassFromString(@"EONull")]
