@@ -1,13 +1,13 @@
 /** 
    EOObjectStore.m <title>EOObjectStore</title>
 
-   Copyright (C) 2000 Free Software Foundation, Inc.
+   Copyright (C) 2000,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: June 2000
 
-   $Revision: 1.8 $
-   $Date: 2004/03/19 16:41:20 $
+   $Revision: 21645 $
+   $Date: 2005-08-17 10:07:57 +0200 (Mit, 17 Aug 2005) $
 
    <abstract></abstract>
 
@@ -27,13 +27,13 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
 #include "config.h"
 
-RCS_ID("$Id: EOObjectStore.m,v 1.8 2004/03/19 16:41:20 ayers Exp $")
+RCS_ID("$Id: EOObjectStore.m 21645 2005-08-17 08:07:57Z ayers $")
 
 #ifdef GNUSTEP
 #include <Foundation/NSString.h>
@@ -53,10 +53,10 @@ NSString *EOObjectsChangedInStoreNotification = @"EOObjectsChangedInStoreNotific
 
 NSString *EOInvalidatedAllObjectsInStoreNotification = @"EOInvalidatedAllObjectsInStoreNotification";
 
-NSString *EODeletedKey = @"inserted";
-NSString *EOInsertedKey = @"deleted";
-NSString *EOInvalidatedKey = @"updated";
-NSString *EOUpdatedKey = @"invalidated";
+NSString *EODeletedKey = @"deleted";
+NSString *EOInsertedKey = @"inserted";
+NSString *EOInvalidatedKey = @"invalidated";
+NSString *EOUpdatedKey = @"updated";
 
 
 @implementation EOObjectStore
