@@ -1,13 +1,13 @@
 /** 
    EOOrQualifier.m <title>EOOrQualifier</title>
 
-   Copyright (C) 2000-2002 Free Software Foundation, Inc.
+   Copyright (C) 2000-2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: February 2000
 
-   $Revision: 1.14 $
-   $Date: 2004/03/19 16:41:20 $
+   $Revision: 21766 $
+   $Date: 2005-10-02 15:56:00 +0200 (Son, 02 Okt 2005) $
 
    <abstract></abstract>
 
@@ -27,13 +27,13 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
 #include "config.h"
 
-RCS_ID("$Id: EOOrQualifier.m,v 1.14 2004/03/19 16:41:20 ayers Exp $")
+RCS_ID("$Id: EOOrQualifier.m 21766 2005-10-02 13:56:00Z ayers $")
 
 #ifdef GNUSTEP
 #include <Foundation/NSDictionary.h>
@@ -192,9 +192,13 @@ RCS_ID("$Id: EOOrQualifier.m,v 1.14 2004/03/19 16:41:20 ayers Exp $")
   return resultQualifier;
 }
 
-- (id) initWithKeyValueUnarchiver: (id)param0
+- (id) initWithKeyValueUnarchiver: (id) archiver
 {
-  return [self notImplemented: _cmd]; //TODO
+  if ((self = [super init])) {
+    id qualifierArray = [archiver decodeObjectForKey:@"qualifiers"];
+    ASSIGN (_qualifiers, qualifierArray);
+  }
+  return self;
 }
 
 - (void) encodeWithKeyValueArchiver: (id)param0
@@ -212,7 +216,20 @@ RCS_ID("$Id: EOOrQualifier.m,v 1.14 2004/03/19 16:41:20 ayers Exp $")
   return nil;
 }
 
-- (id) description
+- (NSString *) description
+{
+  NSString *dscr;
+  
+  dscr = [NSString stringWithFormat: @"(%@)",
+		   [_qualifiers componentsJoinedByString: @" OR "]];
+
+  return dscr;
+}
+
+/**
+ * Deprecated debug description.  Expect it to be removed.
+ */
+- (NSString *)debugDescription
 {
   NSString *dscr;
 
@@ -223,4 +240,14 @@ RCS_ID("$Id: EOOrQualifier.m,v 1.14 2004/03/19 16:41:20 ayers Exp $")
   return dscr;
 }
 
+- (void)addQualifierKeysToSet: (NSMutableSet *)keys
+{
+  EOQualifier *qual;
+  unsigned int i,n;
+  for (i=0, n=[_qualifiers count]; i < n; i++)
+    {
+      qual = [_qualifiers objectAtIndex:i];
+      [qual addQualifierKeysToSet: keys];
+    }
+}
 @end
