@@ -1,7 +1,7 @@
 /* -*-objc-*-
    EOQualifier.h
 
-   Copyright (C) 2000 Free Software Foundation, Inc.
+   Copyright (C) 2000,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: February 2000
@@ -21,7 +21,7 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 #ifndef __EOQualifier_h__
@@ -40,6 +40,8 @@
 @class NSArray;
 @class NSDictionary;
 @class NSString;
+@class NSSet;
+@class NSMutableSet;
 @class NSException;
 
 @class EOClassDescription;
@@ -80,6 +82,9 @@
 - (NSArray *)bindingKeys;
 - (NSString *)keyPathForBindingKey: (NSString *)key;
 
+- (NSSet *)allQualifierKeys;
+- (void)addQualifierKeysToSet: (NSMutableSet *)qualKeys;
+
 - (BOOL)evaluateWithObject: (id)object;
 
 @end
