@@ -1,13 +1,13 @@
 /** 
    EOQualifier.m <title>EOQualifier</title>
 
-   Copyright (C) 2000 Free Software Foundation, Inc.
+   Copyright (C) 2000,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: February 2000
 
-   $Revision: 1.21 $
-   $Date: 2004/03/19 16:41:20 $
+   $Revision: 22157 $
+   $Date: 2005-12-05 21:35:08 +0100 (Mon, 05 Dez 2005) $
 
    <abstract></abstract>
 
@@ -27,13 +27,13 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
 #include "config.h"
 
-RCS_ID("$Id: EOQualifier.m,v 1.21 2004/03/19 16:41:20 ayers Exp $")
+RCS_ID("$Id: EOQualifier.m 22157 2005-12-05 20:35:08Z ayers $")
 
 #include <stdio.h>
 #include <string.h>
@@ -100,6 +100,32 @@ NSString *EOQualifierVariableSubstitutionException=@"EOQualifierVariableSubstitu
 @end
 
 @implementation EOQualifier
+static SEL cimSEL = NULL;
+static NSCharacterSet *spaceSet;
+static NSCharacterSet *alnumSet;
+static NSCharacterSet *digitSet;
+static BOOL (*spaceCIM)(id,SEL,unichar);
+static BOOL (*alnumCIM)(id,SEL,unichar);
+static BOOL (*digitCIM)(id,SEL,unichar);
++ (void)initialize
+{
+  if (cimSEL == NULL)
+    {
+      cimSEL = @selector(characterIsMember:);
+
+      spaceSet
+	= RETAIN([NSCharacterSet whitespaceAndNewlineCharacterSet]);
+      spaceCIM = (BOOL(*)(id,SEL,unichar))[spaceSet methodForSelector: cimSEL];
+
+      alnumSet
+	= RETAIN([NSCharacterSet alphanumericCharacterSet]);
+      alnumCIM = (BOOL(*)(id,SEL,unichar))[alnumSet methodForSelector: cimSEL];
+
+      digitSet
+	= RETAIN([NSCharacterSet decimalDigitCharacterSet]);
+      digitCIM = (BOOL(*)(id,SEL,unichar))[digitSet methodForSelector: cimSEL];
+    }
+}
 
 /**
  * Returns an autoreleased qualifier which is constructed by calling
@@ -146,48 +172,59 @@ NSString *EOQualifierVariableSubstitutionException=@"EOQualifierVariableSubstitu
 #endif
 }
 
-static NSString *getOperator(const char **cFormat, const char **s)
+static NSString *getOperator(const unichar **cFormat, const unichar **s)
 {
   NSString *operator;
 
-  while (**s && isspace(**s))
+  while (**s && spaceCIM(spaceSet,cimSEL,**s))
     (*s)++;
 
   *cFormat = *s;
 
-  if (isalnum(**s))
+  if (alnumCIM(alnumSet,cimSEL,**s))
     {
-      while (**s && !isspace(**s) && **s != '%' && **s != '\'')
+      while (**s && !spaceCIM(spaceSet,cimSEL,**s) 
+	     && **s != '%' && **s != '\'')
         {
 	  (*s)++;
         }
 
-      operator = [NSString stringWithCString: *cFormat length: *s - *cFormat];
+      operator = [NSString stringWithCharacters: *cFormat
+			   length: *s - *cFormat];
     }
   else
     {
-      while (**s && !isalnum(**s) && !isspace(**s) && **s != '%' && **s != '\'')
+      while (**s && !alnumCIM(alnumSet, cimSEL, **s) 
+	     && !spaceCIM(spaceSet, cimSEL, **s)
+	     && **s != '%' && **s != '\'')
         {
-	  NSDebugLog(@"avoid gcc 3.1.1 bug which optimizes to segfault", "");
+	  NSDebugLog(@"avoid gcc 3.1.1 bug which optimizes to segfault",
+		     @"avoid gcc 3.1.1 bug which optimizes to segfault");
 	  (*s)++;
         }
 
-      operator = [NSString stringWithCString: *cFormat length: *s - *cFormat];
+      operator = [NSString stringWithCharacters: *cFormat 
+			   length: *s - *cFormat];
     }
 
   *cFormat = *s;
 
   return operator;
 }
-        
-static id getKey(const char **cFormat, const char **s, BOOL *isKeyValue,
-		 va_list *args)
-{
-  NSMutableString *key, *classString = nil;
-  char quoteChar;
-  BOOL quoted = NO;
 
-  while (**s && isspace(**s))
+static id
+getKey(const unichar **cFormat, 
+       const unichar **s,
+       BOOL *isKeyValue,
+       va_list *args)
+{
+  NSMutableString *key;
+  NSString *classString = nil;
+  unichar quoteChar;
+  BOOL quoted = NO;
+  BOOL literalNumber = NO;
+
+  while (**s && spaceCIM(spaceSet, cimSEL, **s))
     (*s)++;
 
   if (isKeyValue)
@@ -199,19 +236,19 @@ static id getKey(const char **cFormat, const char **s, BOOL *isKeyValue,
 	  while (**s && **s != ')')
 	    (*s)++;
 
-	  if (!*s); //TODO exception
+	  NSCAssert(*s, @"Illegal Qualifer format missing bracket.");
 
-	  classString = [NSString stringWithCString: *cFormat
+	  classString = [NSString stringWithCharacters: *cFormat
 				  length: *s - *cFormat];
 
 	  (*s)++; *cFormat = *s;
         }
   
-      if (!strncmp("nil", *s, 3))
+      if ((*s)[0] == 'n' && (*s)[1] == 'i' && (*s)[2] == 'l')
 	{
-	  char value = *(*s+3);
+	  unichar value = (*s)[3];
 
-	  if (value == 0 || value == ' ')
+	  if (value != '_' && !alnumCIM(alnumSet,cimSEL,value))
 	    {
 	      *cFormat = *s = *s+3;
 	      *isKeyValue = YES;
@@ -234,22 +271,35 @@ static id getKey(const char **cFormat, const char **s, BOOL *isKeyValue,
       while (**s && **s != quoteChar)
 	(*s)++;
 
-      key = [NSString stringWithCString: *cFormat length: *s - *cFormat];
+      key = [NSString stringWithCharacters: *cFormat length: *s - *cFormat];
       (*s)++; // skip closing quote
     }
   else
     {
       key = [NSMutableString stringWithCapacity:8];
 
-      while (**s && (isalnum(**s) || **s == '@' || **s == '#' || **s == '_'
-		     || **s == '$' || **s == '%' || **s == '.'))
+      if (classString == nil 
+	  && (digitCIM(digitSet, cimSEL, **s) 
+	      || (**s == '-' && digitCIM(digitSet, cimSEL, *(*s+1)))))
+	{
+	  classString = @"NSNumber";
+	  literalNumber = YES;
+	}
+	
+      while (**s && (alnumCIM(alnumSet,cimSEL,**s) 
+		     || **s == '@' || **s == '#' || **s == '_' || **s == '$' 
+		     || **s == '%' || **s == '.' || **s == '-'))
         {
 	  if (**s == '%')
 	    {
 	      const char *argString;
 	      NSString *argObj;
-	      //float argFloat;
-	      double argFloat; // `float' is promoted to `double' when passed through `...' (so you should pass `double' not `float' to `va_arg')
+	      double argFloat; 
+	      /* 'float' is promoted to 'double' when passed through '...'
+		 (so you should pass 'double' not 'float' to `va_arg')
+		 Ayers: I believe the compiler should do this promotion
+		 implicitly but there are buggy compilers so cast to 
+		 be safe.  */
 
 	      int argInt;
 
@@ -263,8 +313,8 @@ static id getKey(const char **cFormat, const char **s, BOOL *isKeyValue,
 		case '@':
 		  argObj = va_arg(*args, id);
 
-		  if (isKeyValue && *isKeyValue == YES && quoted == NO
-		      && classString == nil)
+		  if (isKeyValue && *isKeyValue == YES 
+		      && quoted == NO && classString == nil)
 		    {
 		      *cFormat = *s = *s+2;
 		      return argObj;
@@ -272,8 +322,12 @@ static id getKey(const char **cFormat, const char **s, BOOL *isKeyValue,
 		  else
 		    {
 		      if (*cFormat != *s)
-			[key appendString: [NSString stringWithCString: *cFormat
-						     length: *s - *cFormat]];
+			{
+			  NSString *str 
+			    = [NSString stringWithCharacters: *cFormat
+					length: *s - *cFormat];
+			  [key appendString: str];
+			}
 
 		      [key appendString: [argObj description]];
 		      *cFormat = *s+2;
@@ -284,20 +338,25 @@ static id getKey(const char **cFormat, const char **s, BOOL *isKeyValue,
 		case 's':
 		  argString = va_arg(*args, const char *);
 
-		  if (isKeyValue && *isKeyValue == YES && quoted == NO
-		      && classString == nil)
+		  if (isKeyValue && *isKeyValue == YES
+		      && quoted == NO && classString == nil)
 		    {
 		      *cFormat = *s = *s + 2;
 		      return [NSString stringWithCString: argString];
 		    }
 		  else
 		    {
+		      NSString *str;
 		      if (*cFormat != *s)
-			[key appendString: [NSString stringWithCString: *cFormat
-						     length: *s - *cFormat]];
+			{
+			  str = [NSString stringWithCharacters: *cFormat
+					  length: *s - *cFormat];
+			  [key appendString: str];
+			}
+
+		      str = [NSString stringWithCString: argString];
+		      [key appendString: str];
 
-		      [key appendString: [NSString
-					   stringWithCString: argString]];
 		      *cFormat = *s + 2;
 		      (*s)++;
 		    }
@@ -306,27 +365,34 @@ static id getKey(const char **cFormat, const char **s, BOOL *isKeyValue,
 		case 'd':
 		  argInt = va_arg(*args, int);
 
-		  if (isKeyValue && *isKeyValue == YES && quoted == NO
-		      && classString == nil)
+		  if (isKeyValue && *isKeyValue == YES
+		      && quoted == NO && classString == nil)
 		    {
 		      *cFormat = *s = *s + 2;
 		      return [NSNumber numberWithInt: argInt];
 		    }
 		  else
 		    {
+		      NSString *str;
 		      if(*cFormat != *s)
-			[key appendString: [NSString stringWithCString: *cFormat
-						     length: *s - *cFormat]];
+			{
+			  str = [NSString stringWithCharacters: *cFormat
+					  length: *s - *cFormat];
+			  [key appendString: str];
+			}
+
+		      str = [NSString stringWithFormat: @"%d", argInt];
+		      [key appendString: str];
 
-		      [key appendString: [NSString stringWithFormat: @"%d",
-						   argInt]];
 		      *cFormat = *s + 2;
 		      (*s)++;
 		    }
 		  break;
 
 		case 'f':
-		  argFloat = va_arg(*args, double);// `float' is promoted to `double' when passed through `...' (so you should pass `double' not `float' to `va_arg')
+		  /* 'float' is promoted to 'double' when passed through '...'
+		     (so you should pass `double' not `float' to `va_arg') */
+		  argFloat = va_arg(*args, double);
 
 		  if (isKeyValue && *isKeyValue == YES && quoted == NO
 		      && classString == nil)
@@ -336,12 +402,18 @@ static id getKey(const char **cFormat, const char **s, BOOL *isKeyValue,
 		    }
 		  else
 		    {
+		      NSString *str;
+
 		      if (*cFormat != *s)
-			[key appendString: [NSString stringWithCString: *cFormat
-						     length: *s - *cFormat]];
+			{
+			  str = [NSString stringWithCharacters: *cFormat
+					  length: *s - *cFormat];
+			  [key appendString: str];
+			}
+				 
+		      str = [NSString stringWithFormat: @"%f", argFloat];
+		      [key appendString: str];
 
-		      [key appendString: [NSString stringWithFormat: @"%f",
-						   argFloat]];
 		      *cFormat = *s + 2;
 		      (*s)++;
 		    }
@@ -350,14 +422,21 @@ static id getKey(const char **cFormat, const char **s, BOOL *isKeyValue,
 		case '%':
 		  *cFormat = *s + 2;
 		  (*s)++;
-		  [key appendString: [NSString stringWithCString: *cFormat
+		  [key appendString: [NSString stringWithCharacters: *cFormat
 					       length: *s - *cFormat]];
 		  break;
 
 		default:
-		  [NSException raise: NSInvalidArgumentException
-			       format: @"%@ -- %@: unrecognized character (%c) in the conversion specification", @"qualifierParser", @"EOQualifier", *(*s + 1)];
-		  break;
+		  {
+		    NSString *fmt 
+		      = @"%@ -- %@: unrecognized character (%@) in the conversion specification";
+		    NSString *specChar 
+		      = [NSString stringWithCharacters: (*s + 1) length: 1];
+		    [NSException raise: NSInvalidArgumentException
+				 format: fmt, @"EOQualifier", 
+				 @"qualifierParser", specChar];
+		    break;
+		  } 
 		}
 	    }
 
@@ -365,18 +444,21 @@ static id getKey(const char **cFormat, const char **s, BOOL *isKeyValue,
         }
 
       if (*cFormat != *s)
-	[key appendString: [NSString stringWithCString: *cFormat
-				     length: *s - *cFormat]];
+	{
+	  NSString *str = [NSString stringWithCharacters: *cFormat
+				    length: *s - *cFormat];
+	  [key appendString: str];
+	}
     }
 
   if (isKeyValue)
     {
-      *isKeyValue = quoted;
+      *isKeyValue = (quoted || literalNumber);
 
       if (classString)
         {
-          key = AUTORELEASE([[NSClassFromString(classString) alloc]
-			      initWithString: key]);
+	  Class cls = NSClassFromString(classString);
+          key = AUTORELEASE([[cls alloc] initWithString: key]);
         }
     }
     
@@ -385,14 +467,16 @@ static id getKey(const char **cFormat, const char **s, BOOL *isKeyValue,
   return key;
 }
 
-static BOOL isNotQualifier(const char **cFormat, const char **s)
+static BOOL isNotQualifier(const unichar **cFormat, const unichar **s)
 {
-  while (**s && isspace(**s))
+  while (**s && spaceCIM(spaceSet,cimSEL,**s))
     (*s)++;
 
   *cFormat = *s;
 
-  if (!strncasecmp(*s, "not", 3))
+  if (((*s)[0]=='n' || (*s)[0]=='N')
+      && ((*s)[1]=='o' || (*s)[1]=='O')
+      && ((*s)[2]=='t' || (*s)[2]=='T'))
     {
       switch ((*s)[3])
         {
@@ -407,14 +491,16 @@ static BOOL isNotQualifier(const char **cFormat, const char **s)
   return NO;
 }
 
-static Class whichQualifier(const char **cFormat, const char **s)
+static Class whichQualifier(const unichar **cFormat, const unichar **s)
 {
-  while (**s && isspace(**s))
+  while (**s && spaceCIM(spaceSet,cimSEL,**s))
     (*s)++;
 
   *cFormat = *s;
 
-  if (!strncasecmp(*s, "and", 3))
+  if (((*s)[0]=='a' || (*s)[0]=='A')
+      && ((*s)[1]=='n' || (*s)[1]=='N')
+      && ((*s)[2]=='d' || (*s)[2]=='D'))
     {
       switch ((*s)[3])
         {
@@ -425,7 +511,8 @@ static Class whichQualifier(const char **cFormat, const char **s)
 	  return [EOAndQualifier class];
         }
     }
-  else if (!strncasecmp(*s, "or", 2))
+  else if (((*s)[0]=='o' || (*s)[0]=='O')
+	   && ((*s)[1]=='r' || (*s)[1]=='R'))
     {
       switch ((*s)[2])
         {
@@ -443,8 +530,9 @@ static Class whichQualifier(const char **cFormat, const char **s)
 + (EOQualifier *)qualifierWithQualifierFormat: (NSString *)format
 				   varargList: (va_list)args
 {
-  const char *s;
-  const char *cFormat;
+  unichar *s0;
+  const unichar *s;
+  const unichar *cFormat;
   NSMutableArray *bracketStack = nil;
   NSMutableArray *qualifierArray = nil;
   NSMutableArray *parentQualifiers = nil;
@@ -457,15 +545,21 @@ static Class whichQualifier(const char **cFormat, const char **s)
   BOOL notQual;
   Class lastQualifierClass = Nil;
   Class qualifierClass = Nil;
+  unsigned formatLen;
 
   bracketStack = [NSMutableArray array];
   parentQualifiers = [NSMutableArray array];
 
-  cFormat = s = [format cString];
+  formatLen = [format length];
+  s0 = GSAutoreleasedBuffer((formatLen+1) * sizeof(unichar));
+  [format getCharacters: s0];
+  s0[formatLen] = '\0';
+
+  cFormat = s = s0;
 
   while (*s)
     {
-      while (*s && isspace(*s))
+      while (*s && spaceCIM(spaceSet,cimSEL,*s))
         (s)++;
 
       while (*s == '(' )
@@ -473,10 +567,11 @@ static Class whichQualifier(const char **cFormat, const char **s)
 	NSMutableDictionary *state;
 
 	state = [NSMutableDictionary dictionaryWithCapacity:4];
-	if (lastQualifierClass != NULL)
+	if (lastQualifierClass != Nil)
 	  {
-	    [state setObject: lastQualifierClass forKey: @"lastQualifierClass"];
-	    lastQualifierClass = NULL;
+	    [state setObject: lastQualifierClass
+		   forKey: @"lastQualifierClass"];
+	    lastQualifierClass = Nil;
 	  }
 	if (qualifierArray != nil)
 	  {
@@ -494,7 +589,7 @@ static Class whichQualifier(const char **cFormat, const char **s)
 	[bracketStack addObject:state];
 
 	(s)++; // skip '('
-	while (*s && isspace(*s))
+	while (*s && spaceCIM(spaceSet, cimSEL, *s))
 	  (s)++;
       }
       
@@ -505,11 +600,14 @@ static Class whichQualifier(const char **cFormat, const char **s)
 
       operatorSelector = [EOQualifier operatorSelectorForString: operator];
       if (!operatorSelector)
-        [NSException raise: NSInvalidArgumentException
-		     format: @"%@ -- %@ 0x%x: no operator or unknown operator: '%@'",
-                     NSStringFromSelector(_cmd),
-		     NSStringFromClass([self class]), self,
-                     operator];
+	{
+	  [NSException raise: NSInvalidArgumentException
+		       format: @"%@ -- %@ 0x%x: no operator or unknown operator: '%@'",
+		       NSStringFromClass([self class]),
+		       NSStringFromSelector(_cmd),
+		       self,
+		       operator];
+	}
 
       EOFLOGObjectLevelArgs(@"EOQualifier",
 			    @"leftKey=%@ operatorSelector=%s rightKey=%@ class=%@",
@@ -539,49 +637,50 @@ static Class whichQualifier(const char **cFormat, const char **s)
 			    @"qualifier=%@",
 			    qualifier);
 
-      while (*s && isspace(*s))
+      while (*s && spaceCIM(spaceSet,cimSEL,*s))
         (s)++;
 
       while (*s == ')' )
-      {
-        NSMutableDictionary *state;
+	{
+	  NSMutableDictionary *state;
 
-        /* clean up inner qualifier */
-        if (qualifierArray != nil)
-	  {
-	    [qualifierArray addObject:qualifier];
-	    qualifier = AUTORELEASE([[qualifierClass alloc]
-				      initWithQualifierArray: qualifierArray]);
-	    qualifierArray = nil;
-	  }
+	  /* clean up inner qualifier */
+	  if (qualifierArray != nil)
+	    {
+	      [qualifierArray addObject:qualifier];
+	      qualifier 
+		= AUTORELEASE([[qualifierClass alloc]
+				initWithQualifierArray: qualifierArray]);
+	      qualifierArray = nil;
+	    }
 
-	while ([parentQualifiers count] != 0)
-	  {
-	    id       parent;
-	    NSArray *quals;
+	  while ([parentQualifiers count] != 0)
+	    {
+	      id       parent;
+	      NSArray *quals;
 	    
-	    parent = [parentQualifiers lastObject];
-	    quals = [[parent qualifiers] arrayByAddingObject: qualifier];
-	    qualifier = AUTORELEASE([[[parent class] alloc]
+	      parent = [parentQualifiers lastObject];
+	      quals = [[parent qualifiers] arrayByAddingObject: qualifier];
+	      qualifier = AUTORELEASE([[[parent class] alloc]
 					initWithQualifierArray: quals]);
-	    [parentQualifiers removeLastObject];
-	  }
+	      [parentQualifiers removeLastObject];
+	    }
 
-	DESTROY(parentQualifiers);
+	  DESTROY(parentQualifiers);
 
-	/* pop bracketStack */
-	state = [bracketStack lastObject];
-	qualifierArray = [state objectForKey:@"qualifierArray"];
-	lastQualifierClass = [state objectForKey:@"lastQualifierClass"];
-	qualifierClass = [state objectForKey:@"qualifierClass"];
-	parentQualifiers = [state objectForKey:@"parentQualifiers"];
+	  /* pop bracketStack */
+	  state = [bracketStack lastObject];
+	  qualifierArray = [state objectForKey:@"qualifierArray"];
+	  lastQualifierClass = [state objectForKey:@"lastQualifierClass"];
+	  qualifierClass = [state objectForKey:@"qualifierClass"];
+	  parentQualifiers = [state objectForKey:@"parentQualifiers"];
 
-	[bracketStack removeLastObject];
+	  [bracketStack removeLastObject];
 
-	(s)++; // skip ')'
-        while (*s && isspace(*s))
-          (s)++;
-      }
+	  (s)++; // skip ')'
+	  while (*s && spaceCIM(spaceSet,cimSEL,*s))
+	    (s)++;
+	}
 
       qualifierClass = whichQualifier(&cFormat, &s);
       EOFLOGObjectLevelArgs(@"EOQualifier", @"qualifierClass=%@",
@@ -768,7 +867,7 @@ static Class whichQualifier(const char **cFormat, const char **s)
   else if (sel_eq(selector, EOQualifierOperatorCaseInsensitiveLike))
     return @"caseInsensitiveLike";
 
-  return nil;
+  return NSStringFromSelector(selector);
 }
 
 + (SEL)operatorSelectorForString: (NSString *)string
@@ -835,7 +934,7 @@ static Class whichQualifier(const char **cFormat, const char **s)
 /** Returns binding keys **/
 - (NSArray *)bindingKeys
 {
-  NSMutableDictionary* bindings = (NSMutableDictionary*)[NSMutableDictionary dictionary];
+  NSMutableDictionary* bindings = (id)[NSMutableDictionary dictionary];
   [self _addBindingsToDictionary:bindings];
   return [bindings allKeys];
 }
@@ -866,6 +965,30 @@ static Class whichQualifier(const char **cFormat, const char **s)
   return NO;
 }
 
+/**
+ * Returns a set containing the keys of the qualifier tree starting
+ * from the receiver.  Subclasses should override -addQualifierKeysToSet:
+ * and rely on the implementation EOQualifier for this method.
+ */
+- (NSSet *)allQualifierKeys
+{
+  NSMutableSet *keys;
+  keys = (id)[NSMutableSet set];
+  [self addQualifierKeysToSet: keys];
+  return [NSSet setWithSet: keys];
+}
+
+/**
+ * Subclasses must override this method to add their qualifier keys to
+ * the provided keys set.  Aggregate qualifiers, such as EOAndQualifer,
+ * should traverse their contents invoking this method on each with the
+ * provided set.
+ */
+- (void)addQualifierKeysToSet: (NSMutableSet *)keys
+{
+  [self subclassResponsibility: _cmd];
+}
+
 @end
 
 @implementation EOQualifier (Privat)
@@ -933,12 +1056,16 @@ static Class whichQualifier(const char **cFormat, const char **s)
 
 - (id) initWithKeyValueUnarchiver: (EOKeyValueUnarchiver *)unarchiver
 {
-  return [self notImplemented: _cmd]; //TODO
+  if ((self = [super init]))
+    {
+      _key = RETAIN([unarchiver decodeObjectForKey: @"_key"]);
+    }
+  return self;
 }
 
 - (void)encodeWithKeyValueArchiver: (EOKeyValueArchiver *)archiver
 {
-  [self notImplemented: _cmd]; //TODO
+  [archiver encodeObject: _key forKey: @"_key"];
 }
 
 @end
@@ -1027,7 +1154,7 @@ _isLike (NSString *self, NSString *regExpr, BOOL isCaseSensative)
   NSString *scanned;
   unsigned c = 0;
   unsigned i = 0;
-  GDL2_BUFFER (tokens, [regExpr cStringLength], id);
+  GDL2_BUFFER (tokens, [regExpr length], id);
 
   if ([self isEqual: regExpr])
     {
@@ -1054,9 +1181,15 @@ _isLike (NSString *self, NSString *regExpr, BOOL isCaseSensative)
 	  if ([regExScanner scanCharactersFromSet: isLikeWildCardSet
 			    intoString: &scanned])
 	    {
-	      const char *cScanned;
+	      unsigned scannedLen;
+	      unichar *cScanned;
 
-	      for (cScanned = [scanned cString]; *cScanned != 0; cScanned++)
+	      scannedLen = [scanned length];
+	      cScanned = GSAutoreleasedBuffer((scannedLen+1) * sizeof(unichar));
+	      [scanned getCharacters: cScanned];
+	      cScanned[scannedLen] = 0;
+
+	      for (; *cScanned != 0; cScanned++)
 		{
 		  if (*cScanned == '?' 
 		      && tokens[c - 1] != isLikeWildCardTokenS)
@@ -1126,7 +1259,7 @@ _isLike (NSString *self, NSString *regExpr, BOOL isCaseSensative)
 {
   unsigned    max = [self count];
 
-  if (max != 0)
+  if (max != 0 && qualifier != nil)
     {
       unsigned  i;
       id        object;
