@@ -1,7 +1,7 @@
 /* -*-objc-*-
    EOSortOrdering.h
 
-   Copyright (C) 2000 Free Software Foundation, Inc.
+   Copyright (C) 2000,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: February 2000
@@ -21,7 +21,7 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 #ifndef __EOSortOrdering_h__
@@ -36,9 +36,10 @@
 #endif
 
 #include <EOControl/EONull.h>
+#include <EOControl/EOKeyValueArchiver.h>
 
 
-@interface EOSortOrdering : NSObject <NSCoding, NSCopying>
+@interface EOSortOrdering : NSObject <NSCoding, NSCopying, EOKeyValueArchiving>
 {
   SEL _selector;
   NSString *_key;
