@@ -1,13 +1,13 @@
 /** 
    EOSortOrdering.m <title>EOSortOrdering</title>
 
-   Copyright (C) 2000-2002 Free Software Foundation, Inc.
+   Copyright (C) 2000-2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Author: Mirko Viviani <mirko.viviani@rccr.cremona.it>
    Date: February 2000
 
-   $Revision: 1.13 $
-   $Date: 2004/04/15 10:33:35 $
+   $Revision: 21645 $
+   $Date: 2005-08-17 10:07:57 +0200 (Mit, 17 Aug 2005) $
 
    <abstract></abstract>
 
@@ -27,19 +27,18 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
 #include "config.h"
 
-RCS_ID("$Id: EOSortOrdering.m,v 1.13 2004/04/15 10:33:35 ayers Exp $")
+RCS_ID("$Id: EOSortOrdering.m 21645 2005-08-17 08:07:57Z ayers $")
 
 #ifdef GNUSTEP
 #include <Foundation/NSCoder.h>
 #include <Foundation/NSException.h>
 #include <Foundation/NSDebug.h>
-#include <Foundation/NSNull.h>
 #else
 #include <Foundation/Foundation.h>
 #endif
@@ -51,10 +50,24 @@ RCS_ID("$Id: EOSortOrdering.m,v 1.13 2004/04/15 10:33:35 ayers Exp $")
 #include <EOControl/EOSortOrdering.h>
 #include <EOControl/EOKeyValueCoding.h>
 #include <EOControl/EOKeyValueArchiver.h>
+#include <EOControl/EONull.h>
 #include <EOControl/EODebug.h>
 
+#include "EOPrivate.h"
+
 @implementation EOSortOrdering
 
++ (void)initialize
+{
+  static BOOL initialized=NO;
+  if (!initialized)
+    {
+      initialized=YES;
+
+      GDL2_PrivateInit();
+    };
+};
+
 /**
  * Returns an autoreleased EOSortOrdering initilaized with key and selector.  
  * The selector should take an id as an argument and return an 
@@ -176,17 +189,11 @@ compareUsingSortOrderings(id    left,
 			  id    right,
 			  void* vpSortOrders)
 {
-  static EONull     *null = nil;
   NSArray           *sortOrders = (NSArray *)vpSortOrders;
   NSComparisonResult r = NSOrderedSame;
   unsigned int       i;
   unsigned int       sortOrdCnt = [sortOrders count];
 
-  if (null == nil)
-    {
-      null = [EONull null];
-    }
-
   /* Loop over all sort orderings until we have an ordering difference. */
   for (i=0; (r == NSOrderedSame) && (i < sortOrdCnt); i++)
     {
@@ -199,15 +206,15 @@ compareUsingSortOrderings(id    left,
       NSComparisonResult (*imp)(id, SEL, id);
 
       /* Use EONull instead of nil. */
-      leftVal  = (leftVal  != nil)?(leftVal) :(null);
-      rightVal = (rightVal != nil)?(rightVal):(null);
+      leftVal  = (leftVal  != nil)?(leftVal) :(GDL2_EONull);
+      rightVal = (rightVal != nil)?(rightVal):(GDL2_EONull);
 
       /* Insure that EONull is not the parameter for 
 	 comparisons with other classes. */
-      if (rightVal == null)
+      if (rightVal == GDL2_EONull)
 	{
 	  rightVal = leftVal;
-	  leftVal  = null;
+	  leftVal  = GDL2_EONull;
 	  inverted = YES;
 	}
 
