@@ -1,13 +1,13 @@
 /* -*-objc-*- 
    NSArray+GSDoc.h <title></title>
 
-   Copyright (C) 2000-2002 Free Software Foundation, Inc.
+   Copyright (C) 2000-2002,2004,2005 Free Software Foundation, Inc.
 
    Author: Manuel Guesdon <mguesdon@orange-concept.com>
    Date: August 2000
 
-   $Revision: 1.2 $
-   $Date: 2004/04/17 11:30:47 $
+   $Revision: 21645 $
+   $Date: 2005-08-17 10:07:57 +0200 (Mit, 17 Aug 2005) $
 
    <abstract></abstract>
 
@@ -27,11 +27,11 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    </license>
 **/
 
-// $Id: NSArray+GSDoc.h,v 1.2 2004/04/17 11:30:47 ayers Exp $
+// $Id: NSArray+GSDoc.h 21645 2005-08-17 08:07:57Z ayers $
 
 #ifndef __NSArray_GSDoc_h__
 #define __NSArray_GSDoc_h__
